%% Center finder function:
% Input: Solar map and associated data
% Output: Struct that houses the center coordinates of the solar disk within the solar map data

% Sunmap is a program developed by Niko Lavonen, Metsähovi Radio observatory (nlavonen@cc.hut.fi).

function center_info = center_finder(map, opt)

% Required input:
    % map: solar map data matrix
    % opt: struct, containing:
    %   x (coordinate vector, x direction)
    %   y  (coordinate vector, y direction)
    %   mapWidth (not needed anymore)
    %   mapHeight (not needed anymore)
    %   rotDeg (polar rotation angle of the Sun, scalar)
    %   rot_disable (whether or not to rotate the solar plot, boolean)

% Output:
    % center_info.hCenterCoord_orig, Non-rotation compensated solar center coordinates (distance from data matrix center):
    % center_info.vCenterCoord_orig
    % center_info.hCenterCoord_rot, Rotation compensated solar center coordinates (distance from data matrix center):
    % center_info.vCenterCoord_rot
    % center_info.radius, Radius of the solar disk, arc minutes
    % All units in arc minutes (This is defined by opt.x and opt.y, if they are defined with different units, then these coordinates are in those different units too)

    % NOTICE: It's assumed that solar maps are made with no ra or dec
    % offsets.
    % How to distinguish between intentional offsets for achieving proper
    % pointing and just a map that's been intentionally aimed off-center of
    % the Sun?
    
    % Map has already been flipped so that RA-maps and DE-maps are
    % similar at this stage (radec direction-wise)    
    
    % Finding center with sphere_fitter(), which uses sphere fitting to 50 % QSL level as center finding method
    fit_opt.map = map;
    fit_opt.x = opt.x;
    fit_opt.y = opt.y;
    
    [sphere_center sphere_radius sphere_residuals] = sphere_fitter(fit_opt);
    
    hCenterCoord = sphere_center(1);
    vCenterCoord = sphere_center(2);
    
    % Write sphere radius into center_info:
    center_info.radius = sphere_radius;
    
    % Save non-rotation compensated center coordinates:
    center_info.hCenterCoord_orig = hCenterCoord;
    center_info.vCenterCoord_orig = vCenterCoord;

    % Compensate center coordinates for map rotation:
    if(opt.rotate_disable == 0) % map rotated => compensation needed
       center_info.hCenterCoord_rot = (hCenterCoord * cosd(opt.rotDeg)) + (vCenterCoord * sind(opt.rotDeg));
       center_info.vCenterCoord_rot = (vCenterCoord * cosd(opt.rotDeg)) - (hCenterCoord * sind(opt.rotDeg));
    end
        
end % end of center_finder()






% OLD CODE:
    
    
    % Making three vectors for each direction => average center
    % location from those used as true center:
        
    
    
%     if(opt.mapWidth >= 30 && opt.mapHeight >= 30) % The data's size is larger or about the same size as the disk of the sun, data has to be from sun-measure
%                                                                 %==> assume the map is _meant_ as a full disk map and that it _contains_ the full disk
% 
%             % Finding the Sun's center based on maximum and minimum values of the
%             % gradient of vectors taken from near the Sun's proposed center
%             % (The greatest change in the values on the map is on the edge of the Sun)
% 
%             %if(mapWidth > 30 && mapHeight > 30) % Map's size makes a full-disk coverage possible ==> try finding center from data
% 
%             % Calculating positions (indexes to start from) for the vectors:
%             [mapH, mapW] = size(map);
%             hIndex1 = round(mapH/2);
%             hIndex2 = round(hIndex1 - (mapH/8));
%             hIndex3 = round(hIndex1 + (mapH/8));
%             hIndex4 = round(hIndex1 - (mapH/5));
%             hIndex5 = round(hIndex1 + (mapH/5));
%             hVector1 = map(hIndex1,:);
%             hVector2 = map(hIndex2,:);
%             hVector3 = map(hIndex3,:);
%             hVector4 = map(hIndex4,:);
%             hVector5 = map(hIndex5,:);
% 
% 
%             vIndex1 = round(mapW/2);
%             vIndex2 = round(vIndex1 - (mapW/8));
%             vIndex3 = round(vIndex1 + (mapW/8));
%             vIndex4 = round(vIndex1 - (mapW/5));
%             vIndex5 = round(vIndex1 + (mapW/5));
%             vVector1 = map(:,vIndex1);
%             vVector2 = map(:,vIndex2);
%             vVector3 = map(:,vIndex3);
%             vVector4 = map(:,vIndex4);
%             vVector5 = map(:,vIndex5);
% 
%             % Horizontal:
%             hGrad1 = gradient(hVector1); % gradients of vectors
%             hGrad2 = gradient(hVector2);
%             hGrad3 = gradient(hVector3);
%             hGrad4 = gradient(hVector4);
%             hGrad5 = gradient(hVector5);
% 
%             [plaa123, hMaxIndex1] = max(hGrad1); % indexes of max values from gradients (Sun's left edge)
%             [plaa123, hMaxIndex2] = max(hGrad2);
%             [plaa123, hMaxIndex3] = max(hGrad3);
%             [plaa123, hMaxIndex4] = max(hGrad4);
%             [plaa123, hMaxIndex5] = max(hGrad5);
%             [plaa123, hMinIndex1] = min(hGrad1); % indexes of min values from gradients (Sun's right edge)
%             [plaa123, hMinIndex2] = min(hGrad2);
%             [plaa123, hMinIndex3] = min(hGrad3);
%             [plaa123, hMinIndex4] = min(hGrad4);
%             [plaa123, hMinIndex5] = min(hGrad5);
% 
%             hCenter1 = (hMaxIndex1 + hMinIndex1) / 2; % center-index readings for the Sun from the three gradients
%             hCenter2 = (hMaxIndex2 + hMinIndex2) / 2;
%             hCenter3 = (hMaxIndex3 + hMinIndex3) / 2;
%             hCenter4 = (hMaxIndex4 + hMinIndex4) / 2;
%             hCenter5 = (hMaxIndex5 + hMinIndex5) / 2;
% 
%             hCenter = (hCenter1 + hCenter2 + hCenter3 + hCenter4 + hCenter5) / 5; % average of five readings used as horizontal center index
% 
%             % Vertical:
%             vGrad1 = gradient(vVector1); % gradients of vectors
%             vGrad2 = gradient(vVector2);
%             vGrad3 = gradient(vVector3);
%             vGrad4 = gradient(vVector4);
%             vGrad5 = gradient(vVector5);
%             [plaa123, vMaxIndex1] = max(vGrad1); % indexes of max values from gradients (Sun's lower edge)
%             [plaa123, vMaxIndex2] = max(vGrad2);
%             [plaa123, vMaxIndex3] = max(vGrad3);
%             [plaa123, vMaxIndex4] = max(vGrad4);
%             [plaa123, vMaxIndex5] = max(vGrad5);
%             [plaa123, vMinIndex1] = min(vGrad1); % indexes of min values from gradients (Sun's top edge)
%             [plaa123, vMinIndex2] = min(vGrad2);
%             [plaa123, vMinIndex3] = min(vGrad3);
%             [plaa123, vMinIndex4] = min(vGrad4);
%             [plaa123, vMinIndex5] = min(vGrad5);
% 
%             vCenter1 = (vMaxIndex1 + vMinIndex1) / 2;   % center-index readings for the Sun from the three gradients
%             vCenter2 = (vMaxIndex2 + vMinIndex2) / 2;
%             vCenter3 = (vMaxIndex3 + vMinIndex3) / 2;
%             vCenter4 = (vMaxIndex4 + vMinIndex4) / 2;
%             vCenter5 = (vMaxIndex5 + vMinIndex5) / 2;
% 
%             vCenter = (vCenter1 + vCenter2 + vCenter3 + vCenter4 + vCenter5) / 5; % average of three readings used as vertical center index
% 
%             if(abs(vCenter - vCenter1) >= 6)
%                 vCenter = (vCenter2 + vCenter3 +vCenter4 + vCenter5) / 4;
%             elseif(abs(vCenter - vCenter2) >= 6)
%                 vCenter = (vCenter1 + vCenter3 +vCenter4 + vCenter5) / 4;
%             elseif(abs(vCenter - vCenter3) >= 6);
%                 vCenter = (vCenter1 + vCenter2 +vCenter4 + vCenter5) / 4;
%             elseif(abs(vCenter - vCenter4) >= 6)
%                 vCenter = (vCenter1 + vCenter2 +vCenter3 + vCenter5) / 4;
%             elseif(abs(vCenter - vCenter5) >= 6)
%                 vCenter = (vCenter1 + vCenter2 +vCenter3 + vCenter4) / 4;
%             end
% 
%             %else % Map is only partial ==> finding center won't work as with whole maps. Using only target coordinates (pointing error will remain)
% 
%               %  data_hCenter = dataDim.startra + (mapWidth / 2);
%                % data_vCenter = dataDim.startdec + (mapHeight / 2);
% 
%                 %hCenter = data_hCenter
%                 %vCenter = data_vCenter           
% 
%             %end
% 
%              %hCenter        
%              %vCenter
% 
%         % Finding the Sun's center by making a gradient matrix of the map matrix,
%         % taking values from certain range, setting those values 1 and the others 0
%         % The indexes of the 1-elements are used to determine the center
%         % Nope; worse than gradient method
% 
%         %             [Dx, Dy] = gradient(map);
%         %             Dx = abs(Dx);
%         %             Dy = abs(Dy);
%         %             Dx = Dx - min(min(Dx)); % set min at zero
%         %             Dy = Dy - min(min(Dy));
%         %             Dx = Dx * (1 / max(max(Dx))); % Set max at 1
%         %             Dy = Dy * (1 / max(max(Dy)));
%         % 
%         %             Dcombo = Dx + Dy;
%         %             Dcombo = Dcombo - min(min(Dcombo)); % set min at zero
%         %             Dcombo = Dcombo * (1 / max(max(Dcombo))); % Set max at 1
%         % %         [D2x, D2y] = gradient(Dcombo);
%         % %         D2x = abs(D2x);
%         % %         D2y = abs(D2y);
%         % %         D2combo = D2x + D2y;
%         % %         figure, contourf(x_orig, y_orig, Dx, 256)
%         % %         shading flat, axis equal, colormap gray,
%         % %         figure, contourf(x_orig, y_orig, Dy, 256)
%         % %         shading flat, axis equal, colormap gray,
%         %             sizeOfMap = size(map);
%         %             for idx = 1 : sizeOfMap(2)
%         %                 for idx2 = 1 : sizeOfMap(1)
%         %                     if(Dcombo(idx2, idx) >  0.5)
%         %                         map_edge(idx2, idx) = 1;
%         %                     else
%         %                         map_edge(idx2, idx) = 0;
%         %                     end
%         %                 end
%         %             end
%         %          
%         %          [y_ind, x_ind] = find(map_edge);
%         %          hCenter = mean(x_ind)
%         %          vCenter = mean(y_ind)
% 
% 
%         % % Finding the Sun's center based on mean values of indexes of elements
%         % % between 20 % and 90 % QSL
%         % Not working as well as the gradient method
% 
%         %         % Find map values between 30 -70 % qsl, set them to 1, others to 0
%         %         sizeOfMap = size(map);
%         %            
%         %         for idx = 1 : sizeOfMap(2)
%         %             for idx2 = 1 : sizeOfMap(1)
%         %                 if(map(idx2, idx) > 20 && map(idx2, idx) < 90)
%         %                     map_edge(idx2, idx) = 1;
%         %                 else
%         %                     map_edge(idx2, idx) = 0;
%         %                 end
%         %             end
%         %         end
%         %         
%         %         % Calculate averages for indexes in map:
%         %         [y_ind, x_ind] = find(map_edge);
%         %         hCenter = mean(x_ind)
%         %         vCenter = mean(y_ind)
%         %         %mean(y_ind)
%         %         %mean(x_ind)
% 
%         % Finding the Sun's center based on 50 % QSL level
%         % (50 % level of QSL is recorded when the beam is on top of the Sun's
%         % edge):
% 
%         %         % Calculating positions (indexes to start from) for the vectors:
%         %         [mapH, mapW] = size(map);
%         %         hIndex1 = round(mapH/2);
%         %         hIndex2 = round(hIndex1 - (mapH/8));
%         %         hIndex3 = round(hIndex1 + (mapH/8));
%         %         hVector1 = map(hIndex1,:);
%         %         hVector2 = map(hIndex2,:);
%         %         hVector3 = map(hIndex3,:);
%         %         
%         %         vIndex1 = round(mapW/2);
%         %         vIndex2 = round(vIndex1 - (mapW/8));
%         %         vIndex3 = round(vIndex1 + (mapW/8));
%         %         vVector1 = map(:,vIndex1);
%         %         vVector2 = map(:,vIndex2);
%         %         vVector3 = map(:,vIndex3);
%         %         
%         % 
%         %         % Horizontal:
%         %         
%         %         % Loops for finding the horizontal rising edge:
%         %         
%         %         % hCenter1up
%         %         for k = 1 : 1 : length(hVector1)
%         %             if(hVector1(k) > 50)
%         %                 break;
%         %             end
%         %         end
%         %         
%         %         hCenter1up = (k-1) + ( (50 - hVector1(k-1)) / (hVector1(k) - hVector1(k-1)) );
%         %         
%         %         % hCenter2up
%         %         for k = 1 : 1 : length(hVector2)
%         %             if(hVector2(k) > 50)
%         %                 break;
%         %             end
%         %         end
%         %         
%         %         hCenter2up = (k-1) + ( (50 - hVector2(k-1)) / (hVector2(k) - hVector2(k-1)) );
%         %         
%         %         % hCenter3up
%         %         for k = 1 : 1 : length(hVector3)
%         %             if(hVector3(k) > 50)
%         %                 break;
%         %             end
%         %         end        
%         %         
%         %         hCenter3up = (k-1) + ( (50 - hVector3(k-1)) / (hVector3(k) - hVector3(k-1)) );
%         %             
%         %         
%         %         % Loops for finding the horizontal setting edge:
%         %         
%         %         % hCenter1dn
%         %         for k = vIndex1 : 1 : length(hVector1) % k starts from approximate center of map to avoid hitting values < 50 % too early
%         %             if(hVector1(k) < 50)
%         %                 break;
%         %             end
%         %         end
%         %         
%         %         hCenter1dn = (k-1) + ( (50 - hVector1(k-1)) / (hVector1(k) - hVector1(k-1)) );
%         %         
%         %         
%         %         % hCenter2dn
%         %         for k = vIndex1 : 1 : length(hVector2) % k starts from approximate center of map to avoid hitting values < 50 % too early
%         %             if(hVector2(k) < 50)
%         %                 break;
%         %             end
%         %         end
%         %         
%         %         hCenter2dn = (k-1) + ( (50 - hVector2(k-1)) / (hVector2(k) - hVector2(k-1)) );
%         %         
%         %         % hCenter3dn
%         %         for k = vIndex1 : 1 : length(hVector3) % k starts from approximate center of map to avoid hitting values < 50 % too early
%         %             if(hVector3(k) < 50)
%         %                 break;
%         %             end
%         %         end         
%         %         
%         %         hCenter3dn= (k-1) + ( (50 - hVector3(k-1)) / (hVector3(k) - hVector3(k-1)) );
%         %         
%         %         % Estimates for vertical center readings:
%         %         hCenter1 = hCenter1up + (hCenter1dn - hCenter1up) / 2;
%         %         hCenter2 = hCenter2up + (hCenter2dn - hCenter2up) / 2;
%         %         hCenter3 = hCenter3up + (hCenter3dn - hCenter3up) / 2;
%         %         
%         %         hCenter = (hCenter1 + hCenter2 + hCenter3) / 3; % average of three readings used as horizontal center index
%         %         
%         %         
%         %         % Vertical:
%         %         
%         %         % Loops for finding the vertical rising edge:
%         %         
%         %         % vCenter1up
%         %         for k = 1 : 1 : length(vVector1)
%         %             if(vVector1(k) > 50)
%         %                 break;
%         %             end
%         %         end
%         %         
%         %         vCenter1up = (k-1) + ( (50 - vVector1(k-1)) / (vVector1(k) - vVector1(k-1)) )
%         %         
%         %         
%         %         % vCenter2up
%         %         for k = 1 : 1 : length(vVector2)
%         %             if(vVector2(k) > 50)
%         %                 break;
%         %             end
%         %         end
%         %         
%         %         vCenter2up = (k-1) + ( (50 - vVector2(k-1)) / (vVector2(k) - vVector2(k-1)) )
%         %         
%         %         % vCenter3up
%         %         for k = 1 : 1 : length(vVector3)
%         %             if(vVector3(k) > 50)
%         %                 break;
%         %             end
%         %         end        
%         %         
%         %         vCenter3up = (k-1) + ( (50 - vVector3(k-1)) / (vVector3(k) - vVector3(k-1)) )
%         % 
%         %         
%         %         % Loops for finding the horizontal setting edge:
%         %         
%         %         % vCenter1dn
%         %         for k = hIndex1 : 1 : length(vVector1) % k starts from approximate center of map to avoid hitting values < 50 % too early
%         %             if(vVector1(k) < 50)
%         %                 break;
%         %             end
%         %         end
%         %         
%         %         vCenter1dn = (k-1) + ( (50 - vVector1(k-1)) / (vVector1(k) - vVector1(k-1)) );
%         %         
%         %         
%         %         % vCenter2dn
%         %         for k = hIndex1 : 1 : length(vVector2) % k starts from approximate center of map to avoid hitting values < 50 % too early
%         %             if(vVector2(k) < 50)
%         %                 break;
%         %             end
%         %         end
%         %         
%         %         vCenter2dn = (k-1) + ( (50 - vVector2(k-1)) / (vVector2(k) - vVector2(k-1)) );
%         %         
%         %         % vCenter3dn
%         %         for k = hIndex1 : 1 : length(vVector3) % k starts from approximate center of map to avoid hitting values < 50 % too early
%         %             if(vVector3(k) < 50)
%         %                 break;
%         %             end
%         %         end        
%         %         
%         %         vCenter3dn = (k-1) + ( (50 - vVector3(k-1)) / (vVector3(k) - vVector3(k-1)) );
%         %         
%         %         % Estimates for vertical center readings:
%         %         vCenter1 = vCenter1up + (vCenter1dn - vCenter1up) / 2
%         %         vCenter2 = vCenter2up + (vCenter2dn - vCenter2up) / 2
%         %         vCenter3 = vCenter3up + (vCenter3dn - vCenter3up) / 2
%         %         
%         %         vCenter = (vCenter1 + vCenter2 + vCenter3) / 3 % average of three readings used as vertical center index
% 
% 
%             % Converting center readings to plotting coordinates:
%             vCenterMax = opt.y(round(vCenter));
%             vCenterMin = opt.y(round(vCenter - 0.5));
%             vCenterCoord = ( (vCenter - round(vCenter - 0.5)) * ( vCenterMax - vCenterMin)) + vCenterMin;
% 
%             hCenterMax = opt.x(round(hCenter));
%             hCenterMin = opt.x(round(hCenter - 0.5));
%             hCenterCoord = ( (hCenter - round(hCenter - 0.5)) * ( hCenterMax - hCenterMin)) + hCenterMin;
% 
%             % Temp variables for rounded center coords:
%             hCenterCoord_temp = hCenterCoord;
%             vCenterCoord_temp = vCenterCoord;
% 
%             % Checking coords for small values
%             if(abs(hCenterCoord) < 0.0001)
%                 hCenterCoord_temp = 0;
%             end
% 
%             if(abs(vCenterCoord) < 0.0001)
%                 vCenterCoord_temp = 0;
%             end
%                 
%     end

        
        