%% Coord finder function:
% Takes data associated with a solar map, gives a struct that
% houses lat, long and realtive long of a given spot on the sun

% Sunmap is a program developed by Niko Lavonen, Metsähovi Radio observatory (nlavonen@cc.hut.fi).

function coord_info = coord_finder(opt)

% Input: opt-struct containing:
% xcoord: Location in the original data coordinates
% ycoord: Location in the original data coordinates
% hCenterCoord: Distance of solar center from data center, horizontal direction
% vCenterCoord: Distance of solar center from data center, vertical direction
% rotDeg: Rot angle of the Sun
% B0: Latitude of the spot on the Sun that is facing Earth
% L0: Longitude of the spot on the Sun that is facing Earth
% radius: The radius of the solar disk in arc minutes

% Output: struct (coord_info) containing:
% lat
% long
% long_rel

% Center-corrected coords:
xcoord_center = opt.xcoord - opt.hCenterCoord; % Location in the solar-center-corrected data coordinates (arc min)
ycoord_center = opt.ycoord - opt.vCenterCoord; % Location in the solar-center-corrected data coordinates (arc min)
xcoord_center_rot = (xcoord_center * cosd(opt.rotDeg)) + (ycoord_center * sind(opt.rotDeg)); % Location in the rotated solar-center-corrected data coordinates
%(xcoord_center_rot / 7.5) * 24.66666
ycoord_center_rot = (ycoord_center * cosd(opt.rotDeg)) - (xcoord_center * sind(opt.rotDeg)); % Location in the rotated solar-center-corrected data coordinates
%(ycoord_center_rot / 7.5) * 24.66666

% Coords in arc secs:
%fprintf('\nX-coord: %s arc min', num2str(xcoord_center_rot, 4));
%fprintf('\nY-coord: %s arc min\n\n', num2str(ycoord_center_rot, 4));

% Calculate lat and long using inverse orthographic projection:
rho = sqrt(xcoord_center_rot^2 + ycoord_center_rot^2);
if(rho >= opt.radius)
    rho = opt.radius; % If area is outside the solar disk let's assume it's on the edge
end
var_c = asind(rho / opt.radius);
lat = asind(   cosd(var_c) * sind(opt.B0) + (  (ycoord_center_rot * sind(var_c) * cosd(opt.B0)) / rho  )   );
long_rel = atand( (xcoord_center_rot * sind(var_c)) / ((rho * cosd(opt.B0) * cosd(var_c)) - (ycoord_center_rot * sind(opt.B0) * sind(var_c))) );
%spotCoordsLong_rel(i) = long; % Save relative long
%L0;
long = long_rel + opt.L0;
if(long > 360)
    long = long - 360;
elseif(long < 0)
    long = 360 + long;
end

coord_info.lat = lat;
coord_info.long = long;
coord_info.long_rel = long_rel;

end
