%% Coronal hole finder function:
% Takes a solar map and associated data, gives a struct that
% houses data about all separate coronal hole regions on the map that have a
% lower brightness than treshold_value (e.g. 97 % QSL)

% Sunmap is a program developed by Niko Lavonen, Metsähovi Radio observatory (nlavonen@cc.hut.fi).

function areaCoords =  coronal_hole_finder(sunmapHandles, treshold_value, opt)

mainHandles = guidata(sunmapHandles);

% INPUT:
% opt.max_temp: Maximum value of map matrix (This is not needed as input. Can be easily obtained from map_orig)
% opt.map_orig: The solar map matrix in "% of QSL" units
% opt.rotDeg: Polar rotation angle of the Sun
% opt.hCenterCoord: Location of the solar disk's center in the data matrix in the horizontal (x) direction
% opt.vCenterCoord: Location of the solar disk's center in the data matrix in the vertical (y) direction
% opt.x_orig: A vector for positioning the data matrix elements in the correct physical coordinates in the x-direction
% opt.y_orig: A vector for positioning the data matrix elements in the correct physical coordinates in the y-direction
% opt.B0: B0 angle of the Sun
% opt.L0: L0 angle of the Sun

% OUTPUT:
% The areaCoords struct will contain info about one area in one of its
% elements. Each element will contain the fields below
% areaCoords(i).X               % Area max temp coord in arc min from data center
% areaCoords(i).Y               % Area max temp coord in arc min from data center
% areaCoords(i).X_rot           % Area max temp coord in arc min from solar center, solar north up
% areaCoords(i).Y_rot           % Area max temp coord in arc min from solar center, solar north up
% areaCoords(i).Y_area          % Area max temp coord in arc min from solar center, solar north up, rounded to areas -7 ... +7
% areaCoords(i).Long_rel        % Area max temp coord, longitude on solar disk as seen from Earth (degrees from center meridian)
% areaCoords(i).Long            % Area max temp coord, longitude on solar disk in absolute solar coordinates
% areaCoords(i).Lat             % Area max temp coord, latitude on solar disk, B0 angle taken into account
%areaCoords(i).Max_temp  % Area max temp

%treshold_start = 104;
% Setting on what increments the treshold level for finding areas is raised on
% every iteration:
%if(opt.dataUnits == 0) % Units are % of QSL --> 0.5 % increments
treshold_jump = 0.5;
%else % Units are original map units (be that milli Volts for SMART data or backend "pseudo" units for sun-measure data)
%    treshold_jump = round(abs(max(max(opt.map_orig)) - min(min(opt.map_orig))) / 300); % Map levels divided into 300 parts (roughly equivalent to 0.5 % of QSL)
%end
areaCoords = [];

% Stuff that is needed from opt struct:
map_orig = opt.map_orig;
rotDeg = opt.rotDeg;
hCenterCoord = opt.hCenterCoord;
vCenterCoord = opt.vCenterCoord;
x_orig = opt.x_orig;
y_orig = opt.y_orig;
B0 = opt.B0;
L0 = opt.L0;

% If the map is smaller than the disk of the Sun, the function won't do
% anything, because in this case we can't know the exact location of the center of the
% Sun's disk in the data.
if(opt.mapWidth < 30 || opt.mapHeight < 30)
    areaCoords = [];
    return;
end

% Create an inverted version of the solar disk region on the map
% This allows reusing area_sniffer.m code for hole finding
% Set all values outside of a give radius from the center of the map to
% an arbitrary small number:
orig_map_size = size(map_orig);
map_inverted = map_orig;
radius_threshold = 10.5; % Max allowed distance from center of solar disk, arc min
for i = 1 : orig_map_size(1)
    for j = 1 : orig_map_size(2)
    center_dist = sqrt((abs(hCenterCoord-x_orig(j)))^2 + (abs(vCenterCoord-y_orig(i)))^2);
        if(center_dist > radius_threshold)
            map_inverted(i,j) = 120;
        end
    end
end
% Invert remaining values:
map_inverted = -map_inverted + 300; % Invert and lift levels away from negative range
treshold_value = -treshold_value + 300; % Invert and lift away from negative range


% Debug:
%levels = (195): 0.1 :(218);
%figure,
%contourf(x_orig,y_orig,map_inverted, levels),
%caxis([195,218]);
%shading flat,
%axis equal,
%/Debug

% Calculate areas starting from treshold_value, treshold lifted incrementally (treshold_jump) higher and
% higher until the maximum temp of the inverted map is reached. Duplicate areas are
% detected and removed
treshold_end = max(max(map_inverted)); %%opt.max_temp;
for tempTreshold = treshold_value : treshold_jump : treshold_end

    inverted_map_size = size(map_inverted);
    map_holes_only = map_inverted;
    
    %center_dist < cursor_opt_radius


    % Set all other values of the map except the ones > tempTreshold, to
    % zero:
    for i = 1 : inverted_map_size(1)
        for j = 1 : inverted_map_size(2)
            if(map_holes_only(i,j) < tempTreshold)
                map_holes_only(i,j) = 0;
            end
        end
    end
    
    % Create a binary image of the map with ones on the indexes that have
    % value >tempTreshold % QSL and zeros on the others:
    max_locations = imextendedmax(map_holes_only, tempTreshold);
    % cc includes info among others, on the number of separate areas found
    % and the indexes of the elements in the areas:
    cc = bwconncomp(max_locations);
       
    % Go through the areas found and save data from them
    for i = 1 : cc.NumObjects
            cc1 = cc.PixelIdxList(i);
            spot = cc1{1};
            [spot_min, spot_min_idx] = min(map_orig(cc1{1}));
            %spotMaxTemp(i) = spot_max;
            spot_min_idx = spot(spot_min_idx);
            [spot_row, spot_col] = ind2sub(size(map_orig), spot_min_idx); % Row and column of spot in data matrix
            
            xcoord = x_orig(spot_col); % Location in the original data coordinates (arc min)
            ycoord = y_orig(spot_row); % Location in the original data coordinates (arc min)
            
            
            % Testing
%             if(i == 1 && tempTreshold == 104)
%                 i
%                 spot_col
%                 spot_row
%                 xcoord
%                 ycoord
%             end
            % / testing
            
                  
            % Center-corrected coords:
            xcoord_center = xcoord - hCenterCoord; % Location in the solar-center-corrected data coordinates
            ycoord_center = ycoord - vCenterCoord; % Location in the solar-center-corrected data coordinates
            xcoord_center_rot = (xcoord_center * cosd(rotDeg)) + (ycoord_center * sind(rotDeg)); % Location in the rotated solar-center-corrected data coordinates
            %(xcoord_center_rot / 7.5) * 24.66666
            ycoord_center_rot = (ycoord_center * cosd(rotDeg)) - (xcoord_center * sind(rotDeg)); % Location in the rotated solar-center-corrected data coordinates
            %(ycoord_center_rot / 7.5) * 24.66666
           
            % Coords in arc secs:
            %fprintf('\nX-coord: %s arc min', num2str(xcoord_center_rot, 4));
            %fprintf('\nY-coord: %s arc min\n\n', num2str(ycoord_center_rot, 4));

            % Calculate lat and long using inverse orthographic projection:
            rho = sqrt(xcoord_center_rot^2 + ycoord_center_rot^2);
            if(rho >= opt.radius)
                rho = opt.radius; % If area is outside the solar disk let's assume it's on the edge
            end
            var_c = asind(rho / opt.radius);
            lat = asind(   cosd(var_c) * sind(B0) + (  (ycoord_center_rot * sind(var_c) * cosd(B0)) / rho  )   );
            long_rel = atand( (xcoord_center_rot * sind(var_c)) / ((rho * cosd(B0) * cosd(var_c)) - (ycoord_center_rot * sind(B0) * sind(var_c))) );
            %spotCoordsLong_rel(i) = long; % Save relative long
            %L0;
            long = long_rel + L0;
            if(long > 360)
                long = long - 360;
            elseif(long < 0)
                long = 360 + long;
            end
            
            % Convert ycoord into areas ranging from -7...7 (no area labeled 0)
            % Radius of solar disk is assumed to be opt.radius arc min
            if(ycoord_center_rot > 0)
                ycoord_area = round(((ycoord_center_rot / opt.radius) * 7) + 0.5);
            else
                ycoord_area = round(((ycoord_center_rot / opt.radius) * 7) - 0.5);
            end
            
            
            % Adding new values to the areaCoords struct:
            
            % The areaCoords struct is empty:
            if(isempty(areaCoords))

                areaCoords(1).X = xcoord;
                areaCoords(1).Y = ycoord;
                areaCoords(1).X_rot = xcoord_center_rot;
                areaCoords(1).Y_rot = ycoord_center_rot;
                areaCoords(1).Y_area = ycoord_area;
                areaCoords(1).Long_rel = long_rel;
                areaCoords(1).Long = long;
                areaCoords(1).Lat = lat;
                areaCoords(1).Max_temp = spot_min;
                areaCoords(1).tempTreshold = tempTreshold;
                areaCoords(1).spotElements = spot;
                
            % The areaCoords struct is not empty
            else
                
                % Go through the areaCoords struct
                checker = 0;
                for n = 1 : length(areaCoords)

                    % The values in areaCoords struct are not empty ==> they can be checked against
                    % the local coords
                    if(isempty(areaCoords(n).X) == 0 && isempty(areaCoords(n).Y) == 0)

                        % Check for duplicates:
                        % If the x and y coordinates do NOT match ANY coords in areaCoords (and we have first gone through all the coords in the struct),
                        % the area is unique and its data can be added to the areaCoords struct
                        if(xcoord  == areaCoords(n).X && ycoord == areaCoords(n).Y)
                            checker = 1;
                        end
                        
                        if(checker == 0 && n == length(areaCoords))
                            areaCoords(end + 1).X = xcoord;
                            areaCoords(end).Y = ycoord;
                            areaCoords(end).X_rot = xcoord_center_rot;
                            areaCoords(end).Y_rot = ycoord_center_rot;
                            areaCoords(end).Y_area = ycoord_area;
                            areaCoords(end).Long_rel = long_rel;
                            areaCoords(end).Long = long;
                            areaCoords(end).Lat = lat;
                            areaCoords(end).Max_temp = spot_min;
                            areaCoords(end).tempTreshold = tempTreshold;
                            areaCoords(end).spotElements = spot;
                        end % Duplicates check & value writing end

                    end % areaCoords values empty check end

                end % areaCoords struct go-through end
            
            end % areaCoords empty check end
            
    end % areas go-through end
    
  
end % treshold values go-through end
    
% Set preview axes as current axes:
%axes(mainHandles.axes2);

end % area_sniffer function end


