%% Function for generating a RADEC matrix from an arbitrary data-coordinate solar map
% Input:
% - filename: name and path of the map data file
% - startra: minimum delta right ascension from solar center, cosine corrected, arc min
% - endra: maximum delta right ascension from solar center, cosine corrected, arc min
% - startdec: minimum delta declination  from solar center, arc min
% - enddec: maximum delta declination from solar center, arc min
% - sps: samples per sweep in output matrix
% - sweeps: number of sweeps in output matrix
% - data_column: column number for the data in the data file
%
function radec_matrix = generate_radec_matrix(filename, startra, endra, startdec, enddec, sps, sweeps, data_column)

        data_file = dlmread(filename, ' ');
        
        %disp(startra);
        %disp(endra);
        %disp(startdec);
        %disp(enddec);
        
        oversample = 1;
        [xi, yi] = meshgrid(linspace(startra,endra,oversample*sps), linspace(startdec,enddec, oversample*sweeps));
        %disp(min(min(xi)));
        %disp(max(max(xi)));
        %disp(min(min(yi)));
        %disp(max(max(yi)));
        
        map_data = data_file(:,data_column); % NL 2015-06-26

        % Detection of "off scale" saturated backend values, i.e.
	% -32767/-100000. We have to first substitute them with a high
	% and invalid value of 100000 so that the interpolation does
	% not produce values close but not under -30000. After
	% interpolation we can change them back to keep the rest of
	% the program as before.
        off_scale_threshold = -30000;
	map_data(map_data<off_scale_threshold) = 100000;
        
        interp = scatteredInterpolant(data_file(:,7), data_file(:,6), map_data, 'natural', 'none'); %data_file(:,data_column)
        radec_matrix = interp(xi, yi);
        
        %radec_matrix = griddata(data_file(:,7), data_file(:,6), data_file(:,data_column), xi,yi, 'v4'); % fit map data to xi,yi grid. (ra,dec,data_lin)
	% Replace nan's with map minimum:
        radec_matrix(isnan(radec_matrix)) = min(min(map_data));
	% Change off-scale values "back" to -100000:
	radec_matrix(radec_matrix>65535) = -100000;
	
        if(oversample ~= 1)
            radec_matrix = imresize(radec_matrix, [sweeps sps], 'bicubic'); % Resize "oversampled" matrix back to original size
        end
        
end
        
