%% info_reader() function:
% Reads parameters from a sun-measure .suninfo1 file and returns them in a
% struct
% Input: infofile name
% Output: parameters read from the .suninfo1 file in a struct

function infoStruct = info_reader(infofilename)
   
% Check if the info file starts with "VERSION 2"
% If it does, we call "parseinfo()" function which is designed for the new
% info file format
%
% If the file does not start with "VERSION 2", the file is a "legacy" .suninfo1 file
% which is handleled using "parseinfo_legacy()"

[fid, msg] = fopen(infofilename, 'r', 'native');
line = fgets(fid, 4096);
fclose(fid);
if(strcmp(line, sprintf('VERSION 2\n')) == 1)
    % Call parseinfo function that returns a struct containing the stuff
    % from the info file:
    info = parseinfo(infofilename);

else
    % Call parseinfo_lagecy function that returns a struct containing the stuff
    % from the info file (old info file format):
    info = parseinfo_legacy(infofilename);

end

% Write fields into the output struct from the struct returned by parseinfo:
% (at this point the field names are changed to notation used within Sunmap)
infoStruct.n_sweeps = str2double(info.numsweeps);
infoStruct.startdec = str2double(info.startdec);
infoStruct.enddec = str2double(info.enddec);
infoStruct.startra = str2double(info.startra);
infoStruct.endra = str2double(info.endra);
infoStruct.obstime = str2double(info.sessionid);
infoStruct.sweepDir = info.sweepdirection; % 'forward'  ("right" to "left") or 'backward'  ("left" to "right")
infoStruct.sampleDir = info.direction; % 'forward'  ("right" to "left") or 'backward'  ("left" to "right")
infoStruct.raoff = str2double(info.raoff);
infoStruct.decoff = str2double(info.decoff);

if(isfield(info, 'coscorrection')) % Cosine correction field may or may not exist in the info file. If exist --> built in cos correction is done, if not then it's not done
    infoStruct.cosCorrect = str2double(info.coscorrection);
else
    infoStruct.cosCorrect = 0;
end

infoStruct.obs_mode = char(info.mode);
infoStruct.mapType = char(info.maptype);

if(isfield(info, 'observer'))
    infoStruct.observer_name = char(info.observer);
else
    infoStruct.observer_name = 'N/A';
end

if(isfield(info, 'freq'))
    infoStruct.freq = [char(info.freq), ' ', 'GHz'];
else
    infoStruct.freq = 'N/A';
end

% Set the "data_integrated" boolean value to 0
% This was used when it was envisioned that info and data would be in
% the same file:
infoStruct.data_integrated = 0;

% Handle the sweeps section of the struct:
% Check if the info struct contains the sweeps part
if(isfield(info, 'sweep'))
    
    i = 0;
    while 1 % Go through the sweeps until there are no more to be found:
        
        swpStr = ['s', num2str(i)]; % field name: s0, s1, s2, ...
        if(isfield(info.sweep, swpStr))
            
            % The first sweep: take start time:
            if(i == 0)
                obs_start_time_str = info.sweep.s0.start;
            end

            % Take end time for use as obs end time:
            info_aux_str = ['info.sweep.', swpStr];
            % ['isfield(', info_aux_str, ', ''end'')'] % This becomes:
            % isfield(info.sweep.s27, 'end') and so forth
            % This is then evaluated and then we have:
            % if(isfield(info.sweep.s27, 'end')) and so forth
            if(eval(['isfield(', info_aux_str, ', ''end'')']))
                obs_end_time_str = eval(['info.sweep.', swpStr, '.end']);
            else
                obs_end_time_str = '';
            end
            
        elseif(isfield(info.sweep, ['s', num2str(i + 1)]))
            i = i + 1; % The current field isn't found, but the next on is --> jump the loop to the next one
            continue;
        else
            break; % The i'th and the i+1'th fields can't be found --> no more sweeps available (most likely) --> break loop
        end
        
        i = i + 1;
    end
    
    infoStruct.start_time = str2double(obs_start_time_str);
    infoStruct.end_time = str2double(obs_end_time_str);
    
    % Transform start time into a string:
    start_time_total = str2double(obs_start_time_str) / 86400 + datenum('Jan-01-1970 0:0:0');
    start_date_str = datestr(start_time_total, 29);
    start_clock_str = datestr(start_time_total, 13);
%  start_sm_year = datestr(start_time_total, 10); % YYYY
%  start_sm_month = datestr(start_time_total, 5); % MM
%  start_sm_day = datestr(start_time_total, 7); % DD    
%  start_sm_hour = [start_clock_str(1), start_clock_str(2)]; % HH
%  start_sm_minute = [start_clock_str(4), start_clock_str(5)]; % MM
%  start_sm_second = [start_clock_str(7), start_clock_str(8)]; % SS
    
    % Transform end time into a string:
    if(isempty(obs_end_time_str) == 0)
        end_time_total = str2double(obs_end_time_str) / 86400 + datenum('Jan-01-1970 0:0:0');
    %  end_date_str = datestr(end_time_total, 29);
        end_clock_str = datestr(end_time_total, 13);
    %  end_sm_year = datestr(end_time_total, 10); % YYYY
    %  end_sm_month = datestr(end_time_total, 5); % MM
    %  end_sm_day = datestr(end_time_total, 7); % DD    
    %  end_sm_hour = [end_clock_str(1), end_clock_str(2)]; % HH
    %  end_sm_minute = [end_clock_str(4), end_clock_str(5)]; % MM
    %  end_sm_second = [end_clock_str(7), end_clock_str(8)]; % SS

        % Create a string for the duration of the observation:
        % YYYY-MM-DD, HH:MM:SS - HH:MM:SS
        infoStruct.time_str = [start_date_str, ', ', start_clock_str, ' - ', end_clock_str];
    else % End time is not available --> only start time used:
        infoStruct.time_str = [start_date_str, ', ', start_clock_str]; 
    end
    
else % There are no sweeps --> observation will not have an end time and the start time is calculated from the session id:
    
    time_total = str2double(info.sessionid) / 86400 + datenum('Jan-01-1970 0:0:0');
    date_str = datestr(time_total, 29);
    clock_str = datestr(time_total, 13);
    sm_year = datestr(time_total, 10); % YYYY
    sm_month = datestr(time_total, 5); % MM
    sm_day = datestr(time_total, 7); % DD    
    sm_hour = [clock_str(1), clock_str(2)]; % HH
    sm_minute = [clock_str(4), clock_str(5)]; % MM
    sm_second = [clock_str(7), clock_str(8)]; % SS        

    infoStruct.time_str = [date_str, ', ', clock_str]; % Same as smart obstime but lacks end time
    
end
    

end






