%% Function map_handler()

% - Flips a solar map data matrix in the right RA-DEC orientation (north up, west to right)
% - Creates x and y coordinate vectors to be used in plotting
% - Corrects number of sweeps read from info file into the actual number of
%    sweeps in the data in the case that the two values differ

% Input: A struct inside which are the actual input variables
% - map: The map data matrix, matrix
% - mapType: The right ascension/declination sweep type of the map, 'ramap' or 'demap', string
% - sweepDir: The sweep direction of a sun-measure solar map, 'forward' or 'backward', string
% - sampleDir: The samples direction of a sun-measure solar map, 'forward' or 'backward', string 
% - nswp: The number of sweeps in the data, scalar
% - sps: The number of samples per sweep in the data, scalar
% - w: Width of the map data in decimal arc minutes, scalar(?)
% - h: height of the map data matrix, decimal arc minutes, scalar(?)

% Output:
% - mapWidth: Cosine-corrected width of the map data in decimal arc minutes, scalar
% - mapHeight: Height of the map data matrix, decimal arc minutes, scalar
% - nswp: The number of sweeps in the data, possibly corrected for difference in info file and data, scalar
% - x_fov: x-direction field of view to be used in plotting
% - y_fov: y-direction field of view to be used in plotting
% - x: Coordinate vector, x-direction, used in plotting
% - y: Coordinate vector, y-direction, used in plotting


function mhStructOut = map_handler(mhStructIn)    
        
% Define output struct:
mhStructOut = struct;

% Write input variables from input struct:

% Common variables:
map = mhStructIn.map;
dataSource = mhStructIn.dataSource;
mapType = mhStructIn.mapType;
nswp = mhStructIn.nswp;
sps = mhStructIn.sps;
w = mhStructIn.w;
h = mhStructIn.h;
dec = mhStructIn.dec;

%mhStructIn

% sun-measure and map-measure specific variables:
if(strcmp(dataSource, 'sm') == 1 || strcmp(dataSource, 'mm') == 1)
    sweepDir = mhStructIn.sweepDir;
    sampleDir = mhStructIn.sampleDir;
    cosCorrect =  mhStructIn.cosCorrect;
end


% SUN-MEASURE:
if(strcmp(dataSource, 'sm') == 1 || strcmp(dataSource, 'mm') == 1) % sun-measure and map-measure data

    if(strcmp(mapType, 'ramap') == 1 || strcmp(mapType, 'azel') == 1) % sun-measure RA-map or map-measure AzEl-map (converted to radec):

        if(cosCorrect == 0)
            mapWidth = w * cosd(dec);
        else
            mapWidth = w;
        end
        mapHeight = h;

        % Dealing with non-matching info-file -announced number of
        % sweeps and actual number of sweeps in data:
        dataSize = size(map);
        if(nswp ~= dataSize(1))
            mapHeight = mapHeight * (dataSize(1) / nswp);
            nswp = dataSize(1);
        end

        if(strcmp(sweepDir, 'forward') == 1)
            map = fliplr(map);           % Flipping the data into its mirror image if 'sweepDir' is 'forward'
        end

        if(strcmp(sampleDir, 'backward') == 1)
            map = flipud(map);           % Flipping the data upside down if sampleDir is 'backward'
        end

        auxy =  mapHeight / 2;
        vary =  1 / ( ( (nswp - 1) / 2 ) / auxy );
        auxx =  mapWidth / 2;
        varx =  1 / ( ( (sps - 1) / 2) / auxx );

        x_fov = 16.8;
        y_fov = 16.8;

    elseif(strcmp(mapType, 'demap') == 1) % sun-measure DE-map:

        if(cosCorrect == 0)
            mapWidth = w *cosd(dec);
        else
            mapWidth = w;
        end
        mapHeight = h;

        % Dealing with non-matching info-file -announced number of
        % sweeps and actual number of sweeps in data:
        dataSize = size(map);
        if(nswp ~= dataSize(1))
            mapHeight = mapHeight * (dataSize(1) / nswp);
            nswp = dataSize(1);
        end

        map = rot90(map, 3);        % Rotating 3*90 degrees counter clock

        if(strcmp(sampleDir, 'backward') == 1)
            map = fliplr(map);           % Flipping the data into its mirror image if sampleDir is 'backward'
        end

        auxx =  mapWidth / 2;
        varx =  1 / ( ( (nswp - 1) / 2 ) / auxx );
        auxy =  mapHeight / 2;
        vary =  1 / ( ( (sps - 1) / 2) / auxy );

        x_fov = 16.8;
        y_fov = 16.8;

    else % Other map types:
        
        
        
    end
    

% SMART:
elseif(strcmp(dataSource, 'SMART') == 1) %  dataSource is "SMART" --> SMART data

    if(strcmp(mapType, 'demap') == 1) % SMART DE-map:

        map = rot90(map, 3);        % Rotating 3*90 degrees counter clock
        map = fliplr(map);             % Flipping into mirror image (image seems to look right, even though this flip makes no sense)      
        mapWidth = w *cosd(dec);
        mapHeight = h;

        auxy =  mapHeight / 2;
        vary =  1 / ( ( (sps - 1) / 2) / auxy );
        auxx =  mapWidth / 2;
        varx =  1 / ( ( (nswp - 1) / 2 ) / auxx );

        x_fov = 16.8;
        y_fov = 16.8;

    else % SMART RA-map:

        map = fliplr(map);           % Flipping the data into its mirror image.
        mapWidth = w *cosd(dec);
        mapHeight = h;
        
        auxy =  mapHeight / 2;
        vary =  1 / ( ( (nswp - 1) / 2 ) / auxy );
        auxx =  mapWidth / 2;
        varx =  1 / ( ( (sps - 1) / 2) / auxx );

        x_fov = 20;
        y_fov = 16.8;

    end

end


% Create x and y coordinate vectors:    
x = -auxx : varx : auxx;
y = -auxy : vary : auxy;


% TEMP:
% Add radec offsets to the radec coordinate vectors:
if((strcmp(dataSource, 'sm') == 1) || strcmp(dataSource, 'mm') == 1)
    %disp('map_handler: temporary sun measure 2.0 mode on');
    %[x, y] = meshgrid(linspace(mhStructIn.startra,mhStructIn.endra,sps), linspace(60*mhStructIn.startdec,60*mhStructIn.enddec, nswp));
    %x = linspace(mhStructIn.startra,mhStructIn.endra,sps);
    %y = linspace(60*mhStructIn.startdec,60*mhStructIn.enddec, nswp);
    ra_offset = (mhStructIn.startra + mhStructIn.endra) / 2;
    dec_offset = (60*mhStructIn.startdec + 60*mhStructIn.enddec) / 2;
    x = x - ra_offset;
    y = y + dec_offset;
end
% /TEMP


% Write variables to output struct:
mhStructOut.map = map;
mhStructOut.x = x;
mhStructOut.y = y;
mhStructOut.x_fov = x_fov;
mhStructOut.y_fov = y_fov;
mhStructOut.mapWidth = mapWidth;
mhStructOut.mapHeight = mapHeight;
mhStructOut.nswp = nswp;

end % End of function



