%% Polar average temperature function:
% Takes a solar map and associated data, gives an average reading of data
% points in polar regions of the map

function [polar_average, north_average, south_average] = polar_average_temperature(opt)

% INPUT:
% opt.max_temp: Maximum value of map matrix (This is not needed as input. Can be easily obtained from map_orig)
% opt.map_orig: The solar map matrix in "% of QSL" units
% opt.rotDeg: Polar rotation angle of the Sun
% opt.hCenterCoord: Location of the solar disk's center in the data matrix in the horizontal (x) direction
% opt.vCenterCoord: Location of the solar disk's center in the data matrix in the vertical (y) direction
% opt.x_orig: A vector for positioning the data matrix elements in the correct physical coordinates in the x-direction
% opt.y_orig: A vector for positioning the data matrix elements in the correct physical coordinates in the y-direction
% opt.B0: B0 angle of the Sun
% opt.L0: L0 angle of the Sun

% OUTPUT:
% Polar average QSL readings

% Stuff that is needed from opt struct:
map_orig = opt.map_orig; % qsl-units, original sized map
rotDeg = opt.rotDeg;
hCenterCoord = opt.hCenterCoord;
vCenterCoord = opt.vCenterCoord;
x_orig = opt.x_orig;
y_orig = opt.y_orig;
B0 = opt.B0;
L0 = opt.L0;
az = opt.az;
el = opt.el;
% If the map is smaller than the disk of the Sun, the function won't do
% anything, because in this case there's no guarantee that the polar
% regions can be found on the map
if(opt.mapWidth < 30 || opt.mapHeight < 30)
    return;
end

orig_map_size = size(map_orig);

% Empty arrays for point readings:
avg_array = [];
north_array = [];
south_array = [];

% Go through all data points on map. Combine only values with lat > 60
% degrees to the average reading.
for i = 1 : orig_map_size(1) % y-direction
    for j = 1 : orig_map_size(2) % x-direction
        xcoord = x_orig(j); % Location in the original data coordinates (arc min)
        ycoord = y_orig(i); % Location in the original data coordinates (arc min)

        % Testing
%             if(i == 1 && tempTreshold == 104)
%                 i
%                 spot_col
%                 spot_row
%                 xcoord
%                 ycoord
%             end
        % / testing


        % Center-corrected coords:
        xcoord_center = xcoord - hCenterCoord; % Location in the solar-center-corrected data coordinates
        ycoord_center = ycoord - vCenterCoord; % Location in the solar-center-corrected data coordinates
        xcoord_center_rot = (xcoord_center * cosd(rotDeg)) + (ycoord_center * sind(rotDeg)); % Location in the rotated solar-center-corrected data coordinates
        ycoord_center_rot = (ycoord_center * cosd(rotDeg)) - (xcoord_center * sind(rotDeg)); % Location in the rotated solar-center-corrected data coordinates

        % Calculate lat and lon using inverse orthographic projection:
        rho = sqrt(xcoord_center_rot^2 + ycoord_center_rot^2);
        var_c = asind(rho / opt.radius);
        lat = asind(   cosd(var_c) * sind(B0) + (  (ycoord_center_rot * sind(var_c) * cosd(B0)) / rho  )   );
        %long_rel = atand( (xcoord_center_rot * sind(var_c)) / ((rho * cosd(B0) * cosd(var_c)) - (ycoord_center_rot * sind(B0) * sind(var_c))) );

        %long = long_rel + L0;
        %if(long > 360)
        %    long = long - 360;
        %elseif(long < 0)
        %    long = 360 + long;
        %end
        
        if(rho > opt.radius) % point is outside the solar disk --> next
            continue;
        elseif(el < 35) % Elevation smaller than given value --> next
            continue;
        end
        
        if(lat > 60 && lat <= 90) % Point is in northern polar region
            north_array(end + 1) = map_orig(i,j);
        end
        
        if(lat < - 60 && lat >= -90) % Point is in southern polar region
            south_array(end + 1)  = map_orig(i,j);
        end
        
        if(abs(lat) > 60 && abs(lat) <= 90) % Point is in either polar region
            avg_array(end + 1) = map_orig(i,j);
        end
        
    end % Map x-direction data points go through end
end % Map y-direction data points go thourgh end

% Calculate average of points in arrays:
polar_average = mean(avg_array);
north_average = mean(north_array);
south_average = mean(south_array);

end % polar_average_temperature function end


