% This file contains functions that are a part of the solar plotting tool
% "Sunmap"
% The functions in this file are used when generating interactive plots of
% solar observations into a preview window in the main GUI of "Sunmap"

% Sunmap is a program developed by Niko Lavonen, Metsähovi Radio observatory (nlavonen@cc.hut.fi).

function preview(plot_map, sunmap_handles) % <-- sunmap.m: preview(..., handles.figure1)

mainHandles = guidata(sunmap_handles);

% Read data unit selection from the GUI
if(get(mainHandles.radiobutton12, 'Value') == 1)
    dataUnits = 0; % % of QSL (default)
elseif(get(mainHandles.radiobutton13, 'Value') == 1)
    dataUnits = 1; % Original units
else
    dataUnits = 0; % % of QSL (default)
end

% Have the function wait for a time period to allow fast browsing of the
% file list without the program trying to plot all the intermediate files
% Needs a timeout type of a solution to work: "Don't continue if another
% call comes in".
%pause(0.25);

% Reset the guidance text:
set(mainHandles.text65, 'Visible', 'Off');
% Set guidance text size to accommodate less text:
prevtextX = 4;
guiPosition = get(mainHandles.figure1, 'position');
guiHeight = guiPosition(4);
plotboxHeight = guiHeight - 266;
prevtextY = plotboxHeight - 32;
prevtextWidth = 200;
prevtextHeight = 16;
set(mainHandles.text65, 'position', [prevtextX, prevtextY, prevtextWidth, prevtextHeight]);
set(mainHandles.text65, 'String', 'Generating preview...');

%% File path(s) from which the .info and .bin raw data are read

% 'dbase': The starting part of the filepath. You can leave this empty if
% the whole path is in the opt.plottingList-list part of the path.
dbase = '';

%%  Reading plot info from the individual .info-files:

for fnum=1:size(plot_map,1)
    
   % fprintf('Starting with new data:\n');
    
    infofilename = [dbase plot_map{fnum}];
    
    % Check if the info file can be opened:
    finfo = fopen(infofilename,'rt');
    if (finfo < 0) % fopen returns 0 ==> info file can't be opened
        % Write a text in the main GUI preview axes about the problem:
        cla(mainHandles.axes2, 'reset');
        axis off,
        set(mainHandles.text65, 'Visible', 'off');
        text(0.23, 0.5, 'Can''t open info file', 'Color', 'r', 'FontSize', 12);
        % Reset "Data analysis" strings (the ones relating to sphere fitting analysis):
        set(mainHandles.text63, 'String', '');
        set(mainHandles.text64, 'String', '');
        set(mainHandles.text79, 'String', '');
        drawnow update;
        continue % Continue to the next file in the file list
    end
    
    
    % SUN-MEASURE .suninfo1-file:
    % ------------------------------------------------
    if(length(infofilename) - strfind(infofilename, '.suninfo1') == 8) % strfind returns the index of the start of the string found within the string from which we're searching 
        % The above if clause actually checks whether the length of the
        % filename is 7. But it also checks that the filename contains the
        % string '.suninfo1'. The length has to be 7 because the 
   
        % Check if file is empty:
        s_info = dir(infofilename); % s.bytes tells the number of bytes in the data file
        if(exist(infofilename, 'file') == 0 || s_info.bytes < 60)
            return;
        end

        % Call info_reader() that returns a struct containing the
        % neccessary variables in a struct:
        infoStruct = info_reader(infofilename);

        % Write stuff from struct to individual variables:
        n_sweeps = infoStruct.n_sweeps;
        startdec = infoStruct.startdec;
        enddec = infoStruct.enddec;
        startra= infoStruct.startra;
        endra = infoStruct.endra;
        obstime = infoStruct.obstime;
        sweepDir = infoStruct.sweepDir;
        sampleDir = infoStruct.sampleDir;
        cosCorrect = infoStruct.cosCorrect;
        obs_mode = infoStruct.obs_mode;
        mapType = infoStruct.mapType;
        observer_name = infoStruct.observer_name;
        freq = infoStruct.freq;
        data_integrated = infoStruct.data_integrated;
        time_str = infoStruct.time_str;
        
        %data_integrated = 0; % This variable tells the difference between a a file with data incorporated in the same file with the info-stuff and a "standalone" suninfo-file
                                         % The default value 0 is set here.
                                         % If an integrated file is
                                         % encountered later, the value is
                                         % set to 1

        % Variable that tells the observation's source (sun-measure or SMART)
        dataSource = 'sm'; % sun-measure

        % Data height in arc-min
        mapHeight = 60 * abs(enddec - startdec);

        % NON-declination- compensated width in arc-min:
        mapWidth = 60 * abs(endra - startra);

        dataDim.startra = startra;
        dataDim.startdec = startdec;
        dataDim.endra = endra;
        dataDim.enddec = enddec;

        % Calculate declination compensated data width for viewing in main
        % GUI:
        [mapWidth_print mapHeight_print] = gui_map_dimensions(mapWidth, mapHeight, cosCorrect, time_str);
        
%         time_total = obstime / 86400 + datenum('Jan-01-1970 0:0:0');
%         date_str = datestr(time_total, 29);
%         clock_str = datestr(time_total, 13);
%         sm_year = datestr(time_total, 10); % YYYY
%         sm_month = datestr(time_total, 5); % MM
%         sm_day = datestr(time_total, 7); % DD    
%         sm_hour = [clock_str(1), clock_str(2)]; % HH
%         sm_minute = [clock_str(4), clock_str(5)]; % MM
%         sm_second = [clock_str(7), clock_str(8)]; % SS          
%         time_str = [date_str, ', ', clock_str]; % Same as smart obstime but lacks end time

        obstitle = regexprep(infofilename, '.suninfo1', ''); % /home/foo/bar/plaa/plaa/1284623590
        for i = 0:length(obstitle)

            if(strcmp(obstitle(end - i), '/') == 1 || strcmp(obstitle(end - i),'\') == 1)
                break;
            end
            obstitle1(i + 1) = obstitle(end - i);

        end
        obstitle = '';
         for i = 0:(length(obstitle1) - 1)
          obstitle(i + 1) = obstitle1(end - i);     
         end

        % / sun-measure .suninfo file
        % ------------------------------------------------
    
    % map-measure .map/.metadata file
    elseif(strfind(infofilename, '.map'))
        map_meta = get_sunmap_metadata(infofilename);
     
    % SMART .info-file:
    % ------------------------------------------------
    else
        
%         % Check if info file is empty:
%         infoFileName_thing = dir(infofilename); % s.bytes tells the number of bytes in the data file
%         if((exist(infofilename, 'file') == 0 || infoFileName_thing.bytes < 60)) % info file doesn't exist
%             % Clear the GUI offset strings:
%             set(mainHandles.text63, 'String', '');
%             set(mainHandles.text64, 'String', '');
%             
%             set(mainHandles.text40, 'String', 'N/A');
%             set(mainHandles.text41, 'String', 'N/A'); % 'XX GHz'
%             set(mainHandles.text42, 'String', 'N/A');
%             set(mainHandles.text43, 'String', 'N/A');
%             max_string = 'Max. temp: -';
%             set(mainHandles.text44, 'String', max_string);
%             observer = ['Observer: ', 'N/A'];
%             set(mainHandles.text45, 'String', observer);
%             set(mainHandles.text49, 'String', '');
%             
%             cla(mainHandles.axes2, 'reset');
%             axis off,
%             text(0.23, 0.5, 'Info file is empty', 'Color', 'r', 'FontSize', 12);
%             
%             return;
%         end
%         
        dataSource = 'SMART';  % Variable that tells the observation's source (sun-measure or SMART)
        
        info_struct = get_sunmap_metadata_smart(finfo);
        
        % Determine from which row the data starts:
        info_all = textread(infofilename, '%s', 'delimiter', '\n');
        info_numrows = length(info_all);
        info_startrow = info_numrows - info_struct.info_Nswp{1};
        
        % Read time info from info file sweep data:
        try
            info_time = dlmread(infofilename, '', info_startrow, 4);
        catch err
            
            % Recongnise an error related to missing sweep data in info
            % file:
            if (strcmp(err.identifier, 'MATLAB:textscan:BuildFormatString'))
                                
                % Disable plotting buttons, clear preview axes, write data into preview strings and return:
                list_size = size(get(mainHandles.listbox1, 'Value')); % How many list items are selected
                if(list_size(2) == 1) % Only one selected --> disable buttons:
                    set(mainHandles.plot_all_button, 'Enable', 'off');
                    set(mainHandles.plot_single_button, 'Enable', 'off');
                    set(mainHandles.plot_selected, 'Enable', 'off');
                    set(mainHandles.Save, 'Enable', 'off');
                else % More than one selected --> enable buttons:
                    set(mainHandles.plot_all_button, 'Enable', 'on');
                    set(mainHandles.plot_single_button, 'Enable', 'on');
                    set(mainHandles.plot_selected, 'Enable', 'on');
                    set(mainHandles.Save, 'Enable', 'on');
                end
                cla(mainHandles.axes2, 'reset');
                axis off;
                text(0.20, 0.5, 'Missing info file sweep data', 'Color', 'r', 'FontSize', 14);
                set(mainHandles.text65, 'Visible', 'off');
                drawnow update;
                set(mainHandles.uipanel37, 'Title', 'Map preview');
                set(mainHandles.text40, 'String', 'ID: -');
                set(mainHandles.text41, 'String', ['Freq: ', char(info_struct.info_rx{1})]); % ' Freq: XX GHz'
                set(mainHandles.text42, 'String', ['Time :', info_struct.info_date{1}]);
                set(mainHandles.text43, 'String', 'Type: N/A');
                set(mainHandles.text86, 'String', 'Dimensions (RAxDEC): N/A');
                if(dataUnits == 0) % [% of QSL] units
                    max_string = 'Max. temp: N/A';
                else % Original units:
                    max_string = 'Min: N/A, QSL: N/A, Max: N/A';
                end
                set(mainHandles.text44, 'String', max_string);
                obsrv_name = info_struct.info_name{1};
                observer_name = obsrv_name{1};
                observer = ['Observer: ', obsrv_name{1}];
                set(mainHandles.text45, 'String', observer);
                az_el_str = 'Az: -, El: -';
                set(mainHandles.text80, 'String', az_el_str);
                angles_str = 'Rot ang: -, B0: -, L0: -';
                set(mainHandles.text81, 'String', angles_str);
                return;
            else % With other errors just print the error as usual:
                set(mainHandles.text39, 'ForegroundColor', 'r');
                set(mainHandles.text39, 'String', 'Problem reading selected file');
                rethrow(err);
            end

        end
            
        dataDim.foo = 'foo'; % Putting something into dataDim so that there won't be any trouble from it being empty or not existing in the future
    %     if length(info_time) < 29 % Arbitrary number of sweeps:
    %         info_time = dlmread(infofilename, '', 10, 4);
    %     end
        %mapType = 'foobar';
        sweepDir = 'default'; % This has to be set here, because "sweepDir" is used with sun-measure data
        sampleDir = 'default'; % This has to be set here, because "sampleDir" is used with sun-measure data
        cosCorrect = 0; % Default value, cosine correction not done in data --> has to be done in Sunmap
    end
    % ------------------------------------------------
    
    fclose(finfo);
    
%%

    % SUN-MEASURE DATA:
    if(length(infofilename) - strfind(infofilename, '.suninfo1')  == 8)
        
        % Checking whether to look for linear or log scale data:
        
        % Linear scale (Linear checked, Log unchecked):
        if(strcmp(get(mainHandles.lin_scale, 'Checked'), 'on') == 1 && strcmp(get(mainHandles.log_scale, 'Checked'), 'off') == 1)
        
            % We're looking for a .txt file or a .matrix0 file:
            dataFileName = regexprep(infofilename, '.suninfo1', '.txt');
            % Check if .txt datafile exists:
            s = dir(dataFileName); % s.bytes tells the number of bytes in the data file
            if((exist(dataFileName, 'file') == 0 || s.bytes < 60)) % .txt datafile doesn't exist --> trying with .matrix0
                dataFileName = regexprep(infofilename, '.suninfo1', '.matrix0');
            end
            
        % Log scale (Linear unchecked, Log checked):
        elseif(strcmp(get(mainHandles.lin_scale, 'Checked'), 'off') == 1 && strcmp(get(mainHandles.log_scale, 'Checked'), 'on') == 1)
        
            % We're looking for a _log.txt file or a _log.matrix0 file:
            dataFileName = regexprep(infofilename, '.suninfo1', '_log.txt');
            % Check if .txt datafile exists:
            s = dir(dataFileName); % s.bytes tells the number of bytes in the data file
            if((exist(dataFileName, 'file') == 0 || s.bytes < 60)) % .txt datafile doesn't exist --> trying with _log.matrix0
                dataFileName = regexprep(infofilename, '.suninfo1', '_log.matrix0');
            end            
            
        else % If something funny is going on, i.e. both the menu items in the GUI are checked or unchecked at the same time, resort to linear data:
            
            % We're looking for a .txt file or a .matrix0 file:
            dataFileName = regexprep(infofilename, '.suninfo1', '.txt');
            % Check if .txt datafile exists:
            s = dir(dataFileName); % s.bytes tells the number of bytes in the data file
            if((exist(dataFileName, 'file') == 0 || s.bytes < 60)) % .txt datafile doesn't exist --> trying with .matrix0
                dataFileName = regexprep(infofilename, '.suninfo1', '.matrix0');
            end            
            
        end
                
        %dec = sun_declination(time_str);
        sunData = sun_data(time_str);
        dec = sunData.dec;
        
        if(exist('freq', 'var') == 0)
             % If the frequency of the observation hasn't been written into
             % the info-file and thus the variable 'freq' doesn't exist, we make a wild guess...
            freq = '37 GHz';
        end
        obstitle = regexprep(infofilename, '.suninfo1', ''); % /home/foo/bar/plaa/plaa/1284623590

    % .matrix0 datafile is not found:
    % (In the case that we need a separate data file (data_integrated == 0)):
    s = dir(dataFileName); % s.bytes tells the number of bytes in the data file
    if((exist(dataFileName, 'file') == 0 || s.bytes < 60) && data_integrated == 0)
        
        % Write map info on main GUI:
        if(strcmp(obs_mode, 'map') == 1)
            
            % Map:
            set(mainHandles.uipanel37, 'Title', 'Map preview');
            set(mainHandles.text40, 'String', ['ID: ', num2str(obstime)]);
            set(mainHandles.text41, 'String', ['Freq: ', freq]); % ' Freq: XX GHz'
            set(mainHandles.text42, 'String', ['Time: ', time_str]);
            set(mainHandles.text86, 'String', ['Dimensions (RAxDEC): ', mapWidth_print, ''' x ', mapHeight_print, '''']);
            if (strcmp(mapType,'demap') == 1)
                direction = 'DE-map, ';
            elseif(strcmp(mapType,'ramap') == 1)
                direction = 'RA-map, ';
            end
            dir_swp_sps = [direction, num2str(n_sweeps), ' swps, ', 'N/A', ' sampl/swp'];
            set(mainHandles.text43, 'String', ['Type: ', dir_swp_sps]);
            if(dataUnits == 0) % [% of QSL] units
                max_string = 'Max. temp: N/A';
            else % Original units:
                max_string = 'Min: N/A, QSL: N/A, Max: N/A';
            end
            set(mainHandles.text44, 'String', max_string);
            observer = ['Observer: ', observer_name];
            set(mainHandles.text45, 'String', observer);
            set(mainHandles.text49, 'String', '');
            azelTime = regexprep(time_str, '-', '/'); % Az / El texts:
            azelTime = regexprep(azelTime, ',', '');
            [Az El] = SolarAzEl(azelTime, 60.217809153, 24.39311053, 60.28); % Time: YYYY/MM/DD hh:mm:ss, Lat, Long, Height
            az_round = round(Az*100)/100; % 2 decimal accuracy
            el_round = round(El*100)/100; % 2 decimal accuracy
            az_el_str = ['Az: ', num2str(az_round), char(176), ', El: ', num2str(el_round), char(176)];
            set(mainHandles.text80, 'String', az_el_str);
            sunData = sun_data(time_str); % Rot angle, B0, L0:
            rotDeg = sunData.rot;
            B0 = sunData.B0;
            L0 = sunData.L0;
            rotDeg_round = round(rotDeg*100)/100; % 2 decimal accuracy
            B0_round = round(B0*100)/100; % 2 decimal accuracy
            L0_round = round(L0*100)/100; % 2 decimal accuracy
            angles_str = ['Rot ang: ', num2str(rotDeg_round), char(176), ', B0: ', num2str(B0_round), char(176), ', L0: ', num2str(L0_round), char(176)];
            set(mainHandles.text81, 'String', angles_str);
            
        else
            
            % Track:
            cla(mainHandles.axes2, 'reset');
             % Setting map info fields in the sunmap-GUI:
            set(mainHandles.uipanel37, 'Title', 'Track preview');
            set(mainHandles.text40, 'String', ['ID: ', num2str(obstime)]);
            set(mainHandles.text41, 'String', ['Freq: ', freq]); % ' Freq: XX GHz'
            set(mainHandles.text42, 'String', ['Time: ', time_str]);
            set(mainHandles.text86, 'String', 'Dimensions (RAxDEC): -');
            dir_swp_sps = 'Track, N/A samples';
            set(mainHandles.text43, 'String', ['Type: ', dir_swp_sps]);
            set(mainHandles.text44, 'String', '');
            set(mainHandles.text49, 'String', '');
            set(mainHandles.text80, 'String', ''); % No Az/El info for track because of long duration
            set(mainHandles.text81, 'String', '');
            %obsrv_name = info_struct.info_name{1};
            
        end
        
        % Clear and reset the main GUI plotting axes:
        axes(mainHandles.axes2),
        cla(mainHandles.axes2, 'reset');
        axis off,
        
%         %if(ispc == 0)
%             pos = get(mainHandles.axes2, 'Position');
%             pos(1) = 41.4;
%             pos(3) = 42.4;
%             set(mainHandles.axes2, 'Position', pos);    
%         %end

        if(ispc) % Sunmap is run on a PC:
            
            if(exist(dataFileName, 'file') == 0) % Data file exists
                
                % Clear the GUI offset and radius strings:
                set(mainHandles.text63, 'String', '');
                set(mainHandles.text64, 'String', '');
                set(mainHandles.text79, 'String', '');
                
                % Linear scale (Linear checked, Log unchecked):
                if(strcmp(get(mainHandles.lin_scale, 'Checked'), 'on') == 1 && strcmp(get(mainHandles.log_scale, 'Checked'), 'off') == 1)
                    text(0.23, 0.5, 'Data file not found', 'Color', 'r', 'FontSize', 12);
                % Log scale (Linear unchecked, Log checked):
                elseif(strcmp(get(mainHandles.lin_scale, 'Checked'), 'off') == 1 && strcmp(get(mainHandles.log_scale, 'Checked'), 'on') == 1)
                    text(0.1, 0.5, 'Logarithmic data file not found', 'Color', 'r', 'FontSize', 10);
                end
                
            else % sunmap is not run on a PC:
                
                % Clear the GUI offset and radius strings:
                set(mainHandles.text63, 'String', '');
                set(mainHandles.text64, 'String', '');
                set(mainHandles.text79, 'String', '');
                
                % Linear scale (Linear checked, Log unchecked):
                if(strcmp(get(mainHandles.lin_scale, 'Checked'), 'on') == 1 && strcmp(get(mainHandles.log_scale, 'Checked'), 'off') == 1)
                    text(0.23, 0.5, 'Data file is empty', 'Color', 'r', 'FontSize', 12);
                % Log scale (Linear unchecked, Log checked):
                elseif(strcmp(get(mainHandles.lin_scale, 'Checked'), 'off') == 1 && strcmp(get(mainHandles.log_scale, 'Checked'), 'on') == 1)
                    text(0.23, 0.5, 'Logarithmic data file is empty', 'Color', 'r', 'FontSize', 10);
                end
                
            end
            
        else % Sunmap is not run on a PC:
            
             if(exist(dataFileName, 'file') == 0)
                 
                 % Clear the GUI offset and radius strings:
                 set(mainHandles.text63, 'String', '');
                 set(mainHandles.text64, 'String', '');
                 set(mainHandles.text79, 'String', '');
                 
                 % Linear scale (Linear checked, Log unchecked):
                if(strcmp(get(mainHandles.lin_scale, 'Checked'), 'on') == 1 && strcmp(get(mainHandles.log_scale, 'Checked'), 'off') == 1)
                    text(0.23, 0.5, 'Data file not found', 'Color', 'r', 'FontSize', 14);
                % Log scale (Linear unchecked, Log checked):
                elseif(strcmp(get(mainHandles.lin_scale, 'Checked'), 'off') == 1 && strcmp(get(mainHandles.log_scale, 'Checked'), 'on') == 1)                    
                    text(0.13, 0.5, 'Logarithmic data file not found', 'Color', 'r', 'FontSize', 12);
                end
                
             else
                 
                 % Clear the GUI offset and radius strings:
                 set(mainHandles.text63, 'String', '');
                 set(mainHandles.text64, 'String', '');
                 set(mainHandles.text79, 'String', '');
                 
                 % Linear scale (Linear checked, Log unchecked):
                if(strcmp(get(mainHandles.lin_scale, 'Checked'), 'on') == 1 && strcmp(get(mainHandles.log_scale, 'Checked'), 'off') == 1)                 
                    text(0.23, 0.5, 'Data file is empty', 'Color', 'r', 'FontSize', 14);
                % Log scale (Linear unchecked, Log checked):
                elseif(strcmp(get(mainHandles.lin_scale, 'Checked'), 'off') == 1 && strcmp(get(mainHandles.log_scale, 'Checked'), 'on') == 1)
                    text(0.23, 0.5, 'Logarithmic data file is empty', 'Color', 'r', 'FontSize', 12);
                end
             end
             
        end
        
        observer = ['Observer: ', observer_name];
        set(mainHandles.text45, 'String', observer);
        
        list_size = size(get(mainHandles.listbox1, 'Value'));
        if(list_size(2) == 1)
            set(mainHandles.plot_all_button, 'Enable', 'off');
            set(mainHandles.plot_single_button, 'Enable', 'off');
            set(mainHandles.plot_selected, 'Enable', 'off');
            set(mainHandles.Save, 'Enable', 'off');
        else
            set(mainHandles.plot_all_button, 'Enable', 'on');
            set(mainHandles.plot_single_button, 'Enable', 'on');
            set(mainHandles.plot_selected, 'Enable', 'on');
            set(mainHandles.Save, 'Enable', 'on');
        end
        
   
    else  % .txt or matrix0 datafile is found:
        
        % Data is from separate .txt-file:
        if(data_integrated == 0)
            mapdata = dlmread(dataFileName);
        end
        % (If the data is from an integrated file with info and data
        % together, the data has already been read)
        
        map_dimensions = size(mapdata);
        sps = map_dimensions(2); % Samples per sweep.
        
        set(mainHandles.plot_all_button, 'Enable', 'on');
        set(mainHandles.plot_single_button, 'Enable', 'on');
        set(mainHandles.plot_selected, 'Enable', 'on');
        set(mainHandles.Save, 'Enable', 'on');

        % CALLING TrackPlot OR sunPlotter:
        if(strcmp(obs_mode, 'track') == 1 || n_sweeps == 1)
            % Data is a track:
            preview_trackPlot(dec, freq, n_sweeps, sps, mapdata, fnum, time_str, num2str(obstime), mapHeight, mapWidth, sunmap_handles, observer_name, dataSource, dataUnits);
        elseif(strcmp(obs_mode, 'map') == 1)
            % Data is a map:
            sunPlotter(dec, freq, n_sweeps, sps, mapdata, fnum, time_str, num2str(obstime), mapHeight, mapWidth, sunmap_handles, observer_name, mapType, sweepDir, sampleDir, cosCorrect, dataDim, dataSource, dataUnits);
        end
        
    end
 
        
   % END OF SUN-MEASURE DATA
   % ------------------------------------------------

    % ----------------------------
    % map-measure data
    elseif(strfind(infofilename, '.map'))
   
        % Checking whether to look for linear or log scale data:
        
        % Linear scale (Linear checked, Log unchecked):
        if(strcmp(get(mainHandles.lin_scale, 'Checked'), 'on') == 1 && strcmp(get(mainHandles.log_scale, 'Checked'), 'off') == 1)
            data_col_num = 8;
        % Log scale (Linear unchecked, Log checked):
        elseif(strcmp(get(mainHandles.lin_scale, 'Checked'), 'off') == 1 && strcmp(get(mainHandles.log_scale, 'Checked'), 'on') == 1)
            data_col_num = 9;
        else
            data_col_num = 8;
        end
        
        sunData = sun_data(map_meta.time_str);
        dec = sunData.dec;
                        
        % Create (temporary) radec matrix file from arbitrary sweep data
        gen_map_height = 60*abs(map_meta.d_dec_min - map_meta.d_dec_max);
        gen_map_width = 60*abs(map_meta.d_ra_cc_min - map_meta.d_ra_cc_max);
        gen_sweeps = round(gen_map_height / (60*map_meta.sweepwidth)); % generated sweeps
        gen_sps = round(gen_map_width / 0.24); % generated samples per sweep (0.24' ~ 200 sps for 0.8 degree map)
        %gen_sweeps = 500;
        %gen_sps = 500;
        mapType = map_meta.mapType; %'ramap';
        sweepDir = 'forward'; % fix this if something breaks
        sampleDir = 'forward'; % fix this if something breaks
        cosCorrect = 1;
        dataSource = 'sm'; % sun-measure (for now at least)

        % Data height in arc-min
        dataDim.startra = map_meta.d_ra_cc_min; % Check if cc or non cc is needed
        dataDim.endra = map_meta.d_ra_cc_max;
        dataDim.startdec = map_meta.d_dec_min;
        dataDim.enddec = map_meta.d_dec_max;
        
%         data_file = dlmread(infofilename, ' ');
%         %disp(15*map_meta.d_ra_cc_min);
%         %disp(15*map_meta.d_ra_cc_max);
%         %disp(map_meta.d_dec_min);
%         %disp(map_meta.d_dec_max);
%         [xi, yi] = meshgrid(linspace(15*map_meta.d_ra_cc_min,15*map_meta.d_ra_cc_max,2*gen_sps), linspace(map_meta.d_dec_min,map_meta.d_dec_max, 2*gen_sweeps));
%         
%         interp = scatteredInterpolant(15*data_file(:,7), data_file(:,6), data_file(:,data_col_num), 'natural', 'none');
%         zi = interp(xi, yi);
%         
%         %zi = griddata(15*data_file(:,7), data_file(:,6), data_file(:,data_col_num), xi,yi); % fit map data to xi,yi grid. (ra,dec,data_lin)
%         zi(isnan(zi)) = min(min(zi)); % Replace nan's with map minimum
%         zi = imresize(zi, [gen_sweeps gen_sps], 'bilinear');
        radec_matrix = generate_radec_matrix(infofilename, dataDim.startra, dataDim.endra, dataDim.startdec, dataDim.enddec, gen_sps, gen_sweeps, data_col_num);
        
        sunPlotter(dec, map_meta.freq, gen_sweeps, gen_sps, radec_matrix, fnum, map_meta.time_str, map_meta.obstime, gen_map_height, gen_map_width, sunmap_handles, map_meta.observer_name, mapType, sweepDir, sampleDir, cosCorrect, dataDim, dataSource, dataUnits);
        
    % SMART DATA:
    % ------------------------------------------------
    else % This goes on for about 300 lines...
    
        % Write variables into info_fields struct for use with
        % info_reader_smart():
        info_fields.info_df = info_struct.info_df;
        info_fields.info_Tswp = info_struct.info_Tswp;
        info_fields.info_time = info_time;
        info_fields.info_Nswp = info_struct.info_Nswp;
        info_fields.infofilename = infofilename;
        info_fields.info_date = info_struct.info_date;
        info_fields.info_rx = info_struct.info_rx;
        info_fields.info_cent = info_struct.info_cent;
        % Call info_reader_smart():
        infoStructSmart = info_reader_smart(info_fields);
        % Write output from info_reader_smart() to variables:
        mapWidth = infoStructSmart.mapWidth; % Non declination compensated width of the map in decimal arc minutes as read from the SMART info file, scalar
        mapHeight = infoStructSmart.mapHeight; % Height of the map in decimal arc minutes as read from the SMART info file, scalar
        mapType = infoStructSmart.mapType; % The type of the map; right ascension map ('ramap') or declination map ('ramap'), string
        obstitle = infoStructSmart.obstitle; % The name of the map file without the file name extension (e.g. 'sun09080603'), string
        obstime = infoStructSmart.obstime; % The full observation time of the map, start - end, (e.g. '2009-08-06,  6:31:53.660 - 06:40:31.66'), string
        freq= infoStructSmart.freq;  % The frequency of the receiver used, (e.g. '37 GHz', '22 GHz'), string
        dec = infoStructSmart.dec; % The declination of the Sun's center at the time of the observation in decimal degrees, scalar  
      
        %% READING THE SUNMAP RAW-DATA (.BIN-FILE) INTO A MATRIX.

        % Creating the .bin file name from .info file name:
        if(strcmp(infofilename(end-3), 'i') == 1)
            binfilename = strrep(infofilename, '.info', '.bin');
        elseif((strcmp(infofilename(end-3), 'I') == 1))
            binfilename = strrep(infofilename, '.INFO', '.BIN');
        end

        % Calling read_sun_data_ultimate() that reads the .bin-file.
        % (Can be found in read_sun_data_ultimate.m. 'ultimate' comes from the
        % ability to read any kind of an array from a .bin-file)

        obsrv_name = info_struct.info_name{1};
        observer_name = obsrv_name{1};
       
    % If .bin-file is not found:
    if(exist(binfilename, 'file') == 0)
        
        % Reset "Data analysis" strings (the ones relating to sphere fitting analysis):
        set(mainHandles.text63, 'String', '');
        set(mainHandles.text64, 'String', '');
        set(mainHandles.text79, 'String', '');
        
        binFileNotFound = 1;
        
        if(strcmp(obstitle(1), 's') == 1)
            
            % Map:
            set(mainHandles.uipanel37, 'Title', 'Map preview');
            set(mainHandles.text40, 'String', ['ID: ', obstitle]);
            set(mainHandles.text41, 'String', ['Freq: ', freq]); % ' Freq: XX GHz'
            set(mainHandles.text42, 'String', ['Time :', obstime]);
            set(mainHandles.text86, 'String', 'Dimensions (RAxDEC): -:'); % Temporary, data dimensions have to be read from info file first
             if info_struct.info_Nswp{1} == 43
                direction = 'DE-map, ';
             else
                direction = 'RA-map, ';
             end
            dir_swp_sps = [direction, num2str(info_struct.info_Nswp{1}), ' swps, ', num2str(info_struct.info_sps{1}), ' sampl/swp'];
            set(mainHandles.text43, 'String', ['Type: ', dir_swp_sps]);
            if(dataUnits == 0) % [% of QSL] units
                max_string = 'Max. temp: N/A';
            else % Original units:
                max_string = 'Min: N/A, QSL: N/A, Max: N/A';
            end
            set(mainHandles.text44, 'String', max_string);
            obsrv_name = info_struct.info_name{1};
            observer_name = obsrv_name{1};
            observer = ['Observer: ', obsrv_name{1}];
            set(mainHandles.text45, 'String', observer);
            azelTime = regexprep(obstime, '-', '/'); % Az / El texts:
            azelTime = regexprep(azelTime, ',', '');
            [Az El] = SolarAzEl(azelTime, 60.217809153, 24.39311053, 60.28); % Time: YYYY/MM/DD hh:mm:ss, Lat, Long, Height
            az_round = round(Az*100)/100; % 2 decimal accuracy
            el_round = round(El*100)/100; % 2 decimal accuracy
            az_el_str = ['Az: ', num2str(az_round), char(176), ', El: ', num2str(el_round), char(176)];
            set(mainHandles.text80, 'String', az_el_str);
            sunData = sun_data(obstime); % Rot angle, B0, L0:
            rotDeg = sunData.rot;
            B0 = sunData.B0;
            L0 = sunData.L0;
            rotDeg_round = round(rotDeg*100)/100; % 2 decimal accuracy
            B0_round = round(B0*100)/100; % 2 decimal accuracy
            L0_round = round(L0*100)/100; % 2 decimal accuracy
            angles_str = ['Rot ang: ', num2str(rotDeg_round), char(176), ', B0: ', num2str(B0_round), char(176), ', L0: ', num2str(L0_round), char(176)];
            set(mainHandles.text81, 'String', angles_str);
        
        else
            
            % Track:
            cla(mainHandles.axes2, 'reset');
             % Setting map info fields in the sunmap-GUI:
            set(mainHandles.uipanel37, 'Title', 'Track preview');
            set(mainHandles.text40, 'String', ['ID: ', obstitle]);
            set(mainHandles.text41, 'String', ['Freq: ', freq]); % ' Freq: XX GHz'
            set(mainHandles.text42, 'String', ['Time :', obstime]);
            set(mainHandles.text86, 'String', 'Dimensions (RAxDEC): -');
            dir_swp_sps = ['Track, ', num2str(info_struct.info_sps{1}), ' samples'];
            set(mainHandles.text43, 'String', ['Type: ', dir_swp_sps]);
            set(mainHandles.text44, 'String', '');
            set(mainHandles.text80, 'String', '');
            set(mainHandles.text81, 'String', '');
            obsrv_name = info_struct.info_name{1};
            observer_name = obsrv_name{1};
            
        end
        
        axes(mainHandles.axes2),
        cla(mainHandles.axes2, 'reset');
        axis off,
%         %if(ispc == 0)
%             pos = get(mainHandles.axes2, 'Position');
%             pos(1) = 41.4;
%             pos(3) = 42.4;
%             set(mainHandles.axes2, 'Position', pos);    
%         %end
        if(ispc)
            text(0.23, 0.5, '.bin file not found', 'Color', 'r', 'FontSize', 12);
            set(mainHandles.text65, 'Visible', 'off');
            drawnow update;
        else
            text(0.23, 0.5, '.bin file not found', 'Color', 'r', 'FontSize', 14);
            set(mainHandles.text65, 'Visible', 'off');
            drawnow update;
        end
        observer = ['Observer: ', obsrv_name{1}];
        set(mainHandles.text45, 'String', observer);
        
        list_size = size(get(mainHandles.listbox1, 'Value'));
        if(list_size(2) == 1)
            set(mainHandles.plot_all_button, 'Enable', 'off');
            set(mainHandles.plot_single_button, 'Enable', 'off');
            set(mainHandles.plot_selected, 'Enable', 'off');
            set(mainHandles.Save, 'Enable', 'off');
        else
            set(mainHandles.plot_all_button, 'Enable', 'on');
            set(mainHandles.plot_single_button, 'Enable', 'on');
            set(mainHandles.plot_selected, 'Enable', 'on');
            set(mainHandles.Save, 'Enable', 'on');
        end
        
    else % .bin file is found:
        
        binFileNotFound = 0;
        set(mainHandles.plot_all_button, 'Enable', 'on');
        set(mainHandles.plot_single_button, 'Enable', 'on');
        set(mainHandles.plot_selected, 'Enable', 'on');
        set(mainHandles.Save, 'Enable', 'on');
        
        mapdata = read_sun_data_ultimate(binfilename, info_struct.info_Nswp{1}, info_struct.info_sps{1});
        
    end % END OF ELSE-CLAUSE FOR SMART-DATA
    %%
   % ------------------------------------------------
   
   %% Calling trackPlot or sunPlotter.

   
    % SUN-MEASURE DATA: (THIS PART IS REDUNDANT, BECAUSE THE SAME THING IS DONE AT THE END OF THE IF CLAUSE FOR SUN-MEASURE DATA)
   % ------------------------------------------------
    if(strcmp(dataSource, 'sm') == 1)
    
        if(strcmp(obs_type, 'track') == 1)
            % Data is a track:
            preview_trackPlot(dec, freq, n_sweeps, sps, mapdata, fnum, time_str, obstitle, mapHeight, mapWidth, sunmap_handles, observer, dataSource);
        elseif(strcmp(obs_type, 'map') == 1)
            % Data is a map:
            sunPlot(dec, freq, n_sweeps, sps, mapdata, fnum, time_str, obstitle, mapHeight, mapWidth, sunmap_handles, observer, mapType, sweepDir, sampleDir, cosCorrect, dataSource, dataUnits);
            % This part is redundant
        end
    % ------------------------------------------------
        
    % SMART DATA:
    % ------------------------------------------------
    elseif(strcmp(dataSource, 'SMART') == 1 && binFileNotFound == 0)
        
        if(strcmp(obstitle(1), 't') == 1 || strcmp(obstitle(1), 'T') == 1)
            % Data is a track:
            preview_trackPlot(dec, freq, info_struct.info_Nswp{1}, info_struct.info_sps{1}, mapdata, fnum, obstime, obstitle, mapHeight, mapWidth, sunmap_handles, observer_name, dataSource);
        else
            % Data is a map:
            sunPlotter(dec, freq, info_struct.info_Nswp{1}, info_struct.info_sps{1}, mapdata, fnum, obstime, obstitle, mapHeight, mapWidth, sunmap_handles, observer_name, mapType, sweepDir, sampleDir, cosCorrect, dataDim, dataSource, dataUnits);
        end
   
    end
    % ------------------------------------------------        
        
    end
    

end

end % preview() ENDS



%% !!!!----------- preview_trackPlot -function -----------!!!!
function preview_trackPlot(dec, freq, nswp, sps, map, num, time, obstitle, h, w, sunmap_handles, observer_name, dataSource, dataUnits)
    
    mainHandles = guidata(sunmap_handles);    
    axes(mainHandles.axes2),
    cla(mainHandles.axes2, 'reset');
    
     % Setting map info fields in the sunmap-GUI:
    set(mainHandles.uipanel37, 'Title', 'Track preview');
    set(mainHandles.text40, 'String', ['ID: ', obstitle]);
    set(mainHandles.text41, 'String', ['Freq: ', freq]); % ' Freq: XX GHz'
    set(mainHandles.text42, 'String', ['Time: ', time]);
    set(mainHandles.text86, 'String', 'Dimensions (RAxDEC): -');
    dir_swp_sps = ['Track, ', num2str(sps), ' samples'];
    set(mainHandles.text43, 'String', ['Type: ', dir_swp_sps]);
    set(mainHandles.text44, 'String', '');
    observer = ['Observer: ', observer_name];
    set(mainHandles.text45, 'String', observer);
    set(mainHandles.text49, 'String', '');
    % Clear the offset and radius strings:
    set(mainHandles.text63, 'String', '');
    set(mainHandles.text64, 'String', '');
    set(mainHandles.text79, 'String', '');
    set(mainHandles.text80, 'String', '');
    set(mainHandles.text81, 'String', '');
    
%     %if(ispc == 0)
%         pos = get(mainHandles.axes2, 'Position');
%         pos(1) = 48;
%         pos(3) = 30;
%         set(mainHandles.axes2, 'Position', pos);    
    %end
    %set(mainHandles.axes2, 'Color', [0.906, 0.906 0.906]) % Color of gird background
    plot(map, 'Color', 'b');
    axis off,
%     if(get(mainHandles.checkbox28, 'Value') == 1.0) % "Data cursor" checkbox is selected
%     datacursormode on,
%         cursor = datacursormode(gcf);
%         set(cursor, 'SnapToDataVertex','off');
% 
%         set(gca,'Layer','top'),
%     else
%         datacursormode off,
%     end
    %whitebg([1 1 1])
    %set(gca, 'Clipping', 'off');
    
end



%% !!!!----------- sunPlotter -function -----------!!!!
% sunPlotter handles plotting of solar maps on the main GUI preview
function sunPlotter(dec, freq, nswp, sps, map, num, time, obstitle, h, w, sunmap_handles, observer_name, mapType, sweepDir, sampleDir, cosCorrect, dataDim, dataSource, dataUnits)

    warning('off', 'MATLAB:hg:patch:PatchFaceVertexCDataLengthMustEqualVerticesLength');
    %warning off; % This is done to avoid the above type of warning flooding the command line with some maps
    %disp(map);
% destFolder = opt.destFolder;
% beam_size = opt.beam_size;
% solar_north = opt.solar_north;
% grid_toggle = opt.grid_toggle;
% cmap = opt.cmap;
% abs_scale = opt.abs_scale;
% c_min = opt.min;
% c_max = opt.max;


%     sunPlotter() function info:
%
%       PARAMETERS:
%
%           - dec: declination of the Suns center (for calculating round
%             shape for Sun). E.g: 21,73563 (scalar)
%
%           - freq: The frequency of the observation. E.g: '37 GHz'
%             (string)
% 
%           - nswp: Number of sweeps in data. E.g: 29 (scalar)
%
%           - sps: Samples per sweep in data. E.g: 91 (scalar)
%
%           - map: The data matrix of size nswp x sps samples
%
%           - num: A running number that is used to determine how many
%             figures have been printed (scalar)
% 
%           - time: The observation date (start and stop).
%             '2009-07-14,  7:25:27.620 - 07:33:51.62' (string)
% 
%           - obstitle: The plain name of the observation without any
%             filename extention. E.g: 'sun09071506' or ''string)
%
%           - h: The angular height of the data (in arc min), (scalar)
%            
%           - w: The NON DECLINATION-CORRECTED angular width of the data
%             (in arc min) 
%           
%           - sunmap_handles: The handles object of the main GUI (Sunmap)
%mhStructIn
%           - observer_name: The alias or user name of the user who made
%              the observation (string)
%
%           - mapType: A variable that tells two things; if the map is made
%              with SMART or with sun-measure and the sweep direction if
%              the map is made with sun-measure (string)
%               - 'foobar' --> SMART
%               - 'ramap' --> RA-map made with sun-measure
%               - 'demap' --> DE-map made with sun-measure
%
%           - sweepDir: The direction of sweeps made with sun-measure
%               - 'default' --> SMART data, i.e. info is not available, and the value is just used to avoid errors
%               - 'forward' --> sun-measure data, sweep direction from
%                  "right to left" with RA-maps and from "down to up" with
%                  DE-maps
%               - 'backward' --> sun-measure data, sweep direction from
%                  " left to right" with RA-maps and from "up to down" with
%                  DE-maps (this hasn't been checked, so dont' rely blindly on this info!)
%
%
%           - beam_size: The option to enable or disable the printing of
%             beam size on the map
%
%           - solar_north: The option to enable or disable the printing of
%             an arrow illustrating the direction of solar north on the map
%
%           - grid_toggle: The option to enable or disable the printing of
%             a grid on the plot (scalar, 1 = 'on' or 0 = 'off')
%
%           - cmap: The string from the GUI that defines the colormap
%             (string)
%
%           - abs_scale: The option to enable or disable user given
%             absolute levels-values (scalar, 1 = 'on' or 0 = 'off')
%
%           - c_min: User given minimum value from which plotting is
%             started
%
%           - c_max: User given maximum value up to which the plotting is
%             done
%
%           - sunmap_handles: The handles-structure of the sunmap -gui
%             window.
%


    mainHandles = guidata(sunmap_handles);

    set(mainHandles.text65, 'Visible', 'on'); % "Generating preview..."
    drawnow update;
    
    % Recognizing the frequency-string:
    if(isempty(regexpi(freq,'37')) == 0)
        freq = '37 GHz';
    end
    
    if(isempty(regexpi(freq,'87')) == 0)
        freq = '87 GHz';
    end
    
    if(isempty(regexpi(freq,'22')) == 0)
        freq = '22 GHz';
    end
    
    
    %% Map operations:

    % Set "overbright" values to map max value if sun-measure or map-measure data:
    off_scale_values = 0; % Variable to tell if map has off scale values
    off_scale_threshold = -30000;
    if(strcmp(dataSource, 'sm') == 1)
        map_size_temp = size(map);
        for ind_j = 1 : map_size_temp(1)
            for ind_g = 1 : map_size_temp(2)
                if map(ind_j,ind_g) < off_scale_threshold
                     map(ind_j,ind_g) = max(max(map));
                     off_scale_values = 1;
                end
            end
        end
    end
    if(off_scale_values == 1)
        %fprintf(['\nSunmap notice: map ' obstitle ' has off-scale (< ' num2str(off_scale_threshold) ') values.\n']);
        % Set color of max-temp string in UI to red:
        set(mainHandles.text44, 'ForegroundColor', 'red');
        set(mainHandles.text44, 'TooltipString', ['Notice: off-scale values in map data (< ' num2str(off_scale_threshold) '). Overwritten with maximum valid values.']);
    else
        % Set color of max-temp string in UI to black:
        set(mainHandles.text44, 'ForegroundColor', 'black');
        set(mainHandles.text44, 'TooltipString', '');
    end
    
    % Convert log units to lin
   if(strcmp(get(mainHandles.lin_scale, 'Checked'), 'off') == 1 && strcmp(get(mainHandles.log_scale, 'Checked'), 'on') == 1) 
        map = log_to_lin(map);
   end
    
    % User has selected % of QSL units (default)
    % ---------------------------------
    if(dataUnits == 0)
                
        % Temporarily dealing with data that has only one value in the
        % whole data matrix/vector
        if(length(unique(map)) == 1)
            axes(mainHandles.axes2),
            cla(mainHandles.axes2, 'reset');
            set(mainHandles.text65, 'Visible', 'off');
            plot(map'),
            axis off,
            set(mainHandles.uipanel37, 'Title', 'Map preview');
            set(mainHandles.text40, 'String', ['ID: ', obstitle]);
            set(mainHandles.text41, 'String', ['Freq: ', freq]); % ' Freq: XX GHz'
            set(mainHandles.text42, 'String', ['Time: ', time]);
            [mapWidth_print, mapHeight_print] = gui_map_dimensions(w, h, cosCorrect, time);
            set(mainHandles.text86, 'String', ['Dimensions (RAxDEC): ', mapWidth_print, ''' x ', mapHeight_print, '''']);
            dir_swp_sps = [num2str(nswp), ' swps, ', num2str(sps), ' sampl/swp'];
            set(mainHandles.text43, 'String', ['Type: ', dir_swp_sps]);
            set(mainHandles.text44, 'String', 'Max. temp: N/A');
            set(mainHandles.text63, 'String', '');
            set(mainHandles.text64, 'String', '');
            set(mainHandles.text79, 'String', '');
            set(mainHandles.text80, 'String', '');            
            set(mainHandles.text81, 'String', '');
            
            drawnow update;
            return;
        end
        
        % Normalizing data so that minimum is at zero:
        map = map - min(min(map));
        
        % Calling qsl_finder() to get Quiet Sun Level:
        quiet_sun = qsl_finder(map, dataUnits);
        
        % Calling qsl_units_transformer() to transform map units from
        % original to '% of QSL' units:
        map = qsl_units_transformer(map, quiet_sun,freq);

       max_temp = max(max(map));
       min_temp = min(min(map));
       
       quiet_sun_level = 100;
       
    end % End of % of QSL data units
    % ---------------------------------

    
    % User has selected original map data units:
    % ---------------------------------
    if(dataUnits == 1)
        
        max_temp = max(max(map));
        min_temp = min(min(map));
        
        % Calling qsl_finder() to get Quiet Sun Level:
        quiet_sun = qsl_finder(map, dataUnits);
        quiet_sun_level = quiet_sun;
       
       % Calculate "low" value for use with automatic levels
       % For this the % of QSL units are needed:
       % ---------------------------------------------------------------
              
       % Normalizing data so that minimum is at zero:
        map_tmp = map - min(min(map));

        
        % Calling qsl_finder() to get Quiet Sun Level:
        quiet_sun_tmp = qsl_finder(map_tmp, dataUnits);
        
       % Transforming map units from mV to Percent (%)
       % Median = 100%
       % minimum = 0%
       % maximum > 100%
       map_tmp = (map_tmp * 100) / quiet_sun_tmp;
              
       % READJUSTING THE MAP SO THAT THE MINIMUM LEVEL (= SKY TEMP LEVEL) IS
       % SET AT 300 KELVIN, WHICH IS ~3,85 % OF QSL (300 K / 7800 K):
       % 22 GHz: 9000 K (300 / 9000 ~0,033333333)
       % 37 GHz: 7800 K (300 / 7800 ~0,038461538)
       % 86 GHz: 7200 K (300 / 7200 ~0,041666667)

       if(strcmp(freq, '37 GHz') == 1)
           map_tmp = map_tmp + (3.8461538*(100-map_tmp) / 100);
           sky_tmp = 3.8461538;
       elseif(strcmp(freq, '87 GHz') == 1)
           map_tmp = map_tmp + (4.1666667*(100-map_tmp) / 100);
           sky_tmp = 4.1666667;
       elseif(strcmp(freq, '22 GHz') == 1)
           map_tmp = map_tmp + (3.333333*(100-map_tmp) / 100);
           sky_tmp = 3.333333;
       else % If something is wrong, resort to 37 GHz default:
           map_tmp = map_tmp + (3.8461538*(100-map_tmp) / 100); 
           sky_tmp = 3.8461538;                      
       end
       max_temp_tmp = max(max(map_tmp));
       low_tmp = (-0.0287*(max_temp_tmp)^2 + 5.3249*max_temp_tmp -148.5441); % 'low_tmp' is the value form which upwards the map is plotted
       
       % Experimental:
       % Calculating the min value for the "auto exposure" in the original
       % data units (when custom levels aren't selected)
       low_tmp = (((100-sky_tmp)*min(min(map))*quiet_sun_tmp) / ((100+sky_tmp)*quiet_sun_tmp)) - ((sky_tmp*quiet_sun_tmp) / (100+sky_tmp)) + ((low_tmp*quiet_sun_tmp) / 100 + sky_tmp);
       % / Experimental
       
       %low_tmp = min_temp + ((low_tmp - 5) / 100) * abs(max_temp - min_temp);
       % ---------------------------------------------------------------

       
    end % End of original data units
    % --------------------------------- 
    
    %% Map levels-operations:
        
    c_min = str2num(get(mainHandles.edit10, 'String'));
    c_max = str2num(get(mainHandles.edit11, 'String'));
    
    % Protection from invalid custom levels values:
    if(get(mainHandles.checkbox31, 'Value') == 1)
        if(str2num(get(mainHandles.edit10, 'String'))) >= str2num(get(mainHandles.edit11, 'String'))
            message = ['The minimum custom levels value (', get(mainHandles.edit10, 'String'), ') has to be lower than the maximum custom levels value (', get(mainHandles.edit11, 'String'), ').'];
            msgbox(message, 'Warning', 'warn', 'modal');
        return;
        end
        if(c_min >= max_temp)
            message = ['The minumun custom levels value (', num2str(c_min), ') has to be lower than the maximum value (', num2str(max_temp), ') of the selected map.'];
            msgbox(message, 'Warning', 'warn', 'modal');
            return;
        end
        if(isempty(get(mainHandles.edit10, 'String')))
            msgbox('Please insert a number value to the minimum custom levels field', 'Warning', 'warn', 'modal');
            return;
        end
        if(isempty(get(mainHandles.edit11, 'String')))
            msgbox('Please insert a number value to the maximum custom levels field', 'Warning', 'warn', 'modal');
            return;
        end
    end
    
    % Protection from invalid number of levels values:
    %if(get(mainHandles.popupmenu9, 'Value') == 2 || get(mainHandles.popupmenu9, 'Value') == 3)
    
        if(str2num(get(mainHandles.edit12, 'String')) <= 0)
            msgbox('Please insert a positive integer number [1,2,3,...] to the number of levels field', 'Warning', 'warn', 'modal');
            return;
        end
        
        if(str2num(get(mainHandles.edit12, 'String')) > 4096)
            message = ['You have entered a very large (> 4096) "Number of levels" value (', get(mainHandles.edit12, 'String'), ') This can make generating the map preview very slow and is not recommended.'];
            msgbox(message, 'Warning', 'warn', 'modal');
            set(mainHandles.text65, 'Visible', 'off');
            drawnow update;
            return;
        end
        
        if(isempty(get(mainHandles.edit12, 'String')))
            msgbox('Please insert a positive integer number [1,2,3,...] to the number of levels field', 'Warning', 'warn', 'modal');
            return;
        end
    
        cmp = strcmp(num2str(str2num(get(mainHandles.edit12, 'String'))), '');
    
        if(cmp == 1)
            msgbox('Please insert a positive integer number [1,2,3,...] to the number of levels field', 'Warning', 'warn', 'modal');
            return;
        end
        
        % Protection from invalid Custom level separation values:
        if(get(mainHandles.checkbox37, 'Value') == 1.0)
            cmp1 = strcmp(num2str(str2num(get(mainHandles.edit13, 'String'))), '');
            if(cmp1 == 1)
                msgbox('Please insert a positive number to the custom level separation field', 'Warning', 'warn', 'modal');
                return;
            elseif(str2num(get(mainHandles.edit13, 'String')) <= 0)
                msgbox('Please insert a positive number to the custom level separation field', 'Warning', 'warn', 'modal');
                return;
            end
        end
    

   if (get(mainHandles.checkbox31, 'Value') == 1) % Custom levels on
       low = c_min;
       
       if(get(mainHandles.checkbox35, 'Value') == 1) % Custom number of levels is on
           lvls = str2num(get(mainHandles.edit12, 'String')); % number of levels defined by user
           jump = (c_max - c_min) / lvls;
           levels = c_min: jump :c_max;
           
       elseif(get(mainHandles.checkbox37, 'Value') == 1) % Custom level separation is checked
           jump = str2num(get(mainHandles.edit13, 'String')); % Separation in defined by user
           levels = c_min: jump :c_max;
           
       elseif(get(mainHandles.checkbox31, 'Value') == 1 && get(mainHandles.popupmenu6, 'Value') == 10) % colormap is "log"-scale "violetjet" --> more levels required
           lvls = 128;
           jump = (c_max - c_min) / lvls;
           levels = c_min: jump :c_max;
           
       else % colormap is some other than "violetjet"
           lvls = 32;
           jump = (c_max - c_min) / lvls;
           levels = c_min: jump :c_max;
       end
       
   else % Custom levels off --> auto levels
       
        if(dataUnits == 0) % % QSL units selected
            low = (-0.0287*(max_temp)^2 + 5.3249*max_temp -148.5441); % 'low' is the value form which upwards the map is plotted
            % the coefficents have been calculated using some known good
            % low-values for different max_temp-values and by fitting a 2nd
            % degree function to those.
        else
            low = low_tmp; % Original units selected --> low has been calculated earlier
        end
        
        if(get(mainHandles.checkbox35, 'Value') == 1) % Custom number of levels is on
            lvls = str2num(get(mainHandles.edit12, 'String')); % number of levels defined by user
            jump = (max_temp - low) / lvls;
            levels = low: jump : max_temp;
            
        elseif(get(mainHandles.checkbox37, 'Value') == 1) % Custom level separation is checked
           jump = str2num(get(mainHandles.edit13, 'String')); % Separation in % of QSL defined by user           
           levels = low: jump :max_temp;

        else
            lvls = 32;
            jump = (max_temp - low) / lvls;
            levels = low: jump : max_temp;
        end
        
   end
   
%%   % Non linear levels experiment:
%    levels = 0;
%    for i = 1 : 1 : lvls
%        
%        if(i == 1)
%            levels(i) = c_min;
%        else
%            levels(i) = levels(i - 1) + ( (1/jump) * (10 / (i^1.5 / lvls^1.5) ) / 10);
%        end
%       
%    end
%     % normalize:
%    levels = (levels * ( c_max / max(levels) ))
%    min_levels = min(levels);
%    levels = levels + (( (max(levels) - levels) / max(levels) ) * (c_min - min_levels))
%    plot(levels)
%    % END of non linear levels experiment

    %end

    %% Getting rid of SMART "off-sun data" (reflections and what not):
    
    map = map_edges_cleaner(map, mapType, dataSource, nswp, sps);
            
    %% Data size vs. grid, map rotation:

    % Make sure variables are double:
    nswp = double(nswp);
    sps = double(sps);
    w = double(w); 
    h = double(h);
    
    % Write variables into struct for calling map_handler():
    
    % Common variables:
    mhStructIn.map = map;
    mhStructIn.dataSource = dataSource;
    mhStructIn.mapType = mapType;
    mhStructIn.nswp = nswp;
    mhStructIn.sps= sps;
    mhStructIn.w = w;
    mhStructIn.h = h;
    mhStructIn.dec = dec;
    
    % sun-measure specific variables:
    if(strcmp(dataSource, 'sm') == 1)
        mhStructIn.sweepDir =  sweepDir;
        mhStructIn.sampleDir = sampleDir;
        mhStructIn.cosCorrect = cosCorrect;
        % Temp
        mhStructIn.startra = dataDim.startra;
        mhStructIn.endra = dataDim.endra;
        mhStructIn.startdec = dataDim.startdec;
        mhStructIn.enddec = dataDim.enddec;
        % / Temp
    end

    % Call map_handler():
    mhStructOut = map_handler(mhStructIn);

    % Write variables from map_handler() output struct:
    map = mhStructOut.map;
    x = mhStructOut.x;
    y = mhStructOut.y;
    x_fov = mhStructOut.x_fov;
    y_fov = mhStructOut.y_fov;
    mapWidth = mhStructOut.mapWidth;
    mapHeight = mhStructOut.mapHeight;
    nswp = mhStructOut.nswp;

    %% Convert numbers to strings for printing on GUI
    if(dataUnits == 0) % % of QSL units
        max_temp_str = num2str(round(max_temp * 100) / 100);
    else
        max_temp_str = num2str(round(max_temp)); % Lower accuracy is enough, because original units are usually in the order of 10^4, when qsl levels are at 10^2
    end
    qsl_temp_str = num2str(round(quiet_sun_level));
    min_temp_str = num2str(round(min_temp));
        
    %% Setting map info fields in the sunmap-GUI:
    set(mainHandles.uipanel37, 'Title', 'Map preview');
    set(mainHandles.text40, 'String', ['ID: ', obstitle]);
    set(mainHandles.text41, 'String', ['Freq: ', freq]); % ' Freq: XX GHz'
    set(mainHandles.text42, 'String', ['Time: ', time]);
    [mapWidth_print mapHeight_print] = gui_map_dimensions(w, h, cosCorrect, time); % gui_map_dimensions takes original width/heigth that are directly from suninfo file
    set(mainHandles.text86, 'String', ['Dimensions (RAxDEC): ', mapWidth_print, ''' x ', mapHeight_print, '''']);
     if (strcmp(mapType,'demap') == 1)
        direction = 'DE-map, ';
     elseif(strcmp(mapType,'ramap') == 1)
        direction = 'RA-map, ';
     else % Added 2015-04-30 to support e.g. azel maps:
         direction = [mapType, ', '];
     end
    dir_swp_sps = [direction, num2str(nswp), ' swps, ', num2str(sps), ' sampl/swp'];
    set(mainHandles.text43, 'String', ['Type: ', dir_swp_sps]);
    if(dataUnits == 0) % % of QSL units
        max_string = ['Max. temp: ', max_temp_str, ' % of QSL'];
    elseif(dataUnits == 1)
        max_string = ['Min: ', min_temp_str, ', QSL: ', qsl_temp_str, ', Max: ', max_temp_str];
    else
        max_string = ['Max. temp: ', max_temp_str, ' % of QSL'];
    end
    set(mainHandles.text44, 'String', max_string);
    observer = ['Observer: ', observer_name];
    set(mainHandles.text45, 'String', observer);
    azelTime = regexprep(time, '-', '/'); % Az / El texts:
    azelTime = regexprep(azelTime, ',', '');
    [Az El] = SolarAzEl(azelTime, 60.217809153, 24.39311053, 60.28); % Time: YYYY/MM/DD hh:mm:ss, Lat, Long, Height
    az_round = round(Az*100)/100; % 2 decimal accuracy
    el_round = round(El*100)/100; % 2 decimal accuracy
    az_el_str = ['Az: ', num2str(az_round), char(176), ', El: ', num2str(el_round), char(176)];
    set(mainHandles.text80, 'String', az_el_str);
    sunData = sun_data(time); % Rot angle, B0, L0:
    rotDeg = sunData.rot;
    B0 = sunData.B0;
    L0 = sunData.L0;
    rotDeg_round = round(rotDeg*100)/100; % 2 decimal accuracy
    B0_round = round(B0*100)/100; % 2 decimal accuracy
    L0_round = round(L0*100)/100; % 2 decimal accuracy
    angles_str = ['Rot ang: ', num2str(rotDeg_round), char(176), ', B0: ', num2str(B0_round), char(176), ', L0: ', num2str(L0_round), char(176)];
    set(mainHandles.text81, 'String', angles_str); 
    set(mainHandles.text63, 'String', ''); % Emptying ra&dec pointing error strings and the solar radius string:
    set(mainHandles.text64, 'String', '');
    set(mainHandles.text79, 'String', '');
    
    %% Plotting the map, tweaking the settings:
    
    % Dealing temporarily with a single SMART sweep or multiple sweeps from the same coordinates (x or y size is zero/empty):
    if(nswp == 1 && strcmp(dataSource, 'SMART') == 1 || isempty(y) || isempty(x))
        axes(mainHandles.axes2),
        cla(mainHandles.axes2, 'reset');
        axis off,
        set(mainHandles.text65, 'Visible', 'off');
        set(mainHandles.text63, 'String', 'Only one sweep, or multiple sweeps');
        set(mainHandles.text64, 'String', 'from the same coordinates');
        plot(map'),
        axis off,
        drawnow update;
        return;
    end

    % Dealing with erroneous height / width values:
    if(exist('h', 'var') && exist('w', 'var') && strcmp(h, 'foo') == 0 && strcmp(w, 'foo') == 0 && nswp ~= 43)
        if(h == 0 || w == 0)
            cla(mainHandles.axes2, 'reset');
            axis off,
            set(mainHandles.text65, 'Visible', 'off');
            drawnow update;
            return;
        end
    end


    % MAIN PLOTTING:

    % Create a hgtransform object for rotating the plots & grids:
    %tr = hgtransform;
    
    % Resize map, x and y when "Solar grid & sphere fit analysis" is selected
    if(get(mainHandles.checkbox38, 'Value') == 1.0)

        % Making a copy of the original map data and size vectors:
        map_orig = map;
        x_orig = x;
        y_orig = y;
        
            %[foo, xSize] = size(x);
            %[foo, ySize] = size(y);
            %[dataHeight, dataWidth] = size(map);
            
            
            % Resize data matrix if necessary to increase the accuracy of
            % the positioning of the solar grid relative to the plot (in
            % the plotted figure).
            % If the position is not accurate enough, visual inspection of
            % the position would be unreliable
            % Update: It seems the position is now much more accurate than
            % before and the accuracy does not depend on the resolution of
            % the underlying data. If this is the case, the scaling can be
            % removed
            [dataHeight, dataWidth] = size(map);
            scalefactor_x = 1;
            scalefactor_y = 1;
            % By choosing 9 as the scaling factor the original data points
            % can be preserved:
            if(dataHeight <= 80)
                scalefactor_y = 9;
            end
            if(dataWidth <= 80)
                scalefactor_x = 9;
            end

%             map = imresize(map, [scalefactor_y*dataHeight scalefactor_x*dataWidth], 'bilinear');
%             
%             %figure,plot(map_orig(15,:))
%             
%             % Resize x and y vectors:
%             x_span = x(end) - x(1);
%             y_span = y(end) - y(1);
%             x_scaled_spacing = x_span / (scalefactor_x*(dataWidth) - 1);
%             y_scaled_spacing = y_span / (scalefactor_y*(dataHeight) - 1);
%             %figure,plot(y)
%             %y(end)
%             %y(2)-y(1)
%             for idx_x = 1 : scalefactor_x*dataWidth
%                 x(idx_x) = x(1) + x_scaled_spacing*(idx_x-1);
%             end
%             
%             for idx_y = 1 : scalefactor_y*dataHeight
%                 y(idx_y) = y(1) + y_scaled_spacing*(idx_y-1);
%             end
            %figure,plot(y)
            %y(end)
            %y_scaled_spacing
            %scalefactor_y*dataHeight
            %y = imresize(y, [1 scalefactor_y*ySize], 'nearest');
            %x = imresize(x, [1 scalefactor_x*xSize], 'nearest');
            
    else
        
        % Making a copy of the original map data and size vectors anyways:
        map_orig = map;
        x_orig = x;
        y_orig = y;
        
    end
    
    % Create a version of the map that is non-resized and that has QSL
    % units regardless of the user's selection in the GUI
    % This version is used when looking for and marking active areas on the
    % map:    
    
    % User has selected "% of QSL" units for plotting --> the map used for plotting is already in the units needed by the center_finder() function
    if(dataUnits == 0)
        
        map_orig_qsl = map_orig; % This version is needed when printing active area contour on the plot
   
    % User has selected original map data matrix units for plotting --> map variable is on the wrong units for center_finder() --> create map variable with "% of QSL" units
    else

        % Normalizing data so that minimum is at zero:
        map_orig_qsl = map_orig - min(min(map_orig));

        % Calling qsl_finder() to get Quiet Sun Level:
        quiet_sun_orig_qsl = qsl_finder(map_orig_qsl, dataUnits);

        % Calling qsl_units_transformer() to transform map units from
        % original to '% of QSL' units:
        map_orig_qsl = qsl_units_transformer(map_orig_qsl, quiet_sun_orig_qsl,freq);
        
    end
 
    
    % SELECT AXES2 OF SUNMAP WINDOW AS TARGET FOR PLOTTING:
    
    axes(mainHandles.axes2)
    cla(mainHandles.axes2, 'reset');
    sunData = sun_data(time);
    
    rotDeg = sunData.rot;
    B0 = sunData.B0;
    L0 = sunData.L0;
    %solar_radius = sunData.radius;
    
    if(ispc)
        set(gcf, 'Renderer', 'painters'); % forcing renderer in win mahcines to keep maps similar in win & linux
    end
    
    rotate_disable = get(mainHandles.checkbox40, 'Value');
    
    if(rotate_disable == 0) % Map is rotated (rotation not disabled):
               
        % Map type is set to "Contour lines only":
        if(get(mainHandles.popupmenu9, 'Value') == 3.0)
            %warning('off', 'MATLAB:contour:DeprecatedV6Argument');
            warning('off', 'MATLAB:contour3:DeprecatedErrorOutputArgument');
            tr = hgtransform;
            [plaa1, cont2] = contour(x, y, map, levels, '-k', 'Parent', tr);
            
            if(get(mainHandles.checkbox36, 'Value') == 1.0)
                clabelObject = clabel(plaa1, cont2, 'FontSize',8, 'Labelspacing',1000, 'Parent', tr); % "Warning: Text handle output is not supported for managed labels."
                set(clabelObject,'BackgroundColor',[1 1 .6],...
                'Edgecolor',[.7 .7 .7], 'Rotation', 0);
                %rotate(clabelObject,[0 0 1],-rotDeg, [0 0 0])
            end
        
        % Map type is set to "Filled contours, no contour lines (Default)" or "Filled contours with contour lines":
        else
            tr = hgtransform;
            [plaa1, cont2] = contourf(x,y,map,levels, 'Parent', tr);
        end

        % TEMPORARY TEST OF DATA CURSOR:
        
%         %if(get(mainHandles.checkbox28, 'Value') == 1.0) % "Data cursor" checkbox is selected
%             datacursormode on,
%             cursor = datacursormode(gcf);
%             set(cursor, 'SnapToDataVertex','off');
%             getCoordinates = 1;
%             if(getCoordinates == 1)
%                 cursor_opt_L0 = L0;
%                 cursor_opt_B0 = B0;
%                 cursor_opt_rotDeg = rotDeg;
%                 cursor_opt_radius = sunData.radius;
%                 cursor_opt_rotate_disable = rotate_disable;
%                 cursor_opt_x = x;
%                 cursor_opt_y = y;
%                 cursor_opt_mapWidth = mapWidth;
%                 cursor_opt_mapHeight = mapHeight;
%                 set(cursor,'UpdateFcn',{@data_cursor_mod}); % calling the function that modifies the output of the data cursor
%             end
%             set(gca,'Layer','top'),
%         %end
        % / TEMPORARY TEST OF DATA CURSOR
        
        
    else % Rotation is disabled:
        
        % Map type is set to "Contour lines only":
        if(get(mainHandles.popupmenu9, 'Value') == 3.0)
            %warning('off', 'MATLAB:contour:DeprecatedV6Argument');
            warning('off', 'MATLAB:contour3:DeprecatedErrorOutputArgument');
            tr = hgtransform;
            [plaa1, cont2] = contour(x, y, map, levels, '-k', 'Parent', tr);
            
            
            if(get(mainHandles.checkbox36, 'Value') == 1.0)
                clabelObject = clabel(plaa1, cont2, 'FontSize',9, 'Labelspacing',1000);
                set(clabelObject,'BackgroundColor',[1 1 .6],...
                'Edgecolor',[.7 .7 .7], 'Rotation', 0);                 
            end
            
        % Map type is set to "Filled contours, no contour lines (Default)" or "Filled contours with contour lines":
        else
            tr = hgtransform;
            [plaa1, cont2] = contourf(x,y,map,levels, 'Parent', tr);
        end
        
        if(get(mainHandles.checkbox28, 'Value') == 1.0) % "Data cursor" checkbox is selected
            datacursormode on,
            cursor = datacursormode(gcf);
            set(cursor, 'SnapToDataVertex','off');
            getCoordinates = 1;
            if(getCoordinates == 1)
                cursor_opt_L0 = L0;
                cursor_opt_B0 = B0;
                cursor_opt_rotDeg = rotDeg;
                cursor_opt_radius = sunData.radius;
                cursor_opt_rotate_disable = rotate_disable;
                cursor_opt_x = x;
                cursor_opt_y = y;
                cursor_opt_mapWidth = mapWidth;
                cursor_opt_mapHeight = mapHeight;
                set(cursor,'UpdateFcn',{@data_cursor_mod}); % calling the function that modifies the output of the data cursor
            end
            set(gca,'Layer','top'),
        end
    end
               
    % Contour line annotations:
    %if(get(mainHandles.popupmenu9, 'Value') == 3.0 && get(mainHandles.checkbox36, 'Value') == 1.0)
        % [C, h] = contour(x, y, map, levels, '-k'),
        %set(cont2,'ShowText','on','TextStep', 5),
       % clabelObject = clabel(plaa1, cont2, 'FontSize',7, 'Labelspacing',1000);
        %size(cont2)
    %end
    
    if(rotDeg ~= -300 && rotate_disable == 0)
         % Rotating solar north straight up:
        warning('off', 'MATLAB:contourf:DeprecatedV6OutputArgument');
        warning('off', 'MATLAB:contour3:DeprecatedErrorOutputArgument');
        set(tr,'Matrix',makehgtform('zrotate',-rotDeg*pi/180))
        datacursormode off,
        set(gca,'Layer','top'),
        if(exist('nswp_mismatch', 'var') == 0 || nswp_mismatch == 0)
            set(mainHandles.text49, 'String', '');
        end
    elseif(rotDeg == -300)
        set(mainHandles.text49, 'ForegroundColor', 'red');
        set(mainHandles.text49, 'String', 'Rot angle not found!');
    end
    

     % Map type is set to "Filled contours, no contour lines (Default)":
    if(get(mainHandles.popupmenu9, 'Value') == 1.0)
        set(cont2,'LineStyle','none');
        %shading flat,
    end
    % QSL contour drawn on map:
    if(get(mainHandles.checkbox27, 'Value') == 1)
        hold on,
        %warning('off', 'MATLAB:contour:DeprecatedV6Argument');
        warning('off', 'MATLAB:contour3:DeprecatedErrorOutputArgument');
        if(get(mainHandles.popupmenu9, 'Value') == 1.0) % "Filled contours, no contour lines"
            [plaa3, qslCont2] = contour(x, y, map, [quiet_sun_level quiet_sun_level], '-k', 'Parent', tr);
        elseif(get(mainHandles.popupmenu9, 'Value') == 2.0) % "Filled contours with contour lines"
            [plaa3, qslCont2] = contour(x, y, map, [quiet_sun_level quiet_sun_level], '-m', 'Parent', tr);
        else % "Contour lines only"
            [plaa3, qslCont2] = contour(x, y, map, [quiet_sun_level quiet_sun_level], '-m', 'Parent', tr);
        end
    end
    % ------------    
    
    % COLORMAP:
    cmap = get_colormap(mainHandles.figure1);
    % Set colormap:
    set_colormap(cmap);
       
    if(get(mainHandles.checkbox31, 'Value') == 1)
        caxis([c_min, c_max]);
    end
    
    if(mainHandles.fig_grid == 1)
       grid on,
    end
    %colormap(gray),
    set(gca,'Layer','top'),
       
    % Reading x_fov and y_fov from hidden text fields in the gui:
    x_fov = str2num(get(mainHandles.text60, 'String'));
    y_fov = str2num(get(mainHandles.text57, 'String'));
    axis([-x_fov x_fov -y_fov y_fov]),
%     readX = mainHandles.xlims;
%     readY = mainHandles.ylims;
%     readX
%     readY
%     axis([mainHandles.xlims mainHandles.ylims]),
%     xlim
%     ylim
    axis equal,
    %set(gca, 'Color', [0.906,0.906,0.906]) % Color of gird background
    %set(gca, 'Xcolor', [0.3 0.3 0.3], 'Ycolor', [0.3 0.3 0.3]);
    %set(gca,'Clipping','on')
    axis off,
    %grid on,
    
    % Testing colorbar
    % Separate axes:
    %axes(mainHandles.axes4),
    %cla(mainHandles.axes4),
    %axis off,
    %cb = colorbar;
    %set(cb, 'location', 'South', 'Position', [0.025 0.025 0.95 0.04])
    % / Testing colorbar
    
    %% Finding the center of the solar disk, drawing solar coordinate grid on the map, printing radec pointing errors in GUI, resizing map matrix 
    
    % NOTICE (OLD): It's assumed that solar maps are made with no ra or dec
    % offsets. Meaning that the pointing errors are given assuming that the
    % maps have been made in the intention of pointing the telescope in the
    % middle of the solar disk.
    % (How to distinguish between intentional offsets for achieving proper
    % pointing and just a map that's been intentionally aimed off-center of
    % the Sun?)

    if(strcmp(dataSource, 'SMART') == 1 && ((mapWidth >= 30 || mapHeight >= 30) && (mapWidth >= 15 && mapHeight >= 15)) == 0)

        if(get(mainHandles.checkbox38, 'Value') == 1.0 && rotDeg ~= -300)
            % Do nothing, if data is from SMART and the map is partial (size in either direction is smaller than 15 arc min)
            % Reset the text fields in main gui that are used with partial sm
            % maps
            set(mainHandles.text58, 'String', '');
            set(mainHandles.text59, 'String', '');
            % Notice user about the map being too small for sphere fitting and
            % consequent data analysis:
            set(mainHandles.text63, 'String', 'Map too small for circle fitting analysis');
            set(mainHandles.text64, 'String', '');
            set(mainHandles.text79, 'String', '');
        else
            % Clear the offset and radius strings:
            set(mainHandles.text63, 'String', '');
            set(mainHandles.text64, 'String', '');
            set(mainHandles.text79, 'String', '');
        end

    elseif((get(mainHandles.checkbox38, 'Value') == 1.0 || strcmp(get(mainHandles.Untitled_121, 'Checked'), 'on') == 1) && rotDeg ~= -300)
        % (Solar grid & sphere fit analysis on OR Solar disk centering is
        % selected) AND rot angle has been found
        
        % Map has already been flipped so that RA-maps and DE-maps are
        % similar at this stage (radec direction-wise)        
        
        % Data size is larger or about the same size as the disk of the sun in one direction and at least  a half a disk in the other direction.
        if((mapWidth >= 30 || mapHeight >= 30) && (mapWidth >= 15 && mapHeight >= 15))

            % Reset invisible auxiliary text fields in main gui that are used with partial sm
            % maps
            set(mainHandles.text58, 'String', '');
            set(mainHandles.text59, 'String', '');

            % Write neccessary variables in opt-struct:
            opt.mapWidth = mapWidth;
            opt.mapHeight = mapHeight;
            opt.rotDeg = rotDeg;
            opt.rotate_disable = rotate_disable;

            % Calling center_finder():
            % User has selected "% of QSL" units for plotting --> the map used for plotting is already in the units needed by the center_finder() function
            if(dataUnits == 0)
            
                opt.x = x;
                opt.y = y;
                center_info = center_finder(map, opt);
            
            % User has selected original map data matrix units for plotting --> map variable is on the wrong units for center_finder() --> create map variable with "% of QSL" units
            else
                
                % Normalizing data so that minimum is at zero:
                map_qsl = map - min(min(map));
                
                % Calling qsl_finder() to get Quiet Sun Level:
                quiet_sun_qsl = qsl_finder(map_qsl, dataUnits);

                % Calling qsl_units_transformer() to transform map units from
                % original to '% of QSL' units:
                map_qsl = qsl_units_transformer(map_qsl, quiet_sun_qsl,freq);

                opt.x = x;
                opt.y = y;
                center_info = center_finder(map_qsl, opt);
                
            end
                        
            hCenterCoord = center_info.hCenterCoord_orig;
            vCenterCoord = center_info.vCenterCoord_orig;

            % Temp variables for rounded center coords:
            hCenterCoord_temp = hCenterCoord;
            vCenterCoord_temp = vCenterCoord;

            if(get(mainHandles.checkbox38, 'Value') == 1.0)
                % The strings in the GUI are only written if the user has
                % selected Solar grid & sphere fitting analysis
                % They are not written if the user has selected only the
                % solar disk centering, because then the values can't be
                % checked visually by seeing the location of the solar grid
                                
                % Checking coords for small values
                if(abs(hCenterCoord) < 0.0001)
                    hCenterCoord_temp = 0;
                end

                if(abs(vCenterCoord) < 0.0001)
                    vCenterCoord_temp = 0;
                end

                % Round the center coordinate printing values (_temp)
                vCenterCoord_temp = round(10000 * vCenterCoord_temp) / 10000;
                hCenterCoord_temp = round(10000 * hCenterCoord_temp) / 10000;
                
                % Write offset information and measured solar radius to preview window:
                set(mainHandles.text63, 'String', ['RA-pointing error: ', num2str(hCenterCoord_temp), '''']);
                set(mainHandles.text64, 'String', ['Dec-pointing error: ', num2str(vCenterCoord_temp), '''']);
                radius_round = round(100 * center_info.radius) / 100; % Round the radius reading to 2 decimals accuracy
                set(mainHandles.text79, 'String', ['Measured solar radius: ', num2str(radius_round), '''']);
            end

        elseif(strcmp(dataSource, 'sm') == 1) % Map smaller than 15 arc min in either direction and sun-measure data ==> use map data center coordinates to illustrate solar center
            
            % Use raoff & decoff to determine wher the partial map has been
            % taken from in solar centered coordinates (radec direction):
            % Units are in degrees, so they have to be converted to arc minutes

            %dataDim.startra
            %dataDim.startdec
            %dataDim.endra
            %dataDim.enddec
            %sweepDir{1}

            % Since center_finder() isn't called, the solar radius has to
            % be "made up":
            %center_info.radius = 16.3;
            
            %if(strcmp(sweepDir{1}, 'forward') == 1) % sweepDir is 'forward' (from right to left, in radec's that's forward)

            hCenterCoord = 60 * (dataDim.startra + dataDim.endra) / 2; % Minus sign because RA grows from right to left
            vCenterCoord = -60 * (dataDim.startdec + dataDim.enddec) / 2;

            % Save center coordinates to two textfields in the main gui:
            set(mainHandles.text58, 'String', num2str(hCenterCoord));
            set(mainHandles.text59, 'String', num2str(vCenterCoord));

            % Clear the offset and radius strings:
            % Notice user about the map being too small for sphere fitting and
            % consequent data analysis:
            set(mainHandles.text63, 'String', 'Map too small for circle fitting analysis');
            set(mainHandles.text64, 'String', 'Grid centered using info file offsets');
            set(mainHandles.text79, 'String', '');
        
        elseif(strcmp(dataSource, 'SMART') == 1) % Map smaller than 15 arc min in either direction and SMART data ==> no fitting or grid, inform user with string:
            
            % Notice user about the map being too small for sphere fitting and
            % consequent data analysis:
            set(mainHandles.text63, 'String', 'Map too small for circle fitting analysis');
            set(mainHandles.text64, 'String', '');
            set(mainHandles.text79, 'String', '');
            
        end

        % Write offset info into a file (Notice "checker"):
        % TODO: Take into account AzEl offsets & RaDec "offsets" (in startra & endra)
        %             Convert RaDec into AzEl
%         checker = 1; % 1: not run, 0: run
% 
%         if(checker == 0)
%             azelTime = regexprep(time, '-', '/');
%             azelTime = regexprep(azelTime, ',', '');
%             [Az El] = SolarAzEl(azelTime, 60.217809153, 24.39311053, 60.28); % Time: YYYY/MM/DD hh:mm:ss, Lat, Long, Height
% %             Raoffsfile_el = fopen('plots/2011-04-13/raOffs_el.txt', 'a');
% %             Decoffsfile_el = fopen('plots/2011-04-13/decOffs_el.txt', 'a');
% %             Raoffsfile_az = fopen('plots/2011-04-13/raOffs_az.txt', 'a');
% %             Decoffsfile_az = fopen('plots/2011-04-13/decOffs_az.txt', 'a');
%             hCenterCoord_str = num2str(hCenterCoord);
%             vCenterCoord_str = num2str(vCenterCoord);
% %             fprintf(sfile_el, '%s %s\n', num2str(El), hCenterCoord_str);
% %             fprintf(Decoffsfile_el, '%s %s\n', num2str(El), vCenterCoord_str);
% %             fprintf(Raoffsfile_az, '%s %s\n', num2str(Az), hCenterCoord_str);
% %             fprintf(Decoffsfile_az, '%s %s\n', num2str(Az), vCenterCoord_str);
% %             fclose(Decoffsfile_el);
% %             fclose(Raoffsfile_el);
% %             fclose(Decoffsfile_az);
% %             fclose(Raoffsfile_az);
%             RaDecoffsfile = fopen('/home/nlavonen/aurinkodata/offsets/sun_offsets.txt', 'a');
%             tab = '\t';
%             s = dir('/home/nlavonen/aurinkodata/offsets/sun_offsets.txt'); % s.bytes tells the number of bytes in the data file
%             if(s.bytes < 5)
%                 fprintf(RaDecoffsfile, [sprintf('%-20s','DecOff_error') tab sprintf('%-20s','RaOff_error')  tab sprintf('%-20s','Az') tab sprintf('%-20s','El') '\n']);
%             end
%             hCenterCoord_str_print = sprintf('%-20s',hCenterCoord_str);
%             vCenterCoord_str_print = sprintf('%-20s',vCenterCoord_str);
%             Az_print = sprintf('%-20s',num2str(Az));
%             El_print = num2str(El); % = sprintf('%-20s',num2str(Az));
%             fprintf(RaDecoffsfile, [hCenterCoord_str_print tab vCenterCoord_str_print tab Az_print tab El_print, '\n']);
% 
%             fclose(RaDecoffsfile);
% 
% 
%         end

        % Save non-rotation compensated center coordinates:
        hCenterCoord_orig = hCenterCoord;
        vCenterCoord_orig = vCenterCoord;
                
        % Compensate center coordinates for map rotation:
        if(rotate_disable == 0) % map rotated => compensation needed
            hCenterCoord = (hCenterCoord_orig * cosd(rotDeg)) + (vCenterCoord_orig * sind(rotDeg));
            vCenterCoord = (vCenterCoord_orig * cosd(rotDeg)) - (hCenterCoord_orig * sind(rotDeg));
        end
              
        % Printing the solar grid when the user has selected "Solar
        % grid & sphere fitting analysis":  
        if(get(mainHandles.checkbox38, 'Value') == 1.0)        
            
            hold on,

             % Drawing coordinate grid on map:
            if(rotate_disable == 0) % Map rotated:
                ax = axesm('ortho','Origin',[B0 L0 0], 'FalseEasting', hCenterCoord, 'FalseNorthing', vCenterCoord, 'ScaleFactor', sunData.radius, 'MLineLocation', 20, 'PLineLocation', 20, 'GColor', [0 0 0]);
            else % Map not rotated => grid must be rotated instead:
                ax = axesm('ortho','Origin',[B0 L0 -rotDeg], 'FalseEasting', hCenterCoord, 'FalseNorthing', vCenterCoord, 'ScaleFactor', sunData.radius, 'MLineLocation', 20, 'PLineLocation', 20, 'GColor', [0 0 0]);
            end
            
            if(L0 <= 180)
                PLabel_meridian = L0 + 20;
            else
                PLabel_meridian = L0 + 20 - 360;
            end

            setm(ax,'MLabelParallel',0, 'GAltitude', 0, 'MLabelLocation', 40, 'PLabelLocation', 40, 'PLabelMeridian', PLabel_meridian); % GAltitude determines the z-axis location of the grid. Seeting it to 0 lets the data cursor be above or "in front of" the grid  % ,'PLabelMeridian',0
            %set(ax,'Position', [2 4 303 303]); % Using "set", not "setm" makes it possible to modify the current axes (which is now a map axes)
            % This is the same as using mainHandles.axes2 as the axes:
            %set(mainHandles.axes2, 'Parent', mainHandles.uipanel37);
            mlabel on,
            plabel on,
            gridm on,
            axis off,
            mlabelzero22pi; % Display longitude labels in the range of 0 to 360 degrees
            
            % Drawing circle on figure:
            rectangle('Curvature',[1 1], 'Position', [hCenterCoord-sunData.radius vCenterCoord-sunData.radius sunData.radius*2 sunData.radius*2], 'Linewidth', 0.5, 'edgecolor', 'k', 'Tag', 'gridCircle', 'LineStyle', '-'); % Tag given so that this rect can be recognized later
        end
        
        % Centering plot on solar disk center:
        if(strcmp(get(mainHandles.Untitled_121, 'Checked'), 'on') == 1)
            axis([-x_fov+hCenterCoord x_fov+hCenterCoord -y_fov+vCenterCoord y_fov+vCenterCoord]),
        end       
        
    else
        
        % Clear the offset and radius strings:
        set(mainHandles.text63, 'String', '');
        set(mainHandles.text64, 'String', '');
        set(mainHandles.text79, 'String', '');
        
    end
    
    %% Function for modifying data cursor / datatip output:

    function dataTipString = data_cursor_mod(empt, cursor_object)

        pos = get(cursor_object, 'Position');
        
        % coord_finder needs:
        % xcoord: Location in the original data coordinates
        % ycoord: Location in the original data coordinates
        % hCenterCoord: Distance of solar center from data center, horizontal direction
        % vCenterCoord: Distance of solar center from data center, vertical direction
        % rotDeg: Rot angle of the Sun
        % B0: Latitude of the spot on the Sun that is facing Earth
        % L0: Longitude of the spot on the Sun that is facing Earth
        coord_opt.xcoord = pos(1); % X position gotten from cursor, arc minutes
        coord_opt.ycoord = pos(2); % Y position gotten from cursor, arc minutes
        
%         % Transform rotated x and y coordinates back to not rotated ones:
%         if(cursor_opt_rotate_disable == 0)
%             %hCenterCoord = (hCenterCoord_orig * cosd(rotDeg)) + (vCenterCoord_orig * sind(rotDeg));
%             %vCenterCoord = (vCenterCoord_orig * cosd(rotDeg)) - (hCenterCoord_orig * sind(rotDeg));
%             %
%             coord_opt.xcoord = (pos(1) * cosd(-rotDeg)) + (pos(2) * sind(-rotDeg));
%             coord_opt.ycoord = (pos(2) * cosd(-rotDeg)) - (pos(1) * sind(-rotDeg));
%         end
        
        coord_opt.rotDeg = cursor_opt_rotDeg; % rotDeg gotten before cursor mod function is called
        coord_opt.B0 = cursor_opt_B0; % L0 gotten before cursor mod function is called
        coord_opt.L0 = cursor_opt_L0; % B0 gotten before cursor mod funtion is called
        
        % Getting hCenterCoord and vCenterCoord by calling center_finder:
        %center_opt.x = cursor_opt_x;
        %center_opt.y = cursor_opt_y;
        center_opt.mapWidth = cursor_opt_mapWidth;
        center_opt.mapHeight = cursor_opt_mapHeight;
        center_opt.rotDeg = cursor_opt_rotDeg; % rotDeg gotten before cursor mod function is called
        center_opt.rotate_disable = cursor_opt_rotate_disable;
        
        if(mapWidth >= 30 && mapHeight >= 30) % Map is a full disk map:
            
            % ((It's important to always call center_finder with the original
            % values (x,y,map), not the possibly scaled ones))
            % 2011-10-07: it actually seems, that the center is in many
            % cases found more accurately when center_finder is called with
            % the scaled data
            %if(get(mainHandles.checkbox38, 'Value') == 1.0) % Solar grid selected
                % Call center_finder() function:
                %center_opt.x = x_orig;
                %center_opt.y = y_orig;
                %local_map = map_orig;
                %c_info = center_finder(local_map, center_opt);
            %else
            
            % Call center_finder() function:
            center_opt.x = cursor_opt_x;
            center_opt.y = cursor_opt_y;
            local_map = map; % "map" is the "work version" of the map matrix; its units and size are varied as needed, so the units can be "% QSL" or original, and its size can be resized or original.
            % --> center_finder() is called with whatever units are needed
            c_info = center_finder(local_map, center_opt);
            %end
            
            coord_opt.hCenterCoord = c_info.hCenterCoord_orig;
            coord_opt.vCenterCoord = c_info.vCenterCoord_orig;
            coord_opt.radius = cursor_opt_radius;
        
            % Calling coord_finder() function with X and Y coordinates:
            coord_info = coord_finder(coord_opt);
        
        else % Map is a partial map:
             % TBD
        end
        
        
        % center_finder needs _original_ data matrix, x and y
        % However, the z-value finder needs the resized (if solar grid
        % selected) map, because xcoord and ycoord will otherwise not match
        % with values gotten from the plot

        %c_info = center_finder(map, center_opt);
               
        % Find map z-value of cursor location using cursor_opt_x, cursor_opt_y, coord_opt.xcoord, coord_opt.ycoord and map
        % Notice: it's important to use the same x, y and map that have
        % been used in the preview plotting: if resized, then use those, if not, then use original.
        % cursor_opt_x and cursor_opt_y always have the same values as the ones used in preview plotting
        x_index = find(cursor_opt_x == coord_opt.xcoord);
        y_index = find(cursor_opt_y == coord_opt.ycoord);
        map_value = round(map(y_index, x_index) * 100) / 100;
        if(length(map_value) > 1) % On the lower and upper sides of the data matrix, map_value is a vector for some reason
            map_value = map_value(1);
        end
                
        if(mapWidth >= 30 && mapHeight >= 30)
            % Compare x-y -location with respect to disk center to get distance
            % from center. If distance is greater than radius, change output:
            center_dist = sqrt((abs(coord_opt.hCenterCoord-pos(1)))^2 + (abs(coord_opt.vCenterCoord-pos(2)))^2);
            if(center_dist < cursor_opt_radius)
                % Round lat/lon/rel lon to 2 decimal accuracy:
                lat_round= round(coord_info.lat * 100) / 100;
                lon_round= round(coord_info.long * 100) / 100;
                rel_lon_round = round(coord_info.long_rel * 100) / 100;
                
                dataTipString = {['X: ', num2str(pos(1)), '''']...
                    ['Y: ', num2str(pos(2)), ''''],...
                    ['Level: ', num2str(map_value)],...
                    ['Lat: ', num2str(lat_round), char(176)],...
                    ['Lon: ', num2str(lon_round), char(176)],...
                    ['Rel Lon: ', num2str(rel_lon_round), char(176)]};
            else
                dataTipString = {['X: ', num2str(pos(1)), '''']...
                    ['Y: ', num2str(pos(2)), ''''],...
                    ['Level: ', num2str(map_value)],...
                    ['Lat: -'],...
                    ['Lon: -'],...
                    ['Rel Lon: -']};
            end
        else
            dataTipString = {['X: ', num2str(pos(1))]...
                    ['Y: ', num2str(pos(2))],...
                    ['Level: ', num2str(map_value)]};
        end
%         % This was used to get coordinats for tracking with SMART data:
%         % pos = get(cursor_object,'Position');
%         % dataTipString = {['SMART X: ',num2str(-pos(1))],...
%         %        ['SMART Y: ',num2str(pos(2))],...
%         %        ['Value: ', '-']};

    end
    
    %% Finding coordinates of possible active regions:
    
    % Reading Custom Active Region detection treshold from GUI:
    % Protection from invalid values:
    if(get(mainHandles.checkbox51, 'Value') == 1)   
        % Checking that the user has entered only numbers into the edit box:
        cmp1 = strcmp(num2str(str2num(get(mainHandles.edit17, 'String'))), '');
        if(cmp1 == 1)
            msgbox('You have entered something else than numbers in the Active Region treshold field. Please use only numbers (e.g. 103 or 102.58)', 'Invalid AR treshold', 'warn', 'modal');
            return;
        end
    
        % Checking that the edit box isn't empty:     
        if(isempty(get(mainHandles.edit17, 'String')))
            msgbox('You have entered something else than numbers in the Active Region treshold field. Please use only numbers (e.g. 103 or 102.58)', 'Invalid AR treshold', 'warn', 'modal');
            return;
        end
        
        tempTreshold = str2num(get(mainHandles.edit17, 'String'));
    else
        % Set default value:
        tempTreshold = 104; % "% of QSL"
    end
       
    %while (tempTreshold < max_temp)
    spotCheck = get(mainHandles.checkbox41, 'Value'); % Marking of active areas on map
   
    % Define a custom max_temp variable to be used with QSL and original
    % units in the condition for executing the active area search/marking:
    if(dataUnits == 0) % QSL units
        max_temp_cond = max_temp;
    else
        max_temp_cond = max(max(map_orig_qsl));
    end
    
    % Conditions for execution:
    % Solar center finding is on,
    % Rot angle is found,
    % Active area marking is on, 
    % Map max temp is higher than the active area treshold
    % Map size is larger than 30 arc min in both directions
    %if(get(mainHandles.checkbox38, 'Value') == 1.0 && rotDeg ~= -300 && spotCheck == 1 && max_temp_cond >= tempTreshold && mapWidth > 30 && mapHeight > 30)
    if(get(mainHandles.checkbox38, 'Value') == 1.0 && rotDeg ~= -300 && spotCheck == 1 && mapWidth > 30 && mapHeight > 30)

        %[spotRows, spotCols] = find(map_orig > 106); % The original version of the map data and "size vectors" are used
        
        %figure(1);
        
        % Setting up variables needed for calling area_sniffer function:
        %tempTreshold = 104; % "% of QSL"
        opt.mapWidth = mapWidth;
        opt.mapHeight = mapHeight;
        opt.max_temp = max_temp;
        opt.map_orig = map_orig_qsl; % Non-resized QSL units version of the map matrix (calcualted earlier in the code even in the case when the user has selected original units)
        opt.rotDeg = rotDeg;
        opt.hCenterCoord = hCenterCoord_orig; % !!
        opt.vCenterCoord = vCenterCoord_orig; % !!
        opt.radius = sunData.radius;
        opt.x_orig = x_orig;
        opt.y_orig = y_orig;
        opt.B0 = B0;
        opt.L0 = L0;
        opt.dataUnits = dataUnits;

        % Calling area_sniffer function. areaCoords is a struct that contains
        % data on all the active areas of the map
        areaCoords = area_sniffer(tempTreshold, opt);
        
        % DEBUG:
        %areaCoords = coronal_hole_finder(sunmap_handles, 98, opt);
        % / DEBUG

        % If areaCoords struct is empty (no spots were found or something went wrong)
        if(isempty(areaCoords))
            set(mainHandles.text65, 'Visible', 'off');
            drawnow update;
            return;
        end
% -------------------------------------------------------------------------------------------
        %for i = 1 : length(areaCoords) % This for loop was for showing the
        % actual levels from which upwards certain areas are regarded as
        % separate areas. When this is in use, the tempTreshold has to be
        % taken from the individual areas: areaCoords(i).tempTreshold
        % The problem is that the contours are drawn everywhere, not only
        % around the areas that use that exact treshold
        
        % TESTING
%         for i = 1 : length(areaCoords)
%             
%             % spotElements is a list of all the indexes of the map matrix elements in
%             % the area in question (the elements in the listed indexes are in the area)
%             spotElements = areaCoords(i).spotElements;
%             map_temp = map_orig;
%             map_temp(:,:) = 0; % Set everything to zero, but keep the original size
%             for j = 1 : length(spotElements)
%                 map_temp(spotElements(j)) = map_orig(spotElements(j)); % Set the elements in the current area into their own values --> only wanted area has values --> contour can be drawn
%                 % Bummer, the contours are very "pointy" and rectangular
%                 % The contour drawing needs values under and over the
%                 % specific level (for interpolation)
%             end
%             
%             % --->
%             % The contour plotting part here
%             axes(mainHandles.axes2),
%             hold on,
%             warning('off', 'MATLAB:contour:DeprecatedV6Argument');
%             warning('off', 'MATLAB:contour3:DeprecatedErrorOutputArgument');
%             [plaa3, areaCont] = contour('v6',x, y, map, [areaCoords(i).tempTreshold areaCoords(i).tempTreshold], '--k');
%             if(rotDeg ~= -300 && get(mainHandles.checkbox40, 'Value') == 0)
%                 rotate(areaCont, [0 0 1], -rotDeg, [0 0 0]);
%             end
%             % <---
%             
%         end
        % / TESTING
        
%
%-->
% TEMPTRESHOLD-CONTOURIN PLOTTAUSKOODI OLI TÄSSÄ
%<--
%
        % TESTING
%         plaa3
%         size(plaa3);
%         cont_x = plaa3(1,:);
%         cont_y = plaa3(2,:);
%         polyarea(cont_x, cont_y)
%         % areaCont is an array of line handle objects.
%         % Every separate contour line has its own handle --> can be deleted if there is a way to know which is which
%         % plaa3 seems to have info on the vertices of the contour lines.
%         % 2xN matrix
%         % All the data is in plaa3. --> How to separate individual
%         % lines/areas?
%         %areaCont
%         delete(areaCont(1))

        %end
        % / TESTING
% -------------------------------------------------------------------------------------------
   
        % Plot a single contour line to tempTreshold (set at 104 % of QSL 2012-01):
        hold on,
        warning('off', 'MATLAB:contour3:DeprecatedErrorOutputArgument');
        [plaa3, areaCont] = contour(x_orig, y_orig, map_orig_qsl, [tempTreshold tempTreshold], '--k', 'Parent', tr);
        %[plaa3, areaCont] = contour(x_orig, y_orig, map_orig_qsl, [98 98], '--k');
        if(rotDeg ~= -300 && get(mainHandles.checkbox40, 'Value') == 0)
            % rotation done by hgtransform. Applied above by setting tr as
            % 'Parent' to the contour object
            %rotate(get(areaCont, 'Children'), [0 0 1], -rotDeg, [0 0 0]);
        end        

        % Reading active area locations and numbers from web:
        % Setting proxy settings:
        if(ispc == 0) % Use only with non-win computers (= in Metsähovi most likely)
            %jobj = com.mathworks.mlwidgets.html.HTMLPrefs; % This doesn't work
            %setUseProxy(jobj,1);
            %setProxyHost(jobj,'proxy');
            %setProxyPort(jobj,'3128');
        end
        %map_grad = grad(map_orig)
        %datevector = datevec(date);
        year1 = [time(1), time(2), time(3), time(4)];
        dateVector = datevec(date);
        
        if(str2num(year1) >= 1996) % If the map being processed is from the year 1996 or later, data from NOAA can be used
            month1 = [time(6), time(7)];
            day1 = [time(9), time(10)];
            srs_filename = [year1, month1, day1, 'SRS.txt'];
            
            % Calling srs_reader()
            [NOAAspotNumber, NOAAspotLat, NOAAspotLon, NOAAspotAbsLong] = srs_reader(time, sunmap_handles);
                                   
            % Compare NOAA lat/long coords with map lat/long coords:
            % Go through areaCoords struct containing info on active areas on the current map
            for i = 1 : length(areaCoords) % 

                NOAAspotNumberStr = []; % Initializing NOAA spot number string as empty
                
                % Go through NOAA spot coords (if it exists)
                if(exist('NOAAspotLat', 'var'))
                    for j = 1 : length(NOAAspotLat)

                        if(NOAAspotLat(j) < areaCoords(i).Lat + 6 && NOAAspotLat(j) > areaCoords(i).Lat - 6 && NOAAspotAbsLong(j) < areaCoords(i).Long + 10 && NOAAspotAbsLong(j) > areaCoords(i).Long - 10)
                            % Match!
                            NOAAspotNumberStr = NOAAspotNumber{j};
                            NOAAspotNumberStr = NOAAspotNumberStr{1};
                            if(str2num(NOAAspotNumberStr) <= 5000 && str2num(year1) > 2002) % Area number 10000 was passed in June 2002 (http://www.cloudynights.com/ubbthreads/showflat.php/Cat/0/Number/3619261/Main/3618592)
                                NOAAspotNumberStr = ['AR1', NOAAspotNumberStr];
                            else
                                NOAAspotNumberStr = ['AR', NOAAspotNumberStr];
                            end

                        elseif(isempty(NOAAspotNumberStr)) % NOAA spot number string is empty ==> no match has been found on this "round"
                            NOAAspotNumberStr = 'No match';
                        end
                    end
                else
                    NOAAspotNumberStr = 'N/A';
                end

                % Text of the info box:
                infoStr = sprintf(['ID: ', NOAAspotNumberStr, '\nLat: ', num2str(areaCoords(i).Lat, 4), char(176), '\nLon: ', num2str(areaCoords(i).Long, 4), char(176), '\nRel Lon: ', num2str(areaCoords(i).Long_rel, 4), char(176), '\nMax: ', num2str(areaCoords(i).Max_temp, 5), ' %%']);
                % X that marks the active area:
                text1 = text(areaCoords(i).X, areaCoords(i).Y, 'X', 'HorizontalAlignment','center', 'Color', 'k', 'FontSize', 10, 'Parent', tr);
                % Calculating position for info box in pixels (units in text-function are in units of the map (arc min in this case). We want a constant position in pixels instead --> Conversion needed):
                axes2pos = get(mainHandles.axes2, 'Position'); % size and position of plotting window in pixels
                amin_px_ratio =  (2 * y_fov) / axes2pos(4); % ratio of arc min per pixels in plotting window (arc min / px) (arc min = px * (arc_min / px))
                x_correct = 25 * amin_px_ratio; % pixels converted into plotting window arc minutes
                y_correct = 25 * amin_px_ratio; % pixels converted into plotting window arc minutes
                % Info box associated with the X and the active area:
                infoText = text(areaCoords(i).X + x_correct, areaCoords(i).Y + y_correct, infoStr, 'HorizontalAlignment','left', 'Color', 'k', 'FontSize', 6, 'BackgroundColor', [0.99 0.99 0.99], 'EdgeColor', 'k', 'Parent', tr);
                
                % Set info box's z value higher so that the box is shown on
                % top of the active area contour lines
                uistack(infoText, 'top'); % uistack(h) raises object h one unit upwards in the z direction
                % ...aand nothing happens, because the contour lines are
                % plotted after this
                
                % Rotate the X's and labels if the map is rotated:
                if(rotate_disable == 0) % map rotated => compensation needed
                    %rotate(text1, [0 0 1], -rotDeg, [0 0 0]);
                    %rotate(infoText, [0 0 1], -rotDeg, [0 0 0]);   
                end
               
            end % End of for loop for going through areaCoords struct containing info active areas on the current map
                        
        else % Map is from before 1996 --> No NOAA SRS file reading & spot id number, only info from map is drawn:
            
            
            for i = 1 : length(areaCoords) % Assume that variables spotLat & spotLong are the same size
                                                         
                NOAAspotNumberStr = 'N/A';
                % Text of the info box:
                infoStr = sprintf(['ID: ', NOAAspotNumberStr, '\nLat: ', num2str(areaCoords(i).Lat, 4), char(176), '\nLon: ', num2str(areaCoords(i).Long, 4), char(176), '\nRel Lon: ', num2str(areaCoords(i).Long_rel, 4), char(176), '\nMax: ', num2str(areaCoords(i).Max_temp, 5), ' %%']);
                %longStr= ['Long: ', num2str(long,3)];
                %maxStr = ['Max: ', num2str(spot_max, 4)];
                % X that marks the active area:
                text1 = text(areaCoords(i).X, areaCoords(i).Y, 'X', 'HorizontalAlignment','center', 'Color', 'k', 'FontSize', 10, 'Parent', tr);
                % Calculating position for info box in pixels (units in text-function are in units of the map (arc min in this case). We want a constant position in pixels instead --> Conversion needed):
                axes2pos = get(mainHandles.axes2, 'Position'); % size and position of plotting window in pixels
                amin_px_ratio =  (2 * y_fov) / axes2pos(4); % ratio of arc min per pixels in plotting window (arc min / px) (arc min = px * (arc_min / px))
                x_correct = 25 * amin_px_ratio; % pixels converted into plotting window arc minutes
                y_correct = 25 * amin_px_ratio; % pixels converted into plotting window arc minutes
                % Info box associated with the X and the active area:
                infoText = text(areaCoords(i).X + x_correct, areaCoords(i).Y + y_correct, infoStr, 'HorizontalAlignment','left', 'Color', 'k', 'FontSize', 6, 'BackgroundColor', [0.99 0.99 0.99], 'EdgeColor', 'k', 'Parent', tr);

                % Rotate the X's and labels if the map is rotated:
                if(rotate_disable == 0) % map rotated => compensation needed
                    %rotate(text1, [0 0 1], -rotDeg, [0 0 0]);
                    %rotate(infoText, [0 0 1], -rotDeg, [0 0 0]);
                end
               
            end
            
        end
               
        %[spotLocationY, spotLocationX] = find(max_locations==max(max_locations(:)));
        %figure, imshow(imresize(max_locations, [200 200]))
        %contourf(max_locations)
        %figure(2), plot(hGrad5),
        %for i = 1 : length(spotRows)
            %fprintf('\nSample: %d\nSweep: %d\n', spotCols(i), spotRows(i))
        %end
        
    end
    %tempTreshold = tempTreshold + 0.5;
    %end
    
%% Drawing all NOAA Active Regions on map:

if(get(mainHandles.checkbox50, 'Value') == 1.0)
    % Calling srs_reader()
    [NOAAspotNumber, NOAAspotLat, NOAAspotLon, NOAAspotAbsLong] = srs_reader(time, sunmap_handles);
end

year1 = [time(1), time(2), time(3), time(4)];

% Conditions for execution:
% Solar center finding is on,
% Rot angle is found,
% Map size is larger than 30 arc min in both directions
% NOAAspotLat exists (i.e. srs_reader() has returned non empty variables)
% NOAA Active area marking is on
if(get(mainHandles.checkbox38, 'Value') == 1.0 && rotDeg ~= -300 && mapWidth > 30 && mapHeight > 30 && exist('NOAAspotLat', 'var') && get(mainHandles.checkbox50, 'Value') == 1)
    for j = 1 : length(NOAAspotLat)
        % Text of the info box:
        NOAAspotNumberStr = NOAAspotNumber{j};
        NOAAspotNumberStr = NOAAspotNumberStr{1};
        if(str2num(NOAAspotNumberStr) <= 5000 && str2num(year1) > 2002) % Area number 10000 was passed in June 2002 (http://www.cloudynights.com/ubbthreads/showflat.php/Cat/0/Number/3619261/Main/3618592)
            NOAAspotNumberStr = ['AR1', NOAAspotNumberStr];
        else
            NOAAspotNumberStr = ['AR', NOAAspotNumberStr];
        end

        if(L0 >= 90 && L0 <= 270)
            L_rel = NOAAspotAbsLong(j) - L0;
        elseif(NOAAspotAbsLong(j) > 180 && L0 < 90)
            L_rel = NOAAspotAbsLong(j) - 360 - L0;
        elseif(NOAAspotAbsLong(j) > 180 && L0 > 270)
            L_rel = NOAAspotAbsLong(j) - L0;      
        elseif(NOAAspotAbsLong(j) <= 180 && L0 < 90)
            L_rel = NOAAspotAbsLong(j) - L0;
        elseif(NOAAspotAbsLong(j) <= 180 && L0 > 270)
            L_rel = NOAAspotAbsLong(j) + 360 -L0;
        end
        
        infoStr = sprintf(['ID: ', NOAAspotNumberStr, '\nLat: ', num2str(NOAAspotLat(j), 4), char(176), '\nLon: ', num2str(NOAAspotAbsLong(j), 4), char(176), '\nRel Lon: ', num2str(L_rel, 4), ' / ', num2str(NOAAspotLon(j)), char(176)]);

        if(L0 <= 180) % L0: 0...180
            L0_mod = L0; % The right half of the solar sphere starting from the zero meridian as seen from Earth 0...180 degrees
        else % L0: 180...360
            L0_mod = L0 - 360; % The left half of the solar sphere starting from the zero meridian as seen from Earth 0...-180 degrees
        end

        % X that marks the active area:
        % Calculate x and y from spot lat & rel long with orthographic projection:
        % Compensate center coordinates for map rotation:
        hCenterCoord_mod = (hCenterCoord_orig * cosd(rotDeg)) + (vCenterCoord_orig * sind(rotDeg));
        vCenterCoord_mod = (vCenterCoord_orig * cosd(rotDeg)) - (hCenterCoord_orig * sind(rotDeg));               
        x_loc = sunData.radius * cosd(NOAAspotLat(j)) * sind(L_rel) + hCenterCoord_mod;
        y_loc = sunData.radius * (((cosd(B0)) * sind(NOAAspotLat(j))) - (sind(B0) * cosd(NOAAspotLat(j)) * cosd(L_rel))) + vCenterCoord_mod;
        tr2 = hgtransform;
        text2 = text(x_loc, y_loc, 'X', 'HorizontalAlignment','center', 'Color', 'w', 'FontSize', 12, 'Parent', tr2);
        % Calculating position for info box in pixels (units in text-function are in units of the map (arc min in this case). We want a constant position in pixels instead --> Conversion needed):
        axes2pos = get(mainHandles.axes2, 'Position'); % size and position of plotting window in pixels
        amin_px_ratio =  (2 * y_fov) / axes2pos(4); % ratio of arc min per pixels in plotting window (arc min / px) (arc min = px * (arc_min / px))
        x_correct = 15 * amin_px_ratio; % pixels converted into plotting window arc minutes
        y_correct = 22 * amin_px_ratio; % pixels converted into plotting window arc minutes        
        % Info box associated with the X and the active area:        
        infoText2 = text(x_loc + x_correct, y_loc + y_correct, infoStr, 'HorizontalAlignment','left', 'Color', 'k', 'FontSize', 6, 'BackgroundColor', [0.99 0.80 0.70], 'EdgeColor', 'k', 'Parent', tr2);
        % Rotate the X's and labels if the map is not rotated:
        if(rotate_disable == 1) % map NOT rotated => compensation needed in the opposite direction than normally (rotDeg vs. -rotDeg)
            set(tr2,'Matrix',makehgtform('zrotate',rotDeg*pi/180))
            %rotate(text2, [0 0 1], rotDeg, [0 0 0]);
            %rotate(infoText2, [0 0 1], rotDeg, [0 0 0]);   
        end                    
    end
end
% / TESTING

    
%% Testing
if(1==0)

    %if(exist('map_orig', 'var'))
       % fit_opt.map = map_orig;
      %  fit_opt.x = x_orig;
     %   fit_opt.y = y_orig;
    %else
    fit_opt.map = map;
    fit_opt.x = x;
    fit_opt.y = y;
    %end
    % fit_opt.nswp = nswp;
    % fit_opt.sps = sps;
    % fit_opt.direction = direction;

    [sphere_center sphere_radius sphere_residuals] = sphere_fitter(fit_opt);
    
    % Draw fitted circle on map:
    rectangle('Curvature',[1 1], 'Position', [sphere_center(1)-sphere_radius sphere_center(2)-sphere_radius sphere_radius*2 sphere_radius*2], 'Linewidth', 1, 'edgecolor', 'k', 'Tag', 'gridCircle', 'LineStyle', '-'); % Tag given so that this rect can be recognized later
    
end

set(mainHandles.text65, 'Visible', 'off'); % "Generating preview..."
drawnow update;

end
    
    
    
  