% Function qsl_finder()
% Finds the Quiet Sun Level (median level) of a solar map matrix.
% Uses the mean value of the maximums of multiple histograms made with different bin sizes
% Input:
% - Solar map matrix (map (matrix))
% - Units of the map (dataUnits (scalar, used as boolean. 1 => original units, 0 => % QSL units))
% Output:
% -Quiet Sun Level (QSL) of the map (quiet_sun (scalar))

function quiet_sun = qsl_finder(map, dataUnits)

    % Finding median level (quiet sun level):
    map_size = size(map);
    max_temp = max(max(map));
    min_temp = min(min(map));
    
    if(dataUnits == 0)
        dist_from_max = 0.7 * max_temp; % Was 7000 before
    elseif(dataUnits == 1)
        dist_from_max = 0.7 * abs(max_temp - min_temp);
    else
        dist_from_max = 0.7 * max_temp; % Was 7000 before
    end
    % The search for the value on the Sun's surface that has the most
    % "area" covered (QSL) is started from (max_temp - dist_from_max)
    vect = (max_temp - dist_from_max) : 50 : max_temp; % 50 is an ok number for getting enough bins for accuracy and not too many bins so that small scale "noise" can skew results

    % Write the values of the map that we want to study in a vector:
    aux_vector(1) = 0;
    t = 1;
    for j=1:map_size(1)
        for g=1:map_size(2)
            if map(j,g) > (max_temp - dist_from_max)
                 aux_vector(t)=map(j,g);
                 t=t+1;
            end
        end
    end

    % Check if the sizes of the vectors are sensible for calling hist()
    if(length(vect) > 5 && length(aux_vector) > 5)
    
        [frequencies, locations] = hist(aux_vector, vect);
    
    else % If there is a problem with the vectors, just write something in the quiet_sun variable
        % When this happens, there are so few levels in the map that it
        % makes no difference what the quiet sun level is
        
        %errordlg('Unable to find proper Quiet Sun Level', 'Error', 'modal');
        quiet_sun = min_temp + 0.5*abs(max_temp - min_temp);
        return;
        
    end

    % For debugging and testing:
    %size(frequencies);
    %figure,
    %hist(aux_vector, vect)
    %figure,
    %histfit(aux_vector, 20, 'kernel')
    % /
    
    % Create multiple histograms from the map data.
    % Write the maximum value of each historgam into a variable
    for index11 = 1 : 30
        if(dataUnits == 0)
            hist_vect = (0.3*max_temp) : (0.3*max_temp/(20*index11)) : max_temp;
        elseif(dataUnits == 1)
            hist_vect = (max_temp - dist_from_max) : (0.3 * abs(max_temp - min_temp)/(20*index11)) : max_temp;
        else
            hist_vect = (0.3*max_temp) : (0.3*max_temp/(20*index11)) : max_temp;
        end
        [freqs, locs] = hist(aux_vector, hist_vect);
        maxs = find(freqs==max(freqs));
        hist_max(index11) = locs(maxs(1));
        %hold on,
        %hist(aux_vector, hist_vect);
        %hist_obj = findobj(gca,'Type','patch');
    end
    
    % Debugging and testing:
    %figure,
    %plot(hist_max);
    %figure,
    %hist(hist_max)
    % /

    % Write the mean value of the histogram maximum values into quiet_sun:
    quiet_sun = mean(hist_max);
    %quiet_sun = 36000; % Backup for manual QSL

end % End of qsl_finder()









%% CODE DUMP:
% (Dumped 2011-12-22)

% ------------------------------------------
% This was for '% of QSL' units:
%         % Finding median level (quiet sun level):
% 
%         map_size = size(map);
%         max_temp = max(max(map));
%         dist_from_max = 0.7 * max_temp; % Was 7000 before
%         % The search for the value on the Sun's surface that has the most
%         % "area" covered (QSL) is started from (max_temp - dist_from_max)
%         % 
%         vect = (max_temp - dist_from_max) : 50 : max_temp; % 50 is an ok number for getting enough bins for accuracy and not too many bins so that small scale "noise" can skew results
%         % (0.3*max_temp/100)
%         aux_vector(1) = 0;
%         t = 1;
%         for j=1:map_size(1)
%             for g=1:map_size(2)
%                 if map(j,g) > (max_temp - dist_from_max)
%                      aux_vector(t)=map(j,g);
%                      t=t+1;
%                 end
%             end
%         end
% 
%         [frequencies, locations] = hist(aux_vector, vect);
%         
%         % For debugging and testing:
%         %size(frequencies);
%         %figure,
%         %hist(aux_vector, vect)
%         for index11 = 1 : 30
%             hist_vect = (0.3*max_temp) : (0.3*max_temp/(20*index11)) : max_temp;
%             [freqs, locs] = hist(aux_vector, hist_vect);
%             maxs = find(freqs==max(freqs));
%             hist_max(index11) = locs(maxs(1));
%             %hold on,
%             %hist(aux_vector, hist_vect);
%             %hist_obj = findobj(gca,'Type','patch');
%         end
% %         figure,
% %         plot(hist_max);
% %         figure,
% %         hist(hist_max)
%         
%         quiet_sun = mean(hist_max);
%         median(hist_max);
% ------------------------------------------


% ------------------------------------------
% This was for Original data units:
% %         % Finding median level (quiet sun level):
% % 
%         map_size = size(map);
%         max_temp = max(max(map));
%         min_temp = min(min(map));
%         dist_from_max = 0.7 * abs(max_temp - min_temp); % Was 7000 before
%         % The search for the value on the Sun's surface that has the most
%         % "area" covered (QSL) is started from (max_temp - dist_from_max)
% 
%         vect = (max_temp - dist_from_max) : 50 : max_temp; % 50 is an ok number for getting enough bins for accuracy and not too many bins so that small scale "noise" can skew results
%         
%         aux_vector(1) = 0;
%         t=1;
%         for j=1:map_size(1)
%             for g=1:map_size(2)
%                 if map(j,g) > (max_temp - dist_from_max)
%                      aux_vector(t)=map(j,g);
%                      t=t+1;
%                 end
%             end
%         end
% 
%        [frequencies, locations] = hist(aux_vector, vect);
% 
%            % For debugging and testing:
%            %size(frequencies);
% %         figure,
% %         hist(aux_vector, vect)
%         for index11 = 1 : 30
%             hist_vect = (max_temp - dist_from_max) : (0.3 * abs(max_temp - min_temp)/(20*index11)) : max_temp;
%             [freqs, locs] = hist(aux_vector, hist_vect);
%             maxs = find(freqs==max(freqs));
%             hist_max(index11) = locs(maxs(1));
%             %hold on,
%             %hist(aux_vector, hist_vect);
%             %hist_obj = findobj(gca,'Type','patch');
%         end
% %         figure,
% %         plot(hist_max);
% %         figure,
% %         hist(hist_max)
%         
%         quiet_sun = mean(hist_max);
%         median(hist_max);
% ------------------------------------------ 

% ------------------------------------------
% This was for '% of QSL' units _within_ the if clause for original data
% units
%         % Finding median level (quiet sun level):
% 
%         map_tmp_size = size(map_tmp);
%         max_temp_tmp = max(max(map_tmp));
%         dist_from_max = 0.7 * max_temp_tmp; % Was 7000 before
%         % The search for the value on the Sun's surface that has the most
%         % "area" covered (QSL) is started from (max_temp - dist_from_max)
% 
%         vect = (max_temp_tmp - dist_from_max) : 50 : max_temp_tmp; % 50 is an ok number for getting enough bins for accuracy and not too many bins so that small scale "noise" can skew results
% 
%         t=1;
%         aux_vector(1) = 0;
%         for j=1:map_tmp_size(1)
%             for g=1:map_tmp_size(2)
%                 if map_tmp(j,g) > (max_temp_tmp - dist_from_max)
%                      aux_vector(t)=map_tmp(j,g);
%                      t=t+1;
%                 end
%             end
%         end
% 
%         %hist(aux_vector, vect);
% 
%         [frequencies, locations] = hist(aux_vector, vect);
%            
%         % For debugging and testing:
%         %size(frequencies);
% %         figure,
% %         hist(aux_vector, vect)
%         for index11 = 1 : 30
%             hist_vect = (0.3*max_temp_tmp) : (0.3*max_temp_tmp/(20*index11)) : max_temp_tmp;
%             [freqs, locs] = hist(aux_vector, hist_vect);
%             maxs = find(freqs==max(freqs));
%             hist_max(index11) = locs(maxs(1));
%             %hold on,
%             %hist(aux_vector, hist_vect);
%             %hist_obj = findobj(gca,'Type','patch');
%         end
% %         figure,
% %         plot(hist_max);
% %         figure,
% %         hist(hist_max)
%         
%         quiet_sun_tmp = mean(hist_max);
% %         median(hist_max);
% ------------------------------------------ 


% ------------------------------------------
% Experimenting with histogram sky level finding:
%         aux_vector1 = map(:);
%         
%         vect1 = min(min(map)) : 20 : max(max(map));
%         [frequencies1, locations1] = hist(aux_vector1, vect1);
%         figure,hist(aux_vector1, vect1)
%         maximums1 = find(frequencies1==max(frequencies1));
%         sky_level = locations1(maximums1(1));
%         % If the histogram based sky level and the minimum of the map
%         % aren't too far apart, the min value of the map can be used as the
%         % sky level.
%         % If the min value doesn't represent the sky level then the
%         % histogram based value will be used
%         %fprintf('Min: %d\nMax: %d\nDiff: %s\nSky level: %d\n', min(min(map)), max(max(map)), num2str(min(min(map)) + 0.05*(max(max(map)) - min(min(map)))), sky_level);
% %         if(min(min(map)) + 0.05*(max(max(map)) - min(min(map))) > sky_level)
% %            map = map - sky_level;
% %         else
% %             map = map - min(min(map));
% %         end
%         
%         [maxtab, mintab] = peakdet(frequencies1, 1, locations1);
%         %hold on; plot(mintab(:,1), mintab(:,2), 'g*');
%         %plot(maxtab(:,1), maxtab(:,2), 'r*');        
%         while 1
%             maxtab_orig = maxtab;
%             [maxtab, mintab] = peakdet(maxtab(:,2), 1, maxtab(:,1));
%             maxtab_size = size(maxtab);
%             if(maxtab_size(1) < 2)
%                 maxtab = maxtab_orig;
%                 break;
%             end
%         end
%         hold on,
%         plot(maxtab(:,1), maxtab(:,2), 'r*');
%         %maxtab
%         %maxtab(:,2)
%         sorted_2 = sort(maxtab(:,2), 'descend'); % sort the frequencies
%         % find two largest frequencies
%         max_idx_1 = find(maxtab(:,2) == sorted_2(1));
%         max_idx_2 = find(maxtab(:,2) == sorted_2(2));
%         
%         if(maxtab(max_idx_1, 1) > maxtab(max_idx_2))
%             quiet_sun = maxtab(max_idx_1);
%             sky_level = maxtab(max_idx_2);
%         else
%             quiet_sun = maxtab(max_idx_2);
%             sky_level = maxtab(max_idx_1);
%         end
%         
%         quiet_sun
%         sky_level
%         
%         map = map - sky_level;
%         quiet_sun = quiet_sun - sky_level;
        
% the one of the two that has the larger value in maxtab(:,1) is
% QSL and the one woth the smaller value is sky temp


%assignin('base', 'map_log', map);
% / Experimenting
% ------------------------------------------



