% Function radius_file_saver()
% For saving solar radius data from a struct into a .txt file

% Input:
% - mainHandles: The handles object to the Sunmap main GUI (handles object)
% - radiusData: Solar radius data (struct)
%       - Fields in radiusData ("i" is just a running integer index)
%            radiusData(i).time:                The observation time of the individual solar map, string ("YYYY-MM-DD, hh:mm:ss - hh:mm:ss")r 
%            radiusData(i).id:                    The solar map file id, "sunYYMMDDNN" for SMART data, 10-digit unix time for sun-measure data,
%            radiusData(i).freq:                 A string that tells the frequency of the observation, "XX Ghz"
%            radiusData(i).time_obj:
%            radiusData(i).sky_qsl_diff
%            radiusData(i).radius_calc
%            radiusData(i).radius_meas
%            radiusData(i).dist = dist;

% Ouput:
% - No variables, just the radius data file containing the data from
% radiusData struct gotten as input

% Sunmap is a program developed by Niko Lavonen, Metsähovi Radio observatory (nlavonen@cc.hut.fi).

function radius_file_saver(sunmap_handles, radiusData)

%Get access to the main GUI's handles object
mainHandles = guidata(sunmap_handles);

% Define default daving location:
radiusFileName_default = [pwd,'/MRO_solar_radius.txt']; % num2str(trackData.first), '-', num2str(trackData.last), '_', trackData.scale, 

% Ask user for file name;
[radiusFileName, radiusFileDir] = uiputfile(radiusFileName_default);

% If the user cancels, return:
if(radiusFileName == 0)
    if(radiusFileDir == 0)
        return;
    end
end

radiusFileNamePath = [radiusFileDir, radiusFileName];

% Create and open the radius data target file:
radius_file = fopen(radiusFileNamePath, 'a'); % 'a': Open or create new file for writing. Append data to the end of the file.

tab = '\t';

% When file is empty, write header row:
s = dir(radiusFileNamePath); % s.bytes tells the number of bytes in the data file
if(s.bytes < 5) % If file is empty, write header row:
    fprintf(radius_file, [sprintf('%-40s','Time') tab sprintf('%-15s', 'Map ID') tab sprintf('%-12s','Freq.') tab sprintf('%-30s','Absolute radius [arc min]') tab sprintf('%-25s','1 AU normalized radius [arc min]'), '\n']);
    %fprintf(radius_file, [sprintf('%-40s','Start time [YYYYMMDDhhmmss]') sprintf('%-25s','1 AU normalized radius (")'), '\n']);
end

% Go through the radiusData struct:
for idx = 1 : length(radiusData)

    radiusFileContents = textread(radiusFileNamePath, '%s', 'delimiter', '\n');

%     % Modify time into "YYYYMMDDhhmmss" format
%     time_str = radiusData(idx).time;
%     time_year_str = time_str(1:4); % YYYY
%     time_month_str = time_str(6:7); % MM
%     time_day_str = time_str(9:10); % DD
%     time_hour_str  = time_str(13:14); % hh
%     time_minute_str =  time_str(16:17); % mm
%     time_second_str = time_str(19:20); % ss
%     
%     time_final = [time_year_str, time_month_str, time_day_str, time_hour_str, time_minute_str, time_second_str];
%     time_final = regexprep(time_final, ' ', '0'); % Replace any whitespaces in the string with zeros. This fixes missing leading zeros in SMART data
    
    time_print = sprintf('%-40s', radiusData(idx).time);
%time_print = sprintf('%-40s', time_final);
    id_print = sprintf('%-15s', radiusData(idx).id);
    freq_print = sprintf('%-15s', radiusData(idx).freq);
    abs_radius_print = sprintf('%-30s', num2str(radiusData(idx).radius_meas));
    %norm_radius_print = sprintf('%-25s', num2str((60 * radiusData(idx).radius_meas * radiusData(idx).dist)));
    norm_radius_print = sprintf('%-25s', num2str((radiusData(idx).radius_meas * radiusData(idx).dist)));
    
    printString = sprintf([time_print, tab, id_print, tab, freq_print, tab, abs_radius_print, tab, norm_radius_print, '\n']);
    %printString = sprintf([time_print, norm_radius_print, '\n']); % Alternative file notation 
    
    % Testing if current string already exists in the file:
    stringChecker = 0;
    for idx4 = 1 : length(radiusFileContents)
        if (strcmp(strtrim(radiusFileContents{idx4}), strtrim(printString)) == 1) % strtrim removes leading and trailing white spaces
            stringChecker = 1; % The exact same line is already in the file --> no saving at this round
        end
    end

    if(stringChecker == 0) % This line does not exist in the file --> write line in file
        fprintf(radius_file, printString);
    end
        
end

% Close radius data file:
fclose(radius_file);

end