%% sphere_fitter function
% A function that is used for fitting a circle to a solar disk

% Input:
% A struct that includes the solar map data matrix itself and two vectors
% used in setting the data points to the right coordinates (basically setting the physical dimensions of the data in the sky)

% Output:
% The center coordinates of the fitted circle
% The radius of the fitted circle
% The residuals from the fitting

% sphere_fitter calls another function, spherefit.m that does the actual
% least squares circle fitting

% Sunmap is a program developed by Niko Lavonen, Metsähovi Radio observatory (nlavonen@cc.hut.fi).

function [sphere_center sphere_radius sphere_residuals] = sphere_fitter(opt)

% Needed:
% opt.map: The map data, normalized in % QSL units (sky...max_temp (~3...100+ % QSL))
% opt.x: The x size vector of the map
% opt.y: The y size vector of the map

% Testing sphere fit:
% 'direction' tells whether map is "RA-map: " or "DE-map: "
   
% Go trough the original map and save indexes from 50% qsl values

ind1 = 1;

% cols_c, rows_c
map_size = size(opt.map); % This is done to avoid any discrepancies between the map data size and nwsp & sps
rows_c = map_size(1);
cols_c = map_size(2);

% Sweep-wise:
%----x----
%-x-----x-
%-x-----x-
%----x----
for ind2 = 1 : rows_c-1

    for ind3 = 1 : cols_c-1

        if(opt.map(ind2, ind3) <= 50 && opt.map(ind2, ind3+1) > 50) % Rising edge
            % Interpolate linearly between values:
            x_c(ind1) = opt.x(ind3) + ( ((50 - opt.map(ind2, ind3)) / (opt.map(ind2, ind3+1) - opt.map(ind2, ind3))) * (opt.x(ind3+1) - opt.x(ind3)) );
            y_c(ind1) = opt.y(ind2); % We're going along the sweep path, so the y-value is taken from the sweep's y-location

            % Draw line on map to test location:
            %line([x_c(ind1) x_c(ind1)], [20 -20], 'color', 'k', 'LineWidth', 1, 'LineStyle', '-'); % Vertical line
            %line([-20 20], [-opt.y(ind2) opt.y(ind2)], 'color', 'k', 'LineWidth', 1, 'LineStyle', '-'); % Horizontal line
            %text1 = text(x_c(ind1), y_c(ind1), 'X', 'HorizontalAlignment','center', 'Color', 'w', 'FontSize', 12);
            ind1 = ind1 + 1;

        end

        if(opt.map(ind2, ind3) > 50 && opt.map(ind2, ind3+1) <= 50) % Setting edge

            % Interpolate linearly between values:
            x_c(ind1) = opt.x(ind3) + ( ((opt.map(ind2, ind3) - 50) / (opt.map(ind2, ind3) - opt.map(ind2, ind3+1))) * (opt.x(ind3+1) - opt.x(ind3)) );
            y_c(ind1) = opt.y(ind2);
            % Draw line on map to test location:
            %line([x_c(ind1) x_c(ind1)], [20 -20], 'color', 'k', 'LineWidth', 1, 'LineStyle', '-'); % Vertical line
            %line([-20 20], [-opt.y(ind2) opt.y(ind2)], 'color', 'k', 'LineWidth', 1, 'LineStyle', '-'); % Horizontal line
            %text1 = text(x_c(ind1), y_c(ind1), 'X', 'HorizontalAlignment','center', 'Color', 'w', 'FontSize', 12);
            ind1 = ind1 + 1;

        end

    end


end

% Sample-wise:
%| | |  x  | | |
%| x |  |  | x |
%| x |  |  | x |
%| | |  x  | | |
for ind4 = 1 : cols_c-1

    for ind5 = 1 : rows_c-1

        if(opt.map(ind5, ind4) <= 50 && opt.map(ind5+1, ind4) > 50) % Rising edge
            % Interpolate linearly between values:
            y_c(ind1) = opt.y(ind5) + ( ((50 - opt.map(ind5, ind4)) / (opt.map(ind5+1, ind4) - opt.map(ind5, ind4))) * (opt.y(ind5+1) - opt.y(ind5)) );
            x_c(ind1) = opt.x(ind4); % We're going along the sample path, so the x-value is taken from the sample's x-location
            %text1 = text(x_c(ind1), y_c(ind1), 'X', 'HorizontalAlignment','center', 'Color', 'w', 'FontSize', 12);
            ind1 = ind1 + 1;

        end

        if(opt.map(ind5, ind4) > 50 && opt.map(ind5+1, ind4) <= 50) % Setting edge

            % Interpolate linearly between values:
            y_c(ind1) = opt.y(ind5) + ( ((opt.map(ind5, ind4) - 50) / (opt.map(ind5, ind4) - opt.map(ind5+1, ind4))) * (opt.y(ind5+1) - opt.y(ind5)) );
            x_c(ind1) = opt.x(ind4);
            %text1 = text(x_c(ind1), y_c(ind1), 'X', 'HorizontalAlignment','center', 'Color', 'w', 'FontSize', 12);
            ind1 = ind1 + 1;

        end

    end

end

%line([12 12], [20 -20], 'color', 'k', 'LineWidth', 1, 'LineStyle', '-'); % Vertical line
%x_c

%--------------------------------------
% fitter() function:
% Drop points that deviate more than a treshold value from the fitted
% sphere, repeat if neccessary:
function [x_out y_out sphere_center, sphere_radius, sphere_residuals] = fitter(x_in, y_in)

    % Fit sphere:

    % fill z-variable with zeros:
    z_in = zeros(1, length(y_in));
    warning('off', 'MATLAB:rankDeficientMatrix');
    [sphere_center, sphere_radius, sphere_residuals] = spherefit(x_in, y_in, z_in);

    % Center correcred x_c and y_c:
    % THESE ARE NOT WHAT THEY ARE SUPPOSED TO BE!
    x_diff = abs(x_in - sphere_center(1));
    y_diff = abs(y_in - sphere_center(2));

    % Go through every point and check if it qualifies:
    for idx2 = 1 : length(x_diff)

        point_radius = sqrt(x_diff(idx2)^2 + y_diff(idx2)^2);

        % The accuracy demand is given here:
        if(abs(point_radius - sphere_radius) < 0.3)
            if(exist('x_out', 'var') == 0)
                x_out(1) = x_in(idx2);
                y_out(1) = y_in(idx2);    
            else
                x_out(end+1) = x_in(idx2);
                y_out(end+1) = y_in(idx2);    
            end

        end

    end

end % End of fitter() function
 %--------------------------------------

[foo1, foo2, sphere_center, sphere_radius(1), sphere_residuals] = fitter(x_c, y_c);

idx4 = 1;

% Keep calling the fitter function until the current and previous fit
% differ from each other less than 0.0005' = 0.03" or until the function
% has been called 20 times
while 1
    [foo1, foo2, sphere_center, sphere_radius(idx4+1), sphere_residuals] = fitter(foo1, foo2);
    radius_diff = abs(sphere_radius(idx4) - sphere_radius(idx4+1));
    if(radius_diff < 0.0005 || idx4 > 20) % ~0.03"
        break;
    end
    idx4 = idx4 + 1;
end

sphere_radius = sphere_radius(end);

% Draw 'X' for each point that is used for sphere fitting:
% The locations are accurate only when solar rotation is disabled!
 %for idx3 = 1 : length(foo1)
 %    text1 = text(foo1(idx3), foo2(idx3), 'X', 'HorizontalAlignment','center', 'Color', 'w', 'FontSize', 12);
 %end

% Two circles, one for the inner error limit, the other for the outer
%rectangle('Curvature',[1 1], 'Position', [sphere_center(1)-(sphere_radius+0.25) sphere_center(2)-(sphere_radius+0.25) (sphere_radius*2+0.5) (sphere_radius*2+0.5)], 'Linewidth', 1, 'edgecolor', 'r', 'Tag', 'gridCircle_outer', 'LineStyle', '-'); % Tag given so that this rect can be recognized later
%rectangle('Curvature',[1 1], 'Position', [sphere_center(1)-(sphere_radius-0.25) sphere_center(2)-(sphere_radius-0.25) (sphere_radius*2-0.5) (sphere_radius*2-0.5)], 'Linewidth', 1, 'edgecolor', [0 0.5 0], 'Tag', 'gridCircle_inner', 'LineStyle', '-'); % Tag given so that this rect can be recognized later

end % End of sphere_fitter() funciton














