%% Function srs_reader()

% Reads Active Region data from a NOAA SRS file based on time of observation of a solar map and a handles
% object of the Sunmap GUI

% This function is a part of Sunmap
% Sunmap is a Matlab program developed by Niko Lavonen, Metsähovi Radio observatory (nlavonen@cc.hut.fi).

function [NOAAspotNumber, NOAAspotLat, NOAAspotLon, NOAAspotAbsLong] = srs_reader(time, sunmap_handles)

% Initialize the output variables as empty:
NOAAspotNumber = [];
NOAAspotLat = [];
NOAAspotLon = [];
NOAAspotAbsLong = [];

mainHandles = guidata(sunmap_handles);

year1 = [time(1), time(2), time(3), time(4)];
dateVector = datevec(date);

if(str2num(year1) >= 1996) % If the map being processed is from the year 1996 or later, data from NOAA can be used
    month1 = [time(6), time(7)];
    day1 = [time(9), time(10)];
    srs_filename = [year1, month1, day1, 'SRS.txt'];

    if(str2num(year1) == dateVector(1)) % If the map is from the current year --> read .txt file from NOAA web server
        spotFileUrl = ['ftp://ftp.swpc.noaa.gov/pub/warehouse/', num2str(year1), '/SRS/', srs_filename]; % Updated 2014-12-16 from: ['http://www.swpc.noaa.gov/ftpdir/warehouse/', num2str(year1), '/SRS/', srs_filename]
        try % Try reading the srs file for the current day
        spotFile = urlread(spotFileUrl);
        % If the file opening fails, notify the user and reset the
        % GUI:
        catch err
            if (strcmp(err.identifier, 'MATLAB:urlread:ConnectionFailed'))
                %message = 'Unable to reach NOAA web server. Recognising active areas with NOAA area numbers can''t be done.';
                %msgbox(message, 'Error', 'error', 'non-modal');
                set(mainHandles.text39, 'ForegroundColor', 'r');
                set(mainHandles.text39, 'String', 'Unable to reach NOAA web server, AR ID''s not available');
            elseif (strcmp(err.identifier, 'MATLAB:urlread:FileNotFound'))
                set(mainHandles.text39, 'ForegroundColor', 'r');
                set(mainHandles.text39, 'String', 'NOAA SRS file not found, AR ID''s not available');
            else % With other errors just print the error as usual:
                rethrow(err);
            end
            %set(mainHandles.text65, 'Visible', 'off');
            %drawnow update;
            %return;
        end
        if(exist('spotFile', 'var'))
            spotFileRows = regexp(spotFile,'\n','split');
        end
    else % Map from last year or earlier --> read .txt file from Sunmap's aux_data folder
        spotFilePath = [pwd, '/aux_data/NOAA SWPC/', num2str(year1), '_SRS/', srs_filename];
        if(exist(spotFilePath, 'file'))
            spotFileRows = textread(spotFilePath, '%s', 'delimiter', '\n');
        else
            %message = ['The NOAA SRS active area data file ', spotFilePath, ' doesn''t seem to exist. If the year has just changed, the data files for last year probably haven''t yet been downloaded to sunmap''s folder: /aux_data/NOAA SWPC/YYYY_SRS/ The data files should be found in NOAA''s web page. E.g. for 2011: http://www.swpc.noaa.gov/ftpdir/warehouse/2011/2011_SRS.tar.gz'];
            %msgbox(message, 'Missing data', 'warn', 'modal');
            set(mainHandles.text39, 'ForegroundColor', 'r');
            set(mainHandles.text39, 'String', 'NOAA SRS file not found, AR ID''s not available');
            %axes(mainHandles.axes2), % Set the plotting axes to axes2 to prevent stuff from being drawn on the error message window
        end
    end


    % Going through the SRS file (if one has been opened)
    if(exist('spotFileRows', 'var'))
        for idx3 = 1 : length(spotFileRows)
            %disp(spotFileRows{idx3});
            %disp('Nmbr Location  Lo  Area  Z   LL   NN Mag Type');
            %disp(strcmp(strtrim(spotFileRows{idx3}), 'Nmbr Location  Lo  Area  Z   LL   NN Mag Type'));
            if(strcmp(strtrim(spotFileRows{idx3}), 'Nmbr Location  Lo  Area  Z   LL   NN Mag Type') == 1 || strcmp(strtrim(spotFileRows{idx3}), 'NMBR LOCATION  LO  AREA  Z   LL   NN MAG TYPE') == 1)
                % Write stuff into the NOAA variables:
                % Take next row, split it into separate 'words'. If there
                % are 8 words in the row, we're dealing with a sunspot row,
                % when there are 10 "words" we've reached the next "header"
                ind = 0;
                thisRow = regexp(strtrim(spotFileRows{idx3+1 + ind}), '\s+', 'split'); % split tabs & spaces
                while (length(thisRow) == 8)
                    
                    % From the SRS file we read:
                    % spot number,
                    % spot latitude,
                    % spot realtive longitude (not in helicentric coords, but from central meridian seen from Earth)
                    % These are enough for comparison with the coords from the map

                    ind = ind + 1;
                    NOAAspotNumber{ind} = thisRow(1);
                    %thisRow(1)
                    NOAAspotLocation{ind} = thisRow(2);

                    % Extract Lat value from lat/long string (SXXWYY / NXXEYY etc.)
                    NOAAcurrentLatStr = thisRow(2);
                    NOAAcurrentLatStr = NOAAcurrentLatStr{1};
                    NOAAspotLatStr = [NOAAcurrentLatStr(1), NOAAcurrentLatStr(2), NOAAcurrentLatStr(3)]; % S25, N05 etc.
                    if(strcmp(NOAAcurrentLatStr(1), 'S') == 1) % South
                        NOAAspotLat(ind) = str2num(['-', NOAAcurrentLatStr(2), NOAAcurrentLatStr(3)]);
                    elseif(strcmp(NOAAcurrentLatStr(1), 'N') == 1) % North
                        NOAAspotLat(ind) = str2num([NOAAcurrentLatStr(2), NOAAcurrentLatStr(3)]);
                    end

                    % TESTING:
                    % Extract Lon value from lat/lon string:
                    NOAAcurrentLonStr = thisRow(2);
                    NOAAcurrentLonStr = NOAAcurrentLonStr{1};
                    NOAAspotLonStr = [NOAAcurrentLonStr(4), NOAAcurrentLonStr(5), NOAAcurrentLonStr(6)]; % W25, E05 etc.
                    if(strcmp(NOAAcurrentLonStr(4), 'E') == 1) % East
                        NOAAspotLon(ind) = str2num(['-', NOAAcurrentLonStr(5), NOAAcurrentLonStr(6)]);
                    elseif(strcmp(NOAAcurrentLonStr(4), 'W') == 1) % West
                        NOAAspotLon(ind) = str2num([NOAAcurrentLonStr(5), NOAAcurrentLonStr(6)]);
                    end   
                    % / TESTING

                    %thisRow(2)
                    NOAAlong_temp = thisRow(3);
                    NOAAspotAbsLong(ind) = str2num(NOAAlong_temp{1});
                    %thisRow(3)
                    NOAAspotArea{ind} = thisRow(4); % This is not used for now
                    %thisRow(4)
                    % Read the next row for use on the next iteration
                    thisRow = regexp(strtrim(spotFileRows{idx3+1 + ind}), '\s+', 'split'); % split by tabs & spaces


                    % Starting to read the "IA. H-alpha Plages without Spots" section:
                    % Checking if we've reached the next header yet
                    % The length of the row that is read above is checked.
                    % This: "IA. H-alpha Plages without Spots.  Locations Valid at 19/2400Z Jun"
                    % ..means that we've reached the next header (length (num of separate "words") is 10 for the row above)
                    % Checking that the row after that is 'Nmbr Location  Lo' confirms this
                    % If there is actual spot data available, the length
                    % of the field after 'Nmbr  Location  Lo' is 3 words
                    if(length(thisRow) == 10 && strcmp(strtrim(spotFileRows{idx3+2 + ind}), 'Nmbr  Location  Lo') == 1 && length(regexp(spotFileRows{idx3+3 + ind}, '\s+', 'split')) == 3)
                        % Reading a new thisRow:
                        thisRow = regexp(strtrim(spotFileRows{idx3+3 + ind}), '\s+', 'split'); % split by tabs & spaces

                        while(length(thisRow) == 3)

                            ind = ind + 1; % Jump ind to the next index
                            NOAAspotNumber{ind} = thisRow(1);
                            %thisRow(1)
                            NOAAspotLocation{ind} = thisRow(2);

                            % Extract Lat Value from lat/lon string (SXXWYY / NXXEYY etc.)
                            NOAAcurrentLatStr = thisRow(2);
                            NOAAcurrentLatStr = NOAAcurrentLatStr{1};
                            NOAAspotLatStr = [NOAAcurrentLatStr(1), NOAAcurrentLatStr(2), NOAAcurrentLatStr(3)]; % S25, N05 etc.
                            if(strcmp(NOAAcurrentLatStr(1), 'S') == 1) % South
                                NOAAspotLat(ind) = str2num(['-', NOAAcurrentLatStr(2), NOAAcurrentLatStr(3)]);
                            elseif(strcmp(NOAAcurrentLatStr(1), 'N') == 1) % North
                                NOAAspotLat(ind) = str2num([NOAAcurrentLatStr(2), NOAAcurrentLatStr(3)]);
                            end

                            % TESTING:
                            % Extract Lon value from lat/lon string:
                            NOAAcurrentLonStr = thisRow(2);
                            NOAAcurrentLonStr = NOAAcurrentLonStr{1};
                            NOAAspotLonStr = [NOAAcurrentLonStr(4), NOAAcurrentLonStr(5), NOAAcurrentLonStr(6)]; % W25, E05 etc.
                            if(strcmp(NOAAcurrentLonStr(4), 'E') == 1) % East
                                NOAAspotLon(ind) = str2num(['-', NOAAcurrentLonStr(5), NOAAcurrentLonStr(6)]);
                            elseif(strcmp(NOAAcurrentLonStr(4), 'W') == 1) % West
                                NOAAspotLon(ind) = str2num([NOAAcurrentLonStr(5), NOAAcurrentLonStr(6)]);
                            end   
                            % / TESTING                                    

                            NOAAlong_temp = thisRow(3);
                            NOAAspotAbsLong(ind) = str2num(NOAAlong_temp{1});
                            thisRow = regexp(spotFileRows{idx3+3 + ind}, '\s+', 'split'); % split by tabs & spaces

                        end % While loop that goes through rows with 3 "words"

                        break;

                    end % If clause for recognizing the section starting with this kind of a row: "IA. H-alpha Plages without Spots.  Locations Valid at 19/2400Z Jun". The second section in the file that has info on active areas

                end % While loop that goes through rows with 8 "words". The first section of the file containing info on active areas


            else
                %disp('asdf');
                % No active areas in the file, or the SRS files aren't as stardardised
                % as has been hoped...
            end
        end
    else % There is no "spotFileRows" variable --> write output variables as empty
        NOAAspotNumber = [];
        NOAAspotLat = [];
        NOAAspotLon = [];
        NOAAspotAbsLong = [];
    end
else
    NOAAspotNumber = [];
    NOAAspotLat = [];
    NOAAspotLon = [];
    NOAAspotAbsLong = [];
end

end