%% sun_data() function, for finding the rot angle, B0, L0 and other sun data

% Input: Date in the form specified a few rows below

% Output: a struct that contains:
% rot angle (sunData.rot), decimal degrees, scalar
% B0 angle (sunData.B0), decimal degrees, scalar
% L0 value (sunData.L0), decimal degrees, scalar
% solar radius (sunData.radius), decimal arc minutes, scalar
% declination (sunData.dec), decimal degrees, scalar
%
% ...of the sun for the give time instant (linearly interpolated values)
%
% Still unread in the almanac files: Right Ascension. If needed that too
% can be read, processed and added in the output

% Data is read from .txt files that are located in the /aux_data/ sub
% folder within the current folder of Sunmap.
% The data files are courtesy of Miika Oksman (maturun@kurp.hut.fi).
% They have been modified by Niko Lavonen (nlavonen@kurp.hut.fi) so that the months
% are marked as numbers instead of strings.
% As of 2011-10-19, there are sun data files available from 1980 to 2015.

% About the accuracy of the values:
% The interpolated data seems to be quite consistent with values given by
% JPL's (NASA) HORIZONS System (http://ssd.jpl.nasa.gov/horizons.cgi).
% E.g. the linearly interpolated L0 values calculated by sun_data() differ from one minute
% resolution HORIZONS data by about 0.03...0.04 degrees. This corresponds
% to a time difference (lag was observed with a small sample) of a few minutes
% by sun_data() data compared to JPL's data. This accuracy should be
% sufficient compared to Sunmap's purpose of use. Some amount of the error
% in the values might originate from the ephemeris files used by
% sun_data(). An error of about -0.02 degrees in the L0 values compared to the
% astronomical almanac is reported to have been observed by Miika Oksman in
% the data files.

% This function is a part of Sunmap
% Sunmap is a Matlab program developed by Niko Lavonen, Metsähovi Radio observatory (nlavonen@cc.hut.fi).

function sunData = sun_data(date) % [rot B0 L0]

% date is in following form:
% '2009-09-09,  5:03:09.471 - 05:11:33.47'
% (string) (SMART)
% OR:
% '2011-02-17, 09:53:32'
% or
% '2011-02-17, 09:53:32 - 10:02:18'
% (string) (sun-measure)

    sunData.rot = -300;
    sunData.B0 = -999;
    sunData.L0 = -999;
    sunData.radius = -999;

    year1 = str2num([date(1), date(2), date(3), date(4)]);
    month1 = str2num([date(6), date(7)]);
    day1 = str2num([date(9), date(10)]);
    
    time1 = regexp(date,':','split'); % '2009-09-09,  5'    '03'    '09.471 - 05'    '11'    '33.47'
    hour1 = time1{1}; % '2009-09-09,  5'
    hour1 = str2num([hour1(end - 1), hour1(end)]); % hour as number
    minute1 = time1{2}; % '03'
    minute1 = str2num([minute1(end - 1), minute1(end)]); % minute as number
    second1 = time1{3}; % ''09.471 - 05''
    second1 = str2num([second1(1), second1(2)]); % second as number
    
    % Convert time to decimal days:
    decimal_time = (hour1 / 24) + (minute1 / 1440) + (second1 / 86400);
        
    % Insert here the path in which the list-file is located
    % The name of the file is assumed to be 'YYYY.txt', e.g "2011.txt"
    % You can of course use a custom filename and path too:
    % Write the path into 'filename'. e.g filename = '/home/example/file.txt'
    if (ispc == 1)
    
        filename = [pwd, '\aux_data\', num2str(year1), '.txt'];
        if (exist(filename, 'file') == 0)
            filename = ['C:\Users\Niko\Documents\Metsähovi\aurinkodata\position_data_2009-2015\', num2str(year1), '.txt'];
        end

    else
     
        filename = [pwd, '/aux_data/', num2str(year1), '.txt'];    
        if (exist(filename, 'file') == 0)    
            filename = ['/home/nlavonen/aurinkodata/position_data_2009-2015/', num2str(year1), '.txt'];
        end
    
    end
    
    % Getting the values of the following day for interpolation:
    % Special case:
    % It's the last day of the year and the next day is in the next year's file
    % Otherwise the next day's values are read in the for-loop ahead
    if(month1 == 12 && day1 == 31)
        
        if (ispc == 1)
            filename2 = [pwd, '\aux_data\', num2str(year1 + 1), '.txt'];
            if (exist(filename2, 'file') == 0)
                filename2 = ['C:\Users\Niko\Documents\Metsähovi\aurinkodata\position_data_2009-2015\', num2str(year1 + 1), '.txt'];
            end
        else
            filename2 = [pwd, '/aux_data/', num2str(year1 + 1), '.txt'];    
            if (exist(filename2, 'file') == 0)    
                filename2 = ['/home/nlavonen/aurinkodata/position_data_2009-2015/', num2str(year1 + 1), '.txt'];
            end
        end
    
        [dump1, year2, month2, day2, dump2, dump3, dump4, dump5, dump6, dump7, dec_sign2, dec_deg2, dec_min2, dec_sec2, dist2, rotation2, B02, L02, radius_min2, radius_sec2] = textread(filename2, '%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s');
        rot_tomorrow = str2num(rotation2{1});
        B0_tomorrow = str2num(B02{1});
        L0_tomorrow = str2num(L02{1});
        radius_min_tomorrow = str2num(radius_min2{1}); % Radius, arc min part
        radius_sec_tomorrow = str2num(radius_sec2{1}); % Radius, arc sec part
        radius_tomorrow = radius_min_tomorrow + (radius_sec_tomorrow / 60); % Radius in decimal arc min
        dec_deg_tomorrow = str2num(dec_deg2{1}); % Declination, degree part
        dec_min_tomorrow = str2num(dec_min2{1}); % Declination, arc min part
        dec_sec_tomorrow = str2num(dec_sec2{1}); % Declination, arc sec part
        dec_tomorrow = dec_deg_tomorrow + (dec_min_tomorrow / 60)  + (dec_sec_tomorrow / 3600);
        dec_sign_tomorrow = [dec_sign2{1}, '1']; % '+' or '-', --> convert into number +1 or -1 --> multiply with dec_tomorrow
        dec_tomorrow = str2double(dec_sign_tomorrow) * dec_tomorrow; % E.g. -1 * 23.6784 --> Declination in decimal degrees (with sign)
        dist_tomorrow = str2double(dist2{1});
        
    end
    
    
    if (exist(filename, 'file') ~= 0)
    
        [dump1, year, month, day, dump2, dump3, dump4, dump5, dump6, dump7, dec_sign, dec_deg, dec_min, dec_sec, dist, rotation, B0, L0, radius_min, radius_sec] = textread(filename, '%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s%s');

        for i = 1:365 % This won't work for leap years! In addition, the data files miss Dec 31 on leap years.

            if (year1 == str2num(year{i})) && (month1 == str2num(month{i})) && (day1 == str2num(day{i}))
                rot_today = str2num(rotation{i}); % The UTC = 00:00 rot angle for the map's date
                B0_today = str2num(B0{i});
                L0_today = str2num(L0{i});
                radius_min_today = str2num(radius_min{i}); % Radius, arc min part
                radius_sec_today = str2num(radius_sec{i}); % Radius, arc sec part
                radius_today = radius_min_today + (radius_sec_today / 60); % Radius in decimal arc min
                dec_deg_today = str2num(dec_deg{i}); % Declination, degree part
                dec_min_today = str2num(dec_min{i}); % Declination, arc min part
                dec_sec_today = str2num(dec_sec{i}); % Declination, arc sec part
                dec_today = dec_deg_today + (dec_min_today / 60)  + (dec_sec_today / 3600);
                dec_sign_today = [dec_sign{i}, '1']; % '+' or '-', --> convert into number +1 or -1 --> multiply with dec_tomorrow
                dec_today = str2double(dec_sign_today) * dec_today; % E.g. -1 * 23.6784 --> Declination in decimal degrees (with sign)
                dist_today = str2double(dist{i});
                
                if(month1 ~= 12 || day1 ~= 31)
                    % Getting the values of the following day for interpolation:
                    rot_tomorrow = str2num(rotation{i + 1});
                    B0_tomorrow = str2num(B0{i + 1});
                    L0_tomorrow = str2num(L0{i + 1});
                    radius_min_tomorrow = str2num(radius_min{i + 1}); % Radius, arc min part
                    radius_sec_tomorrow = str2num(radius_sec{i + 1}); % Radius, arc sec part
                    radius_tomorrow = radius_min_tomorrow + (radius_sec_tomorrow / 60); % Radius in arc min
                    dec_deg_tomorrow = str2num(dec_deg{i + 1}); % Declination, degree part
                    dec_min_tomorrow = str2num(dec_min{i + 1}); % Declination, arc min part
                    dec_sec_tomorrow = str2num(dec_sec{i + 1}); % Declination, arc sec part
                    dec_tomorrow = dec_deg_tomorrow + (dec_min_tomorrow / 60)  + (dec_sec_tomorrow / 3600);
                    dec_sign_tomorrow = [dec_sign{i + 1}, '1']; % '+' or '-', --> convert into number +1 or -1 --> multiply with dec_tomorrow
                    dec_tomorrow = str2double(dec_sign_tomorrow) * dec_tomorrow; % E.g. -1 * 23.6784 --> Declination in decimal degrees (with sign)
                    dist_tomorrow = str2double(dist{i + 1});
                    
                end
                break;
            end

        end

        % Calculate the interpolated values:
        if(rot_today < rot_tomorrow)
            rot = rot_today + (decimal_time * abs(rot_today - rot_tomorrow));
        else
            rot = rot_today - (decimal_time * abs(rot_today - rot_tomorrow));
        end

        if(B0_today < B0_tomorrow)
            B0 = B0_today + (decimal_time * abs(B0_today - B0_tomorrow));
        else
            B0 = B0_today - (decimal_time * abs(B0_today - B0_tomorrow));
        end
        
        if(dec_today < dec_tomorrow)
            dec = dec_today + (decimal_time * abs(dec_today - dec_tomorrow));
        else
            dec = dec_today - (decimal_time * abs(dec_today - dec_tomorrow));
        end

        % L0 goes from 0 to 359 and starts over again
        if(L0_today < L0_tomorrow) % Meridian transit: small number (0 < x < 30) -> large number (330 < x < 360) (numbers made up just for example)
                                                     % i.e. zero is between the two numbers
            delta = L0_today - (L0_tomorrow - 360);
            L0 = L0_today - (decimal_time * delta);
            if(L0 <= 0)
                L0 = L0 + 360;
            end
        else
            L0 = L0_today - (decimal_time * abs(L0_today - L0_tomorrow));
        end
    
        if(radius_today < radius_tomorrow)
            radius = radius_today + (decimal_time * abs(radius_today - radius_tomorrow));
        else
            radius = radius_today - (decimal_time * abs(radius_today - radius_tomorrow));
        end
        
        if(dist_today < dist_tomorrow)
            dist = dist_today + (decimal_time * abs(dist_today - dist_tomorrow));
        else
            dist = dist_today - (decimal_time * abs(dist_today - dist_tomorrow));
        end
        
    end

    % Write variables into result struct:
    sunData.rot = rot;
    sunData.B0 = B0;
    sunData.L0 = L0;
    sunData.radius = radius;
    sunData.dec = dec;
    sunData.dist = dist;

end % END OF sun_data()


