% Function track_log_writer()
% Writes solar track meta data into a text log file

% Input:
% start_time: String, The local time when the "Add" button is pressed in Sunmap's command manager. E.g. "2012-05-02, 05:20:35"
% duration: Scalar, The user defined duration of the tracking observation in hours. E.g. 8.6
% track_type: String, The type of tracking observation made. I.e. partial maps from sky & sun or series of full disk maps, order of partial maps e.g. "sun-sky" of "sun-sun-sky" etc
% NOAA_AR: String, The ID's of the NOAA Active Regions tracked, e.g. "AR11469, AR11471"
% observer: String, the user name or initials of the observer
% lb_string: Cell array, the contents of the listbox in sun_command.fig that contain the sun-measure commands to be fed into the list on daqqer
% weather_comments: String, Information on weather and free form comments

function track_log_writer(start_time, duration, track_type, NOAA_AR, observer, lb_string, weather_comments)

% Define track log file name:
track_log_file_name = '/home/kurpitsa/sun/solar_tracks_log.txt';

% Open the track log file
track_log = fopen(track_log_file_name, 'at'); % Open file, "at" for appending

% If track_file can't be opened, error dialog and return:
if(track_log == 0)
    message = ['Can''t open solar track log file (', trackFileName, ')'];
    errordlg(message, 'Error', 'modal');
    return;
end

% Write file header line if file is empty:
s = dir(track_log_file_name); % s.bytes tells the number of bytes in the data file
if(s.bytes < 5)  % under 5 bytes --> file empty --> write header line into file:
    fprintf(track_log, [sprintf('%-30s','Start time (local time)') sprintf('%-25s', 'Duration (hours)') sprintf('%-25s','Track type') sprintf('%-25s','Tracked NOAA AR''s') sprintf('%-20s', 'Observer') sprintf('%-40s', 'Weather / comments') sprintf('%-80s', 'Coords. arcmin from intended solar center [startra, endra, startdec, enddec]') '\n']);
end

% Extract and calculate startra, endra, startdec, enddec from lb_string contents and
% write into variables:

% Go through lisbox contents cell array:
for i = 1 : length(lb_string)
    % Split single listbox row into separate strings:
    lb_parse = strsplit(lb_string{i});
    % Go through the separate strings:
    for j = 1 : length(lb_parse)
        % If string with index j matches a parameter name, then
        % the string with the index j+1 is the corresponding parameter value:
        if(strcmp(lb_parse(j), '--ra-size') == 1)
            ra_size{i} = char(lb_parse(j + 1));
        elseif(strcmp(lb_parse(j), '--dec-size') == 1)
            dec_size{i} = char(lb_parse(j + 1));
        elseif(strcmp(lb_parse(j), '--ra-off') == 1)
            ra_off{i} = char(lb_parse(j + 1));
        elseif(strcmp(lb_parse(j), '--dec-off') == 1)
            dec_off{i} = char(lb_parse(j + 1));
        end
    end
    
    % Calculate startra, endra, startdec, enddec from ra_size, dec_size,
    % ra_off, dec_off:
    startra(i) = str2num(ra_off{i}) - 0.5 * str2num(ra_size{i});
    endra(i) = str2num(ra_off{i}) + 0.5 * str2num(ra_size{i});
    startdec(i) = str2num(dec_off{i}) - 0.5 * str2num(dec_size{i});
    enddec(i) = str2num(dec_off{i}) + 0.5 * str2num(dec_size{i});
    
end

% Create the printing string for coordinates:
coords = '';
for i = 1 : length(startra)
    if(i == 1)
        coords = [coords, '[', num2str(startra(i)), ', ', num2str(endra(i)), ', ', num2str(startdec(i)), ', ', num2str(enddec(i)), ']'];
    else
        coords = [coords, ' [', num2str(startra(i)), ', ', num2str(endra(i)), ', ', num2str(startdec(i)), ', ', num2str(enddec(i)), ']'];
    end
end

% Write file contents:
fprintf(track_log, [sprintf('%-30s', start_time) sprintf('%-25s', num2str(duration)) sprintf('%-25s', track_type) sprintf('%-25s', NOAA_AR) sprintf('%-20s', observer) sprintf('%-40s', weather_comments) sprintf('%-80s', coords)  '\n']);

% Close the file
fclose(track_log);

end